/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;

import java.util.ArrayList;
import java.util.List;
import javax.swing.JOptionPane;

/**
 * Abstraktní třída obecného aparátu.<br>
 * Umožňuje přidat a ubrat proudy aparátu, kontroluje hmotovou bilanci.<br>
 * Zavádí univerzální metodu výpočtu aparátu.<br>
 * Od této třídy se odvozují všechny konkrétní aparáty, jako SmesovacIdeal, CSTRReaktor<br>
 * Komprese, Trubreaktor, BatchReaktor a další..
 * @author V. Hlubuček
 */
public abstract class Aparat {

    public final double TABS = 273.15;
    protected String VERZE = "V1.4";
    protected int cApar;		// číslo aparátu
    protected int pocetInProudu;
    protected int pocetOutProudu;
    protected int[] navaznost = new int[4]; 
    protected String nazevAparatu;	// název aparátu
    protected boolean hlidatMimobezProudy = true;  // hlídat proudy, které neprochází aparátem?
    protected boolean zobrazujVysledky; // příznak zobrazování výsledků
    protected boolean zadan;            // aparát byl zadan a je schopen vypoctu     
    protected boolean spocten;          // aparát byl spočten  
    protected String[] lTexty;          // pomocná pole pro interaktivní zadávání
    protected String[] tValue;
    
    /**
     * Textové výsledky z aparatu
     */
    protected String report;    
    /**
     * Seznam všech proudů daného aparátu
     */
    protected List<Proud>  proudy = new ArrayList();	// kolekce proudů
    /**
     * Seznam vstupních proudů daného aparátu
     */    
    protected List<Proud>  proudyIn = new ArrayList();	// kolekce vstupních proudů
    /**
     * Seznam výstupních proudů daného aparátu
     */     
    protected List<Proud>  proudyOut = new ArrayList();	// kolekce výstupních proudů
    public void pridejProud(Proud pr) {  proudy.add(pr); }
    public void odeberProud(Proud pr) {  proudy.remove(pr); }    
    public void pridejProudIn(Proud pr) {  proudyIn.add(pr); }
    public void odeberProudIn(Proud pr) {  proudyIn.remove(pr); }    
    public void pridejProudOut(Proud pr) {  proudyOut.add(pr); }
    public void odeberProudOut(Proud pr) {  proudyOut.remove(pr); }
/**
 * Bezparametrový konstruktor Aparat
 */
    public Aparat() {
      nazevAparatu = "Aparát ";  
    }
/**
 * Konstruktor abstraktní třídy Aparat
 * @param cApar číslo aparátu
 * @param nazevAparatu textový název aparátu
 */    
    public Aparat(int cApar, String nazevAparatu) {
        this.cApar = cApar;
        this.nazevAparatu = nazevAparatu;
    }
/**
 * Abstraktní univerzální metoda provádějící výpočet aparátu
 * tisk = true : se zobrazením výsledků; false  bez zobrazení výsledků
 * varianta bez parametru zobrazuje podle nastavení zobrazujVysledky
 */    
    abstract public void spoctiAparat();
    abstract public void spoctiAparat(boolean tisk);
    abstract public boolean jeAparatZadan();
    abstract public void dosadInteraktivniHodnoty();    
    
/**
 * Interaktivní zadání parametrů elementu
 */    
    public void zadejParametryInteraktivne()  {
        JDialogAparatInp jdei = new JDialogAparatInp(null, true, lTexty, tValue); // interaktivní vstup dat elementu
        jdei.setVisible(true); 
        dosadInteraktivniHodnoty();        

    }    
    
/**
 * Metoda vrací pole výstupních proudů aparátu
 * @return pole výstupních proudů
 */
    public Proud[] vystupniProudy() {
        int i = 0;
        Proud[] vystPr = new Proud[6];
        for(Proud pr : proudy) {
            if(jeProudVstupni(pr) == -1) {
                vystPr[i] = pr;
                i++;
            }
        }
        return vystPr;
    }
/**
 * Pomocná funkce převádí string na hodnotu double
 * @param tx
 * @return 
 */    
    protected double doubleFromString(String tx)  {
        return (tx.equals("") || tx.equals("?") || tx.equals("*")) ? 0:Double.parseDouble(tx.replace(',', '.'));
    }
    
    /**
     * Kontrola hmotové bilance
     * @return 
     */
    public boolean bilanceHmoty()  {	// kontrola hmotové bilance
       boolean bilHmot;
       double sum = 0;
       bilHmot = false;
       for (Proud pr :  proudy)  {
           if (pr.getDoUzlu() == this)  {
             sum += pr.getHmotPrutok();	// vstupní proudy
           } else  {
             sum -= pr.getHmotPrutok();	// výstupní proudy
           }
       }
       if (Math.abs(sum) < 1e-6) {
           bilHmot = true;
       } else {
           JOptionPane.showMessageDialog(null,
              "Proudy aparátu " + nazevAparatu + " č. (" + cApar + ") nesplňují hmotovou bilanci !",
              "Chyba hmotové bilance", JOptionPane.WARNING_MESSAGE);
       }
       return bilHmot;
    }
/**
 * Testuje, zda je proud vstupní, výstupní nebo recykl
 * případně jestli neprochází aparátem
 * @param prd zkoumaný Proud
 * @return  -1 výstupní; 0 recykl; 1 vstupní; 2 není označen
 *   -9999 proud nenalezen (neprochází aparátem)
 */    
    public int jeProudVstupni(Proud prd) {  
        int j = 0, k = 0, n = 2;
        boolean nenalezen = true;
        for (Proud pr :  proudy)  { 
            if (prd == pr) {
                if (pr.getDoUzlu()  == this) { j = 1; nenalezen = false;}	
                if (pr.getzUzlu() == this)   {k = -1; nenalezen = false;}
                if (pr.getDoUzlu()  == null && pr.getzUzlu()  == null) j = 2;    // proud neoznačen
                break;
            }
        }
        if (j == 1 && k == 0) n = j;
        if (j == 0 && k == -1) n = k;
        if (j == 1 && k == -1) n = 0;
        if(j == 2) n = 2;
        if (nenalezen && j != 2) {
            n = -9999;
            if(hlidatMimobezProudy) JOptionPane.showMessageDialog(null,
                "Aparátem č. " + cApar + "  " + nazevAparatu 
                 + "\nneprochází proud č. " + prd.cProudu,
                 "Aparát - proudy", JOptionPane.WARNING_MESSAGE);             
        }
        return n;
    }
    
    public boolean lzeSpocitatAparat() {
            boolean nelze = false;
            if(this == null) return false; 
            if(!jeAparatZadan()) nelze = true;
            for(Proud pr : proudy) if(!pr.jeProudZadan()) nelze = true;
            return (!nelze);
    }
/**
 * Zobrazí již spočtený výstupní proud/y aparátu
 */
    public void zobrazVystupniProud() {
        int  n = 0;
        Proud[] prVyst = new Proud[4];
        for(Proud pr : proudy) {
            if(jeProudVstupni(pr)== -1 && pr.spocten) {
                prVyst[n] = pr; n++;
            }
        }
        for(int i = 0; i < n; i++) {
            new JDialogProud(null, nazevAparatu + "  (Verze " 
                        + getVERZE() + ")\n", prVyst[i]); // modálně vyskakovací okno s výsledky              
        }      
    }
    
/**
 * Pomocná metoda, řeší nezadání do seznamu proudy
 */        
    public void usporadatProudy() {
        if(proudy.isEmpty() && !proudyIn.isEmpty()) {
            for(Proud pr : proudyIn) proudy.add(pr.kopirujProud(pr));
        }
    }     
    
// gettery a settery  ----------------------------------------------    
/**
 * Vrací verzi třídy aparátu
 * @return String
 */
    public String getVERZE() {
        return VERZE;
    }
/**
 * Vrací číslo aparátu
 * @return číslo
 */
    public int getcApar() {
        return cApar;
    }
/**
 * Nastavuje číslo teplosměnného výměníku
 * @param cApar 
 */
    public void setcApar(int cApar) {
        this.cApar = cApar;
    }
/**
 * Vrací název aparátu - výměníku
 * @return název
 */
    public String getNazevAparatu() {
        return nazevAparatu;
    }
/**
 * Nastavuje název aparátu - výměníku
 * @param nazevAparatu 
 */
    public void setNazevAparatu(String nazevAparatu) {
        this.nazevAparatu = nazevAparatu;
    }   
    
/**
 * Vrací zda je nastaveno hlídání mimoběžných proudů
 * @return boolean
 */
    public boolean isHlidatMimobezProudy() {
        return hlidatMimobezProudy;
    }
/**
 * Nastavuje zda hlídat mimoběžné proudy
 * @param hlidatMimobezProudy 
 */
    public void setHlidatMimobezProudy(boolean hlidatMimobezProudy) {
        this.hlidatMimobezProudy = hlidatMimobezProudy;
    }
/**
 * Vrací stav zobrazování výsledků
 * @return 
 */
    public boolean isZobrazujVysledky() {
        return zobrazujVysledky;
    }
/**
 * Nastavuje zobrazování výsledků
 * @param zobrazujVysledky 
 */
    public void setZobrazujVysledky(boolean zobrazujVysledky) {
        this.zobrazujVysledky = zobrazujVysledky;
    }
/**
 * Vrací výsledkový report aparátu
 * @return 
 */
    public String getReport() {
        return report;
    }
/**
 * Vrací kolekci vstupních proudů
 * @return 
 */
    public List<Proud> getProudyIn() {
        return proudyIn;
    }
/**
 * Vrací kolekci výstupních proudů
 * @return 
 */
    public List<Proud> getProudyOut() {
        return proudyOut;
    }
    
/**
 * Vrací, zda je aparát schopen výpočtu
 * @return 
 */
    public boolean isZadan() {
        return zadan;
    }
/**
 * Nastavuje, zda je aparát schopen výpočtu
 * @param zadan 
 */
    public void setZadan(boolean zadan) {
        this.zadan = zadan;
    }
    
/**
 * Vrací, zda byl aparát spočten
 * @return 
 */
    public boolean isSpocten() {
        return spocten;
    }
/**
 * Nastavuje, zda byl aparát spočten
 * @param spocten 
 */
    public void setSpocten(boolean spocten) {
        this.spocten = spocten;
    }
/**
 * Vrací pole návazností na aparáty
 * @return 
 */
    public int[] getNavaznost() {
        return navaznost;
    }
/**
 * Nastavuje pole návazností na aparáty
 * @param navaznost 
 */
    public void setNavaznost(int[] navaznost) {
        this.navaznost = navaznost;
    }
/**
 * Vrací i-tý prvek návaznosti aparátu
 * @param i
 * @return 
 */    
    public int getNavaznost(int i) {
        return navaznost[i];
    }
/**
 * Nastavuje i-tý prvek návazznosti
 * @param i index prvku návaznosti
 * @param n číslo navazujícího aparátu
 */    
    public void setNavaznost(int i, int n) {
        navaznost[i] = n;
    }
}   
