/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package cheminzlib;

import fydat.I_Fmin;
import fydat.Numer;
import fydat.NumerException;
import fydatlib.Smes;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;

/**
 * Třída aparátu ideální flash -  rozdělení fází při expanzi proudu.
 * počítá také jen kompresi či expanzi nebo škrcení
 * Předpokládá ideální chování směsi.
 * @author V. Hlubuček
 */
public class FlashIdeal extends Aparat implements I_Fmin {

    protected String VERZE = "1.2";
    protected int nSloz;
    protected boolean adiab;          // je flash adiabatický?
    protected double tFlash, pFlash;  // koncová teplota (K) a tlak (Pa)
    protected double qIzo;            // teplo pro izotermní flash (J/mol)
    protected double vfPoc = 0, vf = 0; // poměry parní fáze k nástřiku
    protected double[] xPoc;          // počáteční složení kapalné fáze
    protected double[] yPoc;          // počáteční složení parní fáze
    protected double[] xKon;          // koncové složení kapalné fáze
    protected double[] yKon;          // koncové složení parní fáze    
    protected Smes sm;                // vstupní směs
    protected Proud pr;               // vstupní proud
    // pomocné interní proměnné
    private int typ;                  // 1-flash; 2-komprese; 3-škrcení
    private boolean pocatek;
    private double  tPom, pPom, hPom;
    
    
    
 /**
 * Konstruktor ideálního flash
     * @param pr  vstupní proud
     * @param adiab  boolean  jde o adiabatický flash?
     * @param tFlash   počáteční teplota (°C) 
     * @param pFlash   flashovací tlak (Pa)
 */    
    public FlashIdeal(Proud pr, boolean adiab, double tFlash, double pFlash) {
        super();
        pocetInProudu = 1;
        pocetOutProudu = 2;        
        nazevAparatu = "Ideální flash ";
        this.pr = pr;
        this.adiab = adiab;
        this.tFlash = tFlash;
        this.pFlash = pFlash;
        proudy.add(pr);
        proudyIn.add(pr);
        proudyOut.add(pr.kopirujProud(pr)); // první fiktivní výstupní pára
        proudyOut.add(pr.kopirujProud(pr)); // druhý fiktivní výstupní kapalina
        myInit();        
    }
    
 /**
 * Bezparametrový konstruktor ideálního flash
 */    
    public FlashIdeal() {
        super();
        pocetInProudu = 1;
        pocetOutProudu = 2;         
        nazevAparatu = "Ideální flash ";
        myInit();        
    }
    
/**
 * Inicializace aparátu
 * popis labelů a default hodnot pro interaktivní zadávání
 */
    public void myInit() {
        lTexty = new String[10];
        tValue = new String[10];
        lTexty[0] = "Flash Ideal";
        lTexty[1] = "Č.aparátu";
        lTexty[2] = "Název aparátu";
        lTexty[3] = "Počet In proudů";
        lTexty[4] = "Počet Out proudů";
        lTexty[5] = "Flash na teplotu (°C)";
        lTexty[6] = "Flash na tlak (Pa)";
        lTexty[7] = "Poměr parní fáze vc nástřiku (V/F)";
        lTexty[8] = "Režim (0-adiab. 1-izotermní)";
        lTexty[9] = ""; // musí být jako ukončovač
        if(pFlash > 0) {
            tValue[1] = String.format("%3d",cApar);
            tValue[2] = nazevAparatu;
            tValue[3] = String.format("%2d",pocetInProudu);
            tValue[4] = String.format("%2d",pocetOutProudu);
            tValue[5] = String.format("%5.1f",tFlash);
            tValue[6] = String.format("%7.0f",pFlash);
            tValue[7] = String.format("%5.3f",vfPoc);
            tValue[8] = (adiab == true)? "0": "1";
        } else {
            tValue[1] = "0";
            tValue[2] = "Flash Ideal";
            tValue[3] = "1";
            tValue[4] = "1";
            tValue[5] = "100";
            tValue[6] = "101320";
            tValue[7] = "0";
            tValue[8] = "0";            
        }             
    }
/**
 * Dosadí do proměnných hodnoty z interaktivního zadání
 */    
    public void dosadInteraktivniHodnoty() {
        cApar = (int)doubleFromString(tValue[1]);
        nazevAparatu = tValue[2];
        pocetInProudu = (int)doubleFromString(tValue[3]);
        pocetOutProudu = (int)doubleFromString(tValue[4]);
        tFlash = doubleFromString(tValue[5]);
        pFlash = doubleFromString(tValue[6]);
        vfPoc = doubleFromString(tValue[7]);
        if((int)doubleFromString(tValue[8]) == 0) adiab = true; else adiab = false;
    }    

/**
 * Ověřuje zadání aparátu
 * @return 
 */
    @Override
    public boolean jeAparatZadan() {
        boolean neni = false;
        if(this == null) return false;
        return (!neni);
    }
    
    /**
     * Výpočet ideálního flash (adiab. nebo izotermní), výsledky do textové proměnné
     * @return String s výsledky
     */
    public String spoctiFlash() {
        int i;
        double tFpoc, pFpoc, pFkon = 0, fmin = -9999, tbv0, trb0;
        double  pom, tFkon = 0, hl = 0, hv = 0;
        typ = 1;        // flash
        sm = pr.getSmes();
        nSloz = sm.getnSloz();
        double[] x = new double[nSloz]; 
        sm.setNeMistniHlasky(true);
        xPoc = new double[nSloz];
        yPoc = new double[nSloz];
        xKon = new double[nSloz];
        yKon = new double[nSloz];
//        tFpoc = Double.parseDouble((jTextFieldFTpoc.getText()).replace(',', '.'));  // pro českou notaci
        tFpoc = pr.tVstup;       // počáteční abs. teplota (K)
        pFpoc = pr.pVstup;              // tlak před flashem (Pa)
        pFkon = pFlash;
        // nalezení výchozího stavu
        tPom = tFpoc + TABS ;   
        pPom = pFpoc;
        tbv0 = sm.sTbv(pFpoc) - TABS;   // počáteční bod varu
        trb0 = sm.sTrb(pFpoc) - TABS;   // počáteční rosný bod
//        if(!adiab && tFpoc < tbv0)JOptionPane.showMessageDialog(null, "Flash - směs zůstane jen kapalnou!", "FlashIdeal", JOptionPane.INFORMATION_MESSAGE);
        x[0] = 0.3;   // odhad V/F
        if(tFpoc > tbv0 && tFpoc < trb0) x[0] = (tFpoc - tbv0)/(trb0 - tbv0);
        x[1] = 0;     // dummy
//        I_Fmin fi = this;  // není třeba
        try {
            // Výchozí stav
            pocatek = true;
            fmin = Numer.m_Fmin1(this, 2, x, 1e-4); // volání minimalizace
            vfPoc = x[0];
            if (vfPoc < 0) {
                vfPoc = 0;      // jen kapalina
            }
            if (vfPoc > 1) {
                vfPoc = 1;      // jen pára
            }
            for (i = 0; i < nSloz; i++) {
                pom = sm.getSlozka(i).tenze(tPom) / pPom;
                x[i] = sm.getxMol(i);
                xPoc[i] = x[i] / (1.0 + vfPoc * (pom - 1.0));
                yPoc[i] = xPoc[i]*pom;  // dříve x[i] / VFpoc - xPoc[i] * (1 / VFpoc - 1.0);  
            }
            normalizuj(nSloz, xPoc);
            normalizuj(nSloz, yPoc);
            // počáteční entalpie
            try {
                sm.zadatSloz(false, yPoc);            // mol.sl. parní fáze
                hv = sm.srHG(tPom, pPom);    // počáteční entalpie par
                if (Double.isNaN(hv)) {
                    hv = 0;
                }
                sm.zadatSloz(false, xPoc);
                hl = sm.sHL(tPom);           // počáteční entalpie kapaliny
                if (Double.isNaN(hl)) {
                    hl = 0;
                }
                sm.zadatSloz(false, x);            // vrátit mol. složení nástřiku
                hPom = vfPoc * hv + (1.0 - vfPoc) * hl; // celková entalpie počátečního stavu
            } catch (IOException e) {
                e.getStackTrace();
            }
            // Koncový stav
            // izotermní i adiab. flash
            tPom = tFpoc + TABS;   
            pPom = pFkon;   // koncový tlak
            x[0] = 0.3;     // odhad V/F
            x[1] = (adiab) ? tFpoc : 0;    // T nebo dummy 
            pocatek = false;        // výpočet koncového stavu
            fmin = Numer.m_Fmin1(this, 2, x, 1e-5); // volání minimalizace
//                System.out.println("* V/F,T, fmin : "+ x[0]+"  "+x[1]+" * "+fmin);
            vf = x[0];
            tFkon = x[1];       // koncová teplota adiab. flashe
            tFlash = tFkon;
            if (adiab) {
                tPom = x[1] + TABS;
            }
            if (vf < 0) {
                vf = 0;     // jen kapalina
            }
            if (vf > 1) {
                vf = 1;     // jen pára
            }
            for (i = 0; i < sm.getnSloz(); i++) {
                pom = sm.getSlozka(i).tenze(tPom) / pPom;   // Ki = P0i/P
                xKon[i] = sm.getxMol(i)/ (1.0 + vf * (pom - 1.0));  //složení kapalné fáze
                yKon[i] = xKon[i] * pom;  // dřívesm.getxMol(i) / vf - xKon[i] * (1 / vf - 1.0);
                if (vf > 0.9999) {
                    xKon[i] = 0.0;              // žádná kapalina
                    yKon[i] = sm.getxMol(i);    // jen pára
                }
                if (vf < 0.0001) {
                    xKon[i] = sm.getxMol(i);    // jen kapalina
                    yKon[i] = 0.0;              // žádná pára
                }
            }
            normalizuj(nSloz, xKon);
            normalizuj(nSloz, yKon);
        } catch (NumerException ex) {
            double value = ex.getValue();
            JOptionPane.showMessageDialog(null, "Překročen max. počet iterací, výsledná hodnota je:" + value);
        }
        // Zápis výsledků
        report = "\nFLASH " + ((adiab) ? " adiabatický" : " izotermní");
        report += "\nPočáteční tlak (MPa): " + String.format("%-6.4f", pFpoc / 1e6);
        report += "   Koncový tlak (MPa): " + String.format("%-6.4f", pFkon / 1e6);
        report += "\nPočáteční teplota (°C): " + String.format("%-5.1f", tFpoc)
                + "  Koncová teplota (°C): "
                + String.format("%-5.1f", (!adiab) ? tFpoc : tFkon);
        report += "\nPočáteční Bod varu a rosný bod (°C): "
                + String.format("%-5.1f", tbv0)
                + "  " + String.format("%-5.1f", trb0);        
        report += "\nKoncový   Bod varu a rosný bod (°C): "
                + String.format("%-5.1f", sm.sTbv(pFkon) - TABS)
                + "  " + String.format("%-5.1f", sm.sTrb(pFkon) - TABS);
        report += "\nPoměr parní fáze k nástřiku (poč, kon): "
                + String.format("%-6.4f", vfPoc) + String.format("  %-6.4f", vf);
        report += "\n Složka                    Pára   Kapalina Nástřik";
        for (i = 0; i < nSloz; i++) {
            report += "\n" + String.format("%-25s", sm.getSlozka(i).getNazev())
                    + "  " + String.format("%-6.4f", yKon[i])
                    + "  " + String.format("%-6.4f", xKon[i])
                    + "  " + String.format("%-6.4f", sm.getxMol(i));
        }
        report += "\n";
        if(vf <0.0001) report += "Směs zůstává jen kapalinou\n";
        if(vf > 0.999) report += "Směs zůstává jen parou\n";
        report += String.format("Fmin = %-6.3g%n", fmin);
        if(!adiab){
            x = sm.getxMol();
            try {
                sm.zadatSloz(false, xKon);
                hl = sm.sHL(tFpoc + TABS);
                sm.zadatSloz(false, yKon);
                hv = sm.srHG(tFpoc + TABS,pFkon);
                hPom = (vf*hv + (1-vf)*hl - hPom);  // potřebné teplo dodané pro izotermní flash (J)
                sm.zadatSloz(false, x);
                report += String.format("Potřebné teplo (kJ/mol, kW): %8.5g  %8.5g%n", hPom/1000, pr.hmotPrutok/sm.sMolHmot()*hPom/1000);
            } catch (IOException ex) {
                Logger.getLogger(FlashIdeal.class.getName()).log(Level.SEVERE, null, ex);
            }                        
        }            
        sm.setNeMistniHlasky(false);
        pr.pVystup = pFkon;
        pr.tVystup = tFkon;
        pr.spocten = true;
        pr.zUzlu = this;        
        if(zobrazujVysledky) {
            pr.setVysledek(report);
            new JDialogProud(null, nazevAparatu + "  vstup (FlashIdeal Verze " 
                        + VERZE + ")\n", pr); // modálně vyskakovací okno s výsledky              
        }
        return report;      // vrací výsledky textově jako String
    }

    /**
     * Cílová funkce pro Fmin  dle interface I_Fmin
     * @param x double[] minimalizační proměnné
     * @return
     */
    public double f(double[] x) {
        int i;
        double sum = 0, sumh = 0, hv = 0, hl = 0, hf = 0, pom, vf = 0;
        switch (typ) {
            case 1:         // FLASH  *************************************
                if (!adiab || pocatek) {  // izotermní flash Rachford-Rice  Suma(yi - xi) = 0  numericky výhodné
                    for (i = 0; i < nSloz; i++) {        
                        sum += sm.getxMol(i) * (sm.getSlozka(i).tenze(tPom) / pPom - 1.0)
                                / (1.0 + x[0] * (sm.getSlozka(i).tenze(tPom) / pPom - 1.0));    // sumace cílové fce
                    }
                    return sum * sum + 0.01 * (x[1] * x[1]);
                } else {                 // adiabatický flash Rachford-Rice  Suma(yi - xi) = 0  numericky výhodné
                    for (i = 0; i < nSloz; i++) {       // přes  pPom
                        sum += sm.getxMol(i) * (sm.getSlozka(i).tenze(x[1] + TABS) / pPom - 1.0)
                                / (1.0 + x[0] * (sm.getSlozka(i).tenze(x[1] + TABS) / pPom - 1.0)); // sumace cílové fce
                    }
                    vf = x[0];
                    if (x[0] < 0) {
                        vf = 0;         // jen kapalina
                    }
                    if (x[0] > 1) {
                        vf = 1;         // jen pára
                    }
                    double[] zz = new double[nSloz];    // složení nástřiku (mol. zlomky)
                    double[] xx = new double[nSloz];    // složení kapalné fáze
                    double[] yy = new double[nSloz];    // složení parní fáze
                    for (i = 0; i < nSloz; i++) {
                        zz[i] = sm.getxMol(i);      // zapamatovat mol.složení nástřiku
                        if (vf > 0.9999) {          // jen pára
                            xx[i] = 0.0;
                            yy[i] = zz[i];
                        } else if (vf < 0.0001) {   // jen kapalina
                            xx[i] = zz[i];
                            yy[i] = 0.0;
                        } else {                    // směs kapaliny a páry
                            pom = sm.getSlozka(i).tenze(x[1] + TABS) / pPom;    // Ki = P0i/pPom
                            xx[i] = sm.getxMol(i)/(1.0 + vf * (pom - 1.0));     //  korigovaně  (1.0 + x[0] *(pom - 1.0));
                            yy[i] = xx[i] * pom; // nebo též zz[i]/vf - xx[i]*(1/vf-1.0);                            
                        }
                    }
                    normalizuj(nSloz, xx);
                    normalizuj(nSloz, yy);
//                    normalizuj(nSloz, zz);          // není třeba už je
                    try {
                        sm.zadatSloz(false, yy);            // mol.sl. parní fáze
                        hv = (vf < 0.0001)? 0 : sm.srHG(x[1] + TABS, pPom);    // koncová entalpie par
                        if (Double.isNaN(hv)) {
                            hv = 0;
                        }
                        sm.zadatSloz(false, xx);
                        hl = (vf > 0.9999) ? 0: sm.sHL(x[1] + TABS);           // koncová entalpie kapaliny
                        if (Double.isNaN(hl)) {
                            hl = 0;
                        }
                        sm.zadatSloz(false, zz);            // vrátit mol. složení nástřiku
                    } catch (IOException e) {
                        e.getStackTrace();
                    }
                    sumh = vf * hv / hPom + (1.0 - vf) * hl / hPom - 1.0;   // entalpická bilance
//            System.out.println("sum: "+sum+"  V/F= "+x[0]+"  T "+x[1]+"  sumh "+sumh);
//            System.out.println("** hf,hl,hv: "+ hPom+"  "+hl+"  "+hv);
                    return sum * sum + sumh * sumh;
                }
            case 2:    // Komprese   (izentropický děj)
                    sumh = sm.srSG(x[0], pFlash) - sm.srSG(tPom, pPom);
//            System.out.println("Ksum: "+sumh+"  T= "+x[0]);
                    return sumh * sumh + (x[1] * x[1]);
            case 3:     // Škrcení  (izentalpický děj)
                    sumh = sm.srHG(x[0], pFlash) - sm.srHG(tPom, pPom);
                    return sumh * sumh + 0.01 * (x[1] * x[1]);
        }
        return 0;
    }
/**
 * Počítá flash a výsledky ukládá do pole tří proudů
 * Proud[0] je vstupní proud do flashe 
 * Proud[1] je parní proud flashe 
 * Proud[2] je kapalný proud flashe 
 * @return Proud[] výsledné proudy
 */    
    public Proud[] proudyFlash() {
        boolean pomZobraz;
        Proud[] prd = new Proud[3];
        prd[0]= pr;         // vstupní proud flashe
        pomZobraz = zobrazujVysledky;
        zobrazujVysledky = false;   // potlač duplicitní zobrazení výsledků
        spoctiFlash();      // výpočet flashe
        zobrazujVysledky = pomZobraz;
        prd[1] = new Proud(pr); // parní proud flashe
        prd[2] = new Proud(pr); // kapalný proud flashe
        {
        try {
            // parní proud
            prd[1].pVystup = pFlash;
            prd[1].tVystup = tFlash;
            prd[1].zUzlu = pr.doUzlu;
            prd[1].hmotPrutok = vf * pr.hmotPrutok;
            prd[1].sm.zadatSloz(false, yKon);
            prd[1].setVysledek(this.report);
            prd[1].nazProudu = "Parní fáze flashe";
            prd[1].spocten = true;
            prd[1].zUzlu = this;
            proudy.add(prd[1]);
            // kapalný proud
            prd[2].pVystup = pFlash;
            prd[2].tVystup = tFlash;
            prd[2].zUzlu = pr.doUzlu;
            prd[2].hmotPrutok = (1 - vf) * pr.hmotPrutok;
            prd[2].sm.zadatSloz(false, xKon);
            prd[2].setVysledek(this.report);
            prd[2].nazProudu = "Kapalná fáze flashe";
            prd[2].spocten = true;
            prd[2].zUzlu = this;
            proudy.add(prd[2]);
            proudyOut.set(0, prd[1]);
            proudyOut.set(1, prd[2]);
            
        } catch (IOException ex) {
            Logger.getLogger(FlashIdeal.class.getName()).log(Level.SEVERE, null, ex);
        }
        if(zobrazujVysledky) {  // zobrazuje okna s výsledky jednotlivých proudů
            pr.setVysledek(report);
            JDialogProud frmProud;
            frmProud = new JDialogProud(null, nazevAparatu + "  vstup (FlashIdeal Verze " 
                        + VERZE + ")\n", pr); // modálně vyskakovací 
            frmProud = new JDialogProud(null, nazevAparatu + " - kapalina  (FlashIdeal Verze " 
                        + VERZE + ")\n", prd[2]); // modálně vyskakovací
            frmProud = new JDialogProud(null, nazevAparatu + " - pára (FlashIdeal Verze " 
                        + VERZE + ")\n", prd[1]); // modálně vyskakovací             
        }        
        }
        return prd;
    }
    
/**
 * Počítá kompresi/expanzi nebo škrcení
 * @param komp boolean  (true - komprese/expanze; false - škrcení)
 * @return výsledkový string
 */
    public String spoctiKompresi(boolean komp) {
    
        double fmin = 9999, tbv0, trb0, tbv1 = -9999, trb1 = -9999;
        double[] x = new double[2];
        if(komp) typ = 2; else typ = 3;
        tPom = pr.tVstup + TABS;    // vstupní teplota (K)
        pPom = pr.pVstup;           // vstupní tlak
        sm = pr.getSmes();
        tbv0 = sm.sTbv(pPom) - TABS;
        trb0 = sm.sTrb(pPom) - TABS;
        x[0] = tPom;
        x[1] = 0;
        report = "\n" + ((typ == 2) ? ((pFlash > pPom) ? "Komprese" : "Expanze") : "Škrcení");
        report += "\nPočáteční tlak (MPa): " + String.format("%-6.4f", pPom / 1e6);
        report += "   Koncový tlak (MPa): " + String.format("%-6.4f", pFlash / 1e6); 
        if(3 == typ && pPom < pFlash) report += "U škrcení je počáteční tlak nižší, než koncový!\n";
        try {
            fmin = Numer.m_Fmin1(this, 2, x, 1e-5); // volání minimalizace
            tFlash = x[0] - TABS;
            tbv1 = sm.sTbv(pFlash) - TABS;
            trb1 = sm.sTrb(pFlash) - TABS;
        } catch (NumerException ex) {
            Logger.getLogger(FlashIdeal.class.getName()).log(Level.SEVERE, null, ex);
        }
        report += "\nPočáteční teplota (°C): " + String.format("%-5.1f", tPom - TABS)
                + "  Koncová teplota (°C): "
                + String.format("%-5.1f", tFlash);
        report += "\nPočáteční Bod varu a rosný bod (°C): "
                + String.format("%-5.1f", tbv0)
                + "  " + String.format("%-5.1f", trb0);   
        if(tPom - TABS < trb0) report += "\nPočáteční směs kondenzuje!";
        report += "\nKoncový   Bod varu a rosný bod (°C): "
                + String.format("%-5.1f", tbv1)
                + "  " + String.format("%-5.1f%n", trb1); 
        if(tFlash < trb1) report += "Koncová směs kondenzuje!\n";
        report += String.format("Fmin = %-6.3g%n", fmin);   
        pr.pVystup = pFlash;
        pr.tVystup = tFlash; 
        if(zobrazujVysledky) {
            pr.setVysledek(report);
            JDialogProud frmProud = new JDialogProud(null, "FlashIdeal vstup (Verze " 
                        + VERZE + ")\n", pr); // modálně vyskakovací okno s výsledky              
        }        
        return report;
    }    
    /**
     * Normalizuje zlomky složení
     * @param n počet prvků pole
     * @param x pole double[]
     */
    private void normalizuj(int n, double[] x) {
        int i;
        double sum = 0;
        for (i = 0; i < n; i++) {
            sum += x[i];
        }
        for (i = 0; i < n; i++) {
            x[i] = (Math.abs(sum) < 1e-8) ? 0.0 : x[i] / sum;
        }
    }
/**
 * implementace abstraktní metody
 */
    @Override
    public void spoctiAparat() {
        proudyFlash();
    }

    @Override
    public void spoctiAparat(boolean tisk) {
        boolean bylo = zobrazujVysledky;
        if(tisk)zobrazujVysledky = true; else zobrazujVysledky = false;
        proudyFlash();
        zobrazujVysledky = bylo;
    }
    
// gettery a settery  ---------------------------------------------    
/**
 * Vrací výsledek
 * @return 
 */
//    @Override
//    public String getReport() {
//        return report;
//    }
/**
 * True když je flash adiabatický
 * @return adiab
 */
    public boolean isAdiab() {
        return adiab;
    }
/**
 * Nastavuje, zda je flash adiabatický (true)
 * @param adiab boolean
 */
    public void setAdiab(boolean adiab) {
        this.adiab = adiab;
    }
/**
 * Vrací výslednou teplotu flashe (K)
 * @return double
 */
    public double gettFlash() {
        return tFlash;
    }
/** 
 * Nastavuje koncovou teplotu flashe (odhad pro adiab. flash)
 * @param tFlash double
 */
    public void settFlash(double tFlash) {
        this.tFlash = tFlash;
    }
/**
 * Vrací koncový tlak flashe (Pa)
 * @return tlak double
 */
    public double getpFlash() {
        return pFlash;
    }
/**
 * Nastavuje koncový tlak flashe (Pa)
 * @param pFlash double
 */
    public void setpFlash(double pFlash) {
        this.pFlash = pFlash;
    }
/**
 * Vrací vstupní proud
 * @return Proud
 */
    public Proud getPr() {
        return pr;
    }
/**
 * Nastavuje vstupní proud
 * @param pr  Proud
 */
    public void setPr(Proud pr) {
        this.pr = pr;
    }
/**
 * Vrací poměr parní fáze ku nástřiku
 * @return V/F  double
 */
    public double getVf() {
        return vf;
    }    
}
