/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;

import fydat.I_Fmin;
import fydat.Numer;
import fydat.NumerException;
import fydatlib.Smes;
import javax.swing.JOptionPane;

/** Aparát komprese nebo škrcení proudu
 * Komprese je izentropická, škrcení izenentalpické<br>
 * Mějte na paměti zda zadání fyzikálně odpovídá
 * (existence plynného stavu)
 * @author V. Hlubuček
 */
public class Komprese extends Aparat implements I_Fmin {

    protected String VERZE = "1.1";
    private boolean proudNastaven;
    protected boolean komprese;
    protected double pFpoc, pFkon, tFpoc, tFkon = 0;
    private double tPom, pPom, hPom;
    protected double[] x = new double[2];   // proměnná pro Newton
    Smes sm;
/**
 * Konstruktor komprese/škrcení
 * @param cApar číslo aparátu
 * @param nazevAparatu textový název aparátu
 */
    public Komprese(int cApar, String nazevAparatu) {
        super(cApar,nazevAparatu);       
    }
    
/**
 * Konstruktor komprese (izentropický děj)/škrcení (izentalpický děj)
 * @param cApar
 * @param nazevAparatu
 * @param pr
 * @param pFpoc - počáteční tlak (Pa)
 * @param pFkon - koncový tlak (Pa)
 * @param tFpoc - počáteční teplota (°C)
 * @param komprese - true; škrcení - false
 */
    public Komprese(int cApar, String nazevAparatu, Proud pr, double pFpoc, 
            double pFkon, double tFpoc, boolean komprese) {
        super(cApar,nazevAparatu);
        pocetInProudu = 1;
        pocetOutProudu = 1; 
        pridejProud(pr);
        if(proudyIn.isEmpty()) proudyIn.add(pr); else proudyIn.set(0, pr);       
        this.pFpoc =pFpoc;
        this.pFkon =pFkon;
        this.tFpoc =tFpoc;  // poč. teplota proudu (°C)
        this.komprese = komprese;
        proudNastaven = true;
        myInit();        
    }    
    
/**
 * Bezparametrový konstruktor komprese/škrcení
 */    
    public Komprese() {
        super();
        pocetInProudu = 1;
        pocetOutProudu = 1;        
        nazevAparatu = "Komprese/Škrcení ";
        if(proudyIn.isEmpty()) proudyIn.add(new Proud());   // dosadí prázdný proud 
        myInit();
    }
/**
 * Inicializace aparátu
 * popis labelů a default hodnot pro interaktivní zadávání
 */
    public void myInit() {
        lTexty = new String[10];
        tValue = new String[10];
        lTexty[0] = "Komprese";
        lTexty[1] = "Č.aparátu";
        lTexty[2] = "Název aparátu";
        lTexty[3] = "Počet In proudů";
        lTexty[4] = "Počet Out proudů";
        lTexty[5] = "Koncová teplota (°C)";
        lTexty[6] = "Koncový tlak (Pa)";
        lTexty[7] = "(0-komprese 1-škrcení)";
        lTexty[8] = ""; // musí být jako ukončovač
        if(pFkon > 0) {
            tValue[1] = String.format("%3d",cApar);
            tValue[2] = nazevAparatu;
            tValue[3] = String.format("%2d",pocetInProudu);
            tValue[4] = String.format("%2d",pocetOutProudu);
            tValue[5] = String.format("%5.1f",tFkon);
            tValue[6] = String.format("%7.0f",pFkon);
            tValue[7] = (komprese == true)? "0": "1";
        } else {
            tValue[1] = "0";
            tValue[2] = "Komprese";
            tValue[3] = "1";
            tValue[4] = "1";
            tValue[5] = "100";
            tValue[6] = "300000";
            tValue[7] = "0";            
        }     
    }
/**
 * Dosadí do proměnných hodnoty z interaktivního zadání
 */    
    public void dosadInteraktivniHodnoty() {
        cApar = (int)doubleFromString(tValue[1]);
        nazevAparatu = tValue[2];
        pocetInProudu = (int)doubleFromString(tValue[3]);
        pocetOutProudu = (int)doubleFromString(tValue[4]);
        tFkon = doubleFromString(tValue[5]);
        pFkon = doubleFromString(tValue[6]);
        if((int)doubleFromString(tValue[7]) == 0) komprese = true; else komprese = false;
    }    
    
/**
 * Přidání proudu do aparátu Komprese/Škrcení
 * @param pr proud
 */    
    @Override
    public void pridejProud(Proud pr) {
        if(!proudNastaven) {
            proudy.add(pr);
            proudy.add(pr.kopirujProud(pr));
            if(proudyIn.isEmpty()) proudyIn.add(pr); else proudyIn.set(0, pr);       
            proudNastaven = true; 
        } else {
            JOptionPane.showMessageDialog(null,
             nazevAparatu + ": Proud již byl zadán: " + proudy.get(0).getNazProudu()
             + "\nChcete-li zadat jiný musíte nejdříve odebrat původní!"
             + "\nLze i jako: odeberProud(null);",
             "Komprese/Škrcení", JOptionPane.WARNING_MESSAGE);
            return;            
        }
    }
/**
 * Odebrání proudu z aparátu Komprese/Směšocač
 * @param pr proud
 */    
    @Override
    public void odeberProud(Proud pr) {
        proudy.clear();     // odstraní oba proudy
        proudNastaven = false;         
    }  
/**
 * Ověřuje zadání aparátu
 * @return 
 */
    @Override
    public boolean jeAparatZadan() {
        boolean neni = false;
        if(this == null) return false;
        return (!neni);
    }    
    /**
     * Komprese/škrcení par
     * @return  výsledky jako string
     */
    public String komprese() {
        String tx;
        double  fmin = -9999;
        double  pom;
        if (proudy.isEmpty()) {
            JOptionPane.showMessageDialog(null,
                nazevAparatu + ": Nebyl dosud zadán vstupní proud!",
                "Komprese/Škrcení", JOptionPane.WARNING_MESSAGE);
            return "";            
        }
        komprese = true;
        if (pFpoc > pFkon) komprese = false;
        sm = proudy.get(0).getSmes();   // směs vstupního proudu
        sm.setNeMistniHlasky(true);
        report = (komprese ? "Komprese" : "Škrcení") + " aparát: " + nazevAparatu +
                "  č.: " + cApar + "\n";

        tPom = tFpoc + TABS;   
        pPom = pFpoc;
        hPom = pFkon;
        if(!komprese && pFpoc < pFkon) {
            JOptionPane.showMessageDialog(null,
                "Zadání neodpovídá klasickému škrcení\nKoncový tlak vyšší, než počáteční.",
                "FYDAT škrcení", JOptionPane.WARNING_MESSAGE);
//            return "";
        }
        if(komprese && pFpoc > pFkon) {
            JOptionPane.showMessageDialog(null,
                "Zadání neodpovídá klasické kompresi\nKoncový tlak nižší, než počáteční.",
                "FYDAT komprese", JOptionPane.WARNING_MESSAGE);
//            return "";
        }
        x[0] = tPom;   // odhad koncové teploty
        x[1] = 0;    // dummy
        I_Fmin fi = this;
        try {
            fmin = Numer.m_Fmin1(fi, 2, x, 1e-4); // volání minimalizace
//            System.out.println("*Komp T, fmin : " + x[0] + " * " + fmin);
        } catch (NumerException e) {
            e.getStackTrace();
        }
        // Zápis výsledků
        tFkon = x[0] - TABS;
        tx = (!komprese ? "ŠKRCENÍ" : "KOMPRESE");
        tx += "\nPočáteční tlak (MPa): " + String.format("%-6.4f", pFpoc / 1e6);
        tx += "   Koncový tlak (MPa): " + String.format("%-6.4f%n", pFkon / 1e6);
        tx += "Počáteční teplota (°C): " + String.format("%-5.1f", tFpoc)
                + "  Koncová teplota (°C): "
                + String.format("%-5.1f%n", tFkon);
        tx += String.format("Hustota před a po (kg/m3): %8.3f  %8.3f%n",
                sm.sHust(tFpoc+TABS, pFpoc),sm.sHust(tFkon+TABS, pFkon)); 
        tx += "Bod varu a rosný bod - výchozí (°C): "
                + String.format("%-5.1f", sm.sTbv(pFpoc) - 273.15)
                + "  " + String.format("%-5.1f%n", sm.sTrb(pFpoc) - 273.15);
        tx += "Bod varu a rosný bod - koncový(°C): "
                + String.format("%-5.1f", sm.sTbv(pFkon) - 273.15)
                + "  " + String.format("%-5.1f%n", sm.sTrb(pFkon) - 273.15);
        pom = sm.srUGW(x[0], hPom) - sm.srUGW(tPom, pPom);
        tx += "Čistá kompresní práce (kJ/kg): "
                + String.format("%-8.2f%n", pom / 1000.);
        tx += "Výkon kompresoru (kW na 1kg/s) :"
                + String.format("%-8.1f%n", (pom / 1000.
                + (hPom / sm.srHustGW(x[0], hPom)
                - pPom / sm.srHustGW(tPom, pPom)) / 1000));
        tx += "Využitelná entalpie (kJ/kg): "
                + String.format("%-8.1f", (sm.srHGW(x[0], hPom)
                - sm.sHLW(sm.sTbv(pFkon))) / 1000.);
        tx += "\n";
        report += tx + "\n";   
        proudy.get(1).setNazProudu("Proud z " + (komprese ? "komprese": "škrcení"));
        proudy.get(1).settVstup(tFpoc);
        proudy.get(1).settVystup(x[0] - TABS);
        proudy.get(1).setpVstup(pFpoc);
        proudy.get(1).setpVystup(pFkon);
        proudy.get(0).spocten = true;
        proudy.get(1).spocten = true;
        proudy.get(1).zUzlu = this;
        proudy.get(1).nazProudu = "Proud z " + this.nazevAparatu;
        proudy.get(0).setVysledek(report);
        proudy.get(1).setVysledek(report);
        pridejProudOut(proudy.get(1));
        sm.setNeMistniHlasky(false);
        if(zobrazujVysledky) {
//            pr.setVysledek(report);
            new JDialogProud(null, nazevAparatu + " výstup (Komprese  Verze " 
                        + VERZE + ")\n", proudy.get(1)); // modálně vyskakovací               
        }        
        return tx;
    }
/**
 * Funkce pro výpočet komprese/škrcení pro Fmin
 * @param x
 * @return výstupní teplota (°C);
 */
    public double f(double[] x) {
        int i, nSloz;
        double  sumh = 0;
        nSloz = sm.getnSloz();
        if(!komprese) {
     // škrcení **********************************
        sumh = sm.srHG(x[0], hPom) - sm.srHG(tPom, pPom);
        return sumh * sumh + 0.01 * (x[1] * x[1]);
        } else {
      // Komprese      
        sumh = sm.srSG(x[0], hPom) - sm.srSG(tPom, pPom);
        return sumh * sumh + 0.01 * (x[1] * x[1]);
        }
    }
    
/**
 * implementace abstraktní metody
 */
    @Override
    public void spoctiAparat() {
        komprese();
    }

    @Override
    public void spoctiAparat(boolean tisk) {
        boolean bylo = zobrazujVysledky;
        if(tisk)zobrazujVysledky = true; else zobrazujVysledky = false;
        komprese();
        zobrazujVysledky = bylo;
    }

    
// gettery a settery  -------------------------------------------------    
/**
 * Vrací spočtený výstupní proud po kompresi či škrcení
 * @return Proud
 */    
    public Proud vystupniProud() {
        return proudy.get(1);
    }
/**
 * Vrací počáteční tlak (Pa)
 * @return 
 */
    public double getpFpoc() {
        return pFpoc;
    }
/**
 * Nastavuje počáteční tlak (Pa)
 * @param pFpoc 
 */
    public void setpFpoc(double pFpoc) {
        this.pFpoc = pFpoc;
    }
/**
 * Vrací koncový tlak (Pa)
 * @return 
 */
    public double getpFkon() {
        return pFkon;
    }
/**
 * Nastavuje koncový tlak (Pa)
 * @param pFkon 
 */
    public void setpFkon(double pFkon) {
        this.pFkon = pFkon;
    }
/**
 * Vrací počáteční teplotu (°C)
 * @return 
 */
    public double gettFpoc() {
        return tFpoc;
    }
/**
 * Nastavuje počáteční teplotu (°C)
 * @param tFpoc 
 */
    public void settFpoc(double tFpoc) {
        this.tFpoc = tFpoc;
    }       

}
