/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;

import javax.swing.JOptionPane;

/**
 * Třída popisuje tlakovou ztrátu v elementu náhlé zúžení nebo rozšíření
 * @author V. Hlubuček
 */
public class NahlaZmena extends Element {

    private double prumerVstup,  drsnost, vyskovyRozdil;
    private double prumerVystup; 
/**
 * Kopnstruktor náhlé změny průměru
 * @param prumerVstup vstupní průměr trubky (m)
 * @param prumerVystup výstupní průměr trubky (m)
 */
    public NahlaZmena(Proud pr,double prumerVstup, double prumerVystup) {
        super();
        this.pr = pr;
        nazElementu = "";
        drsnost = 0.0001;        
        this.prumerVstup = prumerVstup;
        this.prumerVystup = prumerVystup;
        myInit();        
    }
    
/**
 * Inicializace elementu náhlá změna
 * popis labelů a default hodnot pro interaktivní zadávání
 */
    public void myInit() {
        lTexty = new String[6];
        tValue = new String[6];
        lTexty[0] = "Náhlá změna";
        lTexty[1] = "Průměr vstupu(m)";
        lTexty[2] = "Průměr výstupu(m)";
        lTexty[3] = "Drsnost (m)";
        lTexty[4] = "Výškový rozdíl(m)";
        lTexty[5] = "";
//        lTexty[6] = "";
        tValue[1] = "0,03";
        tValue[2] = "0,05";
        tValue[3] = "0,0004";
        tValue[4] = "0";
//        tValue[5] = "0"; 
    }
/**
 * Dosadí do proměnných hodnoty z interaktivního zadání
 */    
    public void dosadInteraktivniHodnoty() {
        prumerVstup = doubleFromString(tValue[1]);
        prumerVystup = doubleFromString(tValue[2]);
        drsnost = doubleFromString(tValue[3]);
        vyskovyRozdil = doubleFromString(tValue[4]);
    }    
    
    
/**
 * Počítá tlakovou ztrátu (Pa) na náhlé změně průměru trubky
 * @param pr proud
 * @return tlaková ztráta (Pa)
 */        
    @Override
    public double tlakZtrata() {
        boolean kapalina;
        double din, dout, gsi, ro, t, p, pom, vin, vout;
        String faze;
        din = prumerVstup;
        dout = prumerVystup;
        t = tVstup+TABS;    // absolutní teplota (K)       
        p = pVstup;
        if (pVstup*tVstup <= 0 && pr.isTisk()) {
           JOptionPane.showMessageDialog(null,
              "V elementu " + nazElementu + " pr. (" + pr.cProudu + ") nulový tlak nebo teplota !",
              "Náhlá změna", JOptionPane.WARNING_MESSAGE);            
            return -9999;   // chyba
        }       
        kapalina = false;
        if (t < pr.getSmes().sTbv(p)) kapalina = true; 
        if (kapalina) {
            ro = pr.getSmes().sHustL(t);
            faze = "l";
        } else {
            ro = pr.getSmes().srHustGW(t, p);
            faze = "g";
        }        
        vin = 4 * pr.getHmotPrutok() / ro / Math.PI / din / din;
        pom = dout*dout/din/din;
        vout = vin/pom;
        if (din < dout) {     // rozšíření
            gsi = (vin-vout)*(vin-vout)/vout/vout + 0.2 *(pom - 1);
            if (nazElementu.equals("")) nazElementu = "Rozšíření";
        } else {            // zúžení
            gsi = 0.5 * (1 - pom);
            if (nazElementu.equals("")) nazElementu = "Zúžení";
        }
        pVystup = p - gsi * vout * vout * ro/2;
        tVystup = tVstup;
        rychlVystup = vout;
        rychlVstup = vin;
        report = "Náhlá změna (" + nazElementu + ") ("+faze+")\n";
        report += " průměr vstupní    (m) " + String.format("%5.2f%n",prumerVstup);
        report += " průměr výstupní   (m) " + String.format("%5.2f%n",prumerVystup);
//        report += " úhel konicity   (°) " + String.format("%5.2f%n",uhel);
        report += " výškový spád      (m) "  + String.format("%5.2f%n",vyskovyRozdil);      
        report += " drsnost potrubí   (m) " + String.format("%7.5f%n",drsnost);
        report +=  " hustota       (kg/m3) " + String.format("%5.2f%n",ro);
        report +=  " teplota bodu varu(°C) " + String.format("%5.1f%n",pr.getSmes().sTbv(p)-TABS);
        report +=  " viskozita      (Pa.s) " + String.format("%6.4g%n",pr.getSmes().sViskL(t));
        report +=  " rychlost vstup  (m/s) " + String.format("%6.3f%n",vin);
        report +=  " rychlost výstup (m/s) " + String.format("%6.3f%n",vout);
//        report +=  " Reynoldsovo č.  (1) " + String.format("%6.4g%n",nRe);
        report +=  " součinitel odporu (1) " + String.format("%6.4f%n",gsi);
        report +=  " teplota          (°C) " + String.format("%5.1f%n",tVstup);
        report +=  " tlak na vstupu   (Pa) " + String.format("%7.0f%n",pVstup);
        report +=  " tlak na výstupu  (Pa) " + String.format("%7.0f%n",pVystup);
        report +=  " tlaková ztráta   (Pa) " + String.format("%6.1f%n",p - pVystup);        
        return pVstup - pVystup;   // tlaková ztráta    
    }

    @Override
    public String vysledkyElement() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
/**
 * Vrací vstupní průměr konické trubky (m)
 * @return vstupní průměr
 */
    public double getPrumerVstup() {
        return prumerVstup;
    }
/**
 * Nastavuje vstupní průměr konické trubky (m)
 * @param prumerVstup 
 */
    public void setPrumerVstup(double prumerVstup) {
        this.prumerVstup = prumerVstup;
    }
/**
 * Vrací abs. drsnost potrubí (m)
 * @return drsnost
 */
    public double getDrsnost() {
        return drsnost;
    }
/**
 * Nastavuje abs. drsnost potrubí (m)
 * @param drsnost 
 */
    public void setDrsnost(double drsnost) {
        this.drsnost = drsnost;
    }
/**
 * Vrací výškový rozdíl (m)
 * @return 
 */
    public double getVyskovyRozdil() {
        return vyskovyRozdil;
    }
/**
 * Nastavuje výškový rozdíl (m)
 * @param vyskovyRozdil 
 */
    public void setVyskovyRozdil(double vyskovyRozdil) {
        this.vyskovyRozdil = vyskovyRozdil;
    }
/**
 * Vrací výstupní průměr konické trubky (m)
 * @return průměr
 */
    public double getPrumerVystup() {
        return prumerVystup;
    }
/**
 * Nastavuje výstupní průměr konické trubky (m)
 * @param prumerVystup 
 */
    public void setPrumerVystup(double prumerVystup) {
        this.prumerVystup = prumerVystup;
    }
//

        
}
