/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;

import fydatlib.Smes;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import org.jdom2.JDOMException;

/**
 * Třída aparátu ideální Směšovač - smísí vstupní proudy do výstupního,<br>
 * stanovení průtoku, složení a teploty po smíšení na základě hmotové<br>
 * a entalpické bilance. Předpokládá stejný tlak vstupních proudů a vychází<br>
 * z hmotových průtoků směšovaných proudů.
 * @author V.Hlubuček
 */
public class SmesovacIdeal extends Aparat{
        protected String VERZE = "1.5";
/**
 * Konstruktor ideálního směšovače
 * @param cApar číslo aparátu
 * @param nazevAparatu textový název aparátu
 */
    public SmesovacIdeal(int cApar, String nazevAparatu) {
        super(cApar,nazevAparatu);
        pocetInProudu = 2;
        pocetOutProudu = 1;        
    }
/**
 * Bezparametrový konstruktor ideálního směšovače
 */    
    public SmesovacIdeal() {
        super();
        pocetInProudu = 2;
        pocetOutProudu = 1;        
        nazevAparatu = "Ideální směšovač ";
    }
    @Override
    public void pridejProud(Proud pr) {
        super.pridejProud(pr);
        proudyIn.add(pr);      // přidá proud do vstupních
        if(proudyOut.isEmpty())proudyOut.add(pr.kopirujProud(pr)); // a současně jeden fiktivní výstupní proud
    }
    
/**
 * Smíšení proudů s možností potlačení zobrazení výsledků
 * @param tisk (true ano; false ne)
 * @return 
 */
    public Proud smisitProudy(boolean tisk) {
        boolean bylo;
        Proud pr;
        bylo = zobrazujVysledky;    // původní nastavení
        if(!tisk) zobrazujVysledky = false;
        pr = smisitProudy();
        zobrazujVysledky = bylo;
        return pr;
    }    
/**
 * Metoda zajistí smíšených vstupních proudů do výstupního
 * na základě hmotové a entalpické bilance
 * @return proud
 */    
    public Proud smisitProudy() {
        int nSlozVystup = 0, iVystup = -999, nMix = 901;
        int[] cSlozek = new int[30];
        double p=101320, t = 290, qCelk = 0, mCelk = 0;    // pro entalpickou bilanci
        Proud proudMix = null;
        Smes smMix = null;
        // zjistit všechny složky výstupního proudu
        report = "SmesovacIdeal c. " + cApar + "  " + nazevAparatu + "\n";
        p = 999e8;
        for (Proud pr : proudy) {
            if (jeProudVstupni(pr) >= 0) {
                pr.pVystup = pr.getpVystup() > 0? pr.getpVystup(): pr.getpVstup();
                pr.tVystup = pr.gettVystup()> 0 ? pr.gettVystup():pr.gettVstup();               
                mCelk += pr.getHmotPrutok();
                qCelk += pr.getHmotPrutok() * 
                      pr.getSmes().sCpW(pr.gettVstup()+TABS, pr.getpVstup())*(pr.gettVstup()+TABS);
                if (pr.getpVystup() < p) p = pr.getpVystup();     // ten nejnižší
                if(pr.gettVystup()< 0.0001) pr.settVystup(pr.gettVystup());
                t = pr.gettVystup() + TABS;
                int[] cisla =  pr.getSmes().getCislaSlozek(); // pole s čísly složek směsi               
                for (int i=0; i<pr.getSmes().getnSloz();i++) {  // přes složky zkoumaného proudu
                    if (neniPrvkem(cisla[i], cSlozek)) {
                        cSlozek[nSlozVystup] = cisla[i];
                        nSlozVystup++;
                    }             
                }
            } else {
               iVystup =  pr.getcProudu();
               proudMix = pr;   // výstupní proud
            }
        }
        try {
            smMix = new Smes(nSlozVystup, cSlozek);
        } catch (JDOMException ex) {
            Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);                 
        } catch (IOException ex) {
            Logger.getLogger(SmesovacIdeal.class.getName()).log(Level.SEVERE, null, ex);
            JOptionPane.showMessageDialog(null,
                "Výstupní směs SmesovacIdeal č. " + cApar + "  " + nazevAparatu + "\nse nepodařilo vytvořit. Končím!",
                 "SmesovacIdeal - smisitProudy", JOptionPane.WARNING_MESSAGE);
            return null;            
        }
        // složková hmotová bilance
        double[] sum = new double[nSlozVystup];
        report += "Vstupní proudy:\n";
        for (Proud pr : proudy) {
            if (jeProudVstupni(pr) >= 0) {
                int[] cisla = pr.getSmes().getCislaSlozek();    // pole s čísly složek směsi            
                for (int i=0; i < pr.getSmes().getnSloz();i++) { 
                  for (int j = 0; j<nSlozVystup;j++) {
                      if (cSlozek[j] == cisla[i]){ sum[j] += pr.getSmes().getxMol(i); }
                  }
                }
                report += String.format("Proud: %s  hmotový průtok (kg/s, kg/h) = %7.4f; %7.1f%n",
                        pr.getNazProudu(),pr.getHmotPrutok(),pr.getHmotPrutok()*3600);
                report += String.format("  teplota (°C) = %4.1f  tlak (kPa) = %7.2f%n",
                        pr.gettVystup()> 0 ? pr.gettVystup():pr.gettVstup(), 
                        pr.getpVstup() > 0? pr.getpVystup()/1000:pr.getpVstup()/1000);
                report += String.format("  hustota(kg/m3) = %4.1f%n",
                        pr.getSmes().sHust(pr.gettVystup()> 0 ? pr.gettVystup()+TABS:pr.gettVstup()+TABS,
                        pr.getpVystup() > 0? pr.getpVystup():pr.getpVstup()));
                report += "  slozky: ";
                for (int i = 0; i < pr.getSmes().getnSloz(); i++) {
                   report += " " + cisla[i]; 
                }
                report += "\n";
            }
        }
        try {
            smMix.zadatSloz(false, sum);
        } catch (IOException ex) {
            Logger.getLogger(SmesovacIdeal.class.getName()).log(Level.SEVERE, null, ex);
        }
        if (iVystup < 0) {  // výstupní proud není dosud definován
            proudMix = new Proud(nMix,"Výstup z " + nazevAparatu);
        } 
        proudMix.setSmes(smMix);            // přiřazení směsi po smíšení
        proudMix.setHmotPrutok(mCelk);      // směšovací hmotový průtok
        proudMix.setpVstup(p);              // nejnižší vstupní tlak
        proudMix.setpVystup(p);             // vystupní tlak = vstupní
        proudMix.settVstup(t);
        double t1 = t + 1;
        while (Math.abs(t - t1) > 0.05) {
            t = t1;
            t1 = qCelk/mCelk/smMix.sCpW(t, p);
            t1 = t + 0.3 * (t1 - t);
//            System.out.println("Cp"+smMix.sCpW(t, p)+" t="+t);
        }
        proudMix.settVstup(t1 - TABS);     // směšovací teplota
        proudMix.settVystup(proudMix.gettVstup());
        proudMix.spocten = true;
        proudMix.zUzlu = this;
        // smíšený výstupní proud do reportu
                int[] cisla = proudMix.getSmes().getCislaSlozek();    // pole s čísly složek směsi            
                for (int i=0; i < proudMix.getSmes().getnSloz();i++) { 
                  for (int j = 0; j<nSlozVystup;j++) {
                      if (cSlozek[j] == cisla[i]){ sum[j] += proudMix.getSmes().getxMol(i); }
                  }
                }
                report += String.format("Smíšený proud:%n %s  hmotový průtok (kg/s, kg/h) = %7.4f; %7.1f%n",
                        proudMix.getNazProudu(),proudMix.getHmotPrutok(),proudMix.getHmotPrutok()*3600);
                report += String.format("  teplota (°C) = %4.1f  tlak (kPa) = %7.2f%n",
                        proudMix.gettVystup()> 0 ? proudMix.gettVystup():proudMix.gettVstup(), 
                        proudMix.getpVstup() > 0? proudMix.getpVystup()/1000:proudMix.getpVstup()/1000);
                report += String.format("  hustota(kg/m3) = %4.1f%n",
                        proudMix.getSmes().sHust(proudMix.gettVystup()> 0 ? proudMix.gettVystup()+TABS:proudMix.gettVstup()+TABS,
                        proudMix.getpVystup() > 0? proudMix.getpVystup():proudMix.getpVstup()));
                report += "  slozky: ";
                for (int i = 0; i < proudMix.getSmes().getnSloz(); i++) {
                   report += " " + cisla[i]; 
                }
                report += "\n";        
        proudMix.setVysledek(report);       // uplatnění výsledkového okna
        proudy.add(proudMix);     // přidá reakční proud do kolekce proudů jako výstupní 
        if(proudyOut.isEmpty()) proudyOut.add(proudMix); else proudyOut.set(0, proudMix);
        pocetInProudu = proudyIn.size();    // skutečný počet vstupních proudů směšovače
        if(zobrazujVysledky) {
            new JDialogProud(null, "Směšovač ideal výstupní proud (Verze " 
                        + VERZE + ")\n", proudMix); // modálně vyskakovací okno s výsledky              
        }        
        return proudMix;
        }
 /**
  * Metoda zjistí, zda je prvek prvkem pole
  * @param prvek testovaný prvek
  * @param pole zkoumané pole prvků
  * @return 
  */   
    public boolean neniPrvkem(int prvek, int[] pole) {
        for (int i = 0;i < pole.length;i++) {
            if (prvek == pole[i]) {
                return false;
            }
        }
        return true;
    }
    
/**
 * implementace abstraktní metody
 */
    @Override
    public void spoctiAparat() {
        smisitProudy();
    }

    @Override
    public void spoctiAparat(boolean tisk) {
        if(tisk) smisitProudy(true); else smisitProudy(false); 
    }

    @Override
    public boolean jeAparatZadan() {
        boolean neni = false;
        if(proudyIn == null) neni = true; else neni = false;
        return (!neni);
    }

    @Override
    public void dosadInteraktivniHodnoty() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
    
    }

