/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package cheminzlib;

import fydatlib.Smes;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import org.jdom2.JDOMException;

/**
 * Třída pro short-cut výpočet vícesložkové rektifikace
 * Předpokládá ideální chování
 * @author Hlubuček
 */
public class SmithBrinkley  extends Aparat {

    protected String VERZE = "1.1";
    protected int nSloz;            // počet složek
    /**
     * Celkový počet pater, z toho pod nástřikem
     */
    protected int nCelk, nStrip;  // celkový počet pater, z toho pod nástřikem
    /**
     * Poměr parní fáze k nástřiku (0 až 1)
     */
    protected double vfFeed = 0, vf = 0; // poměry parní fáze k nástřiku
    /**
     * Refluxní poměr, destilát jako procenta z nástřiku (%)
     */
    protected double reflux, destToFeed;// reflux a destilát jako procenta z nástřiku (%)
    /**
     * Složení nástřiku (mol.zl.)
     */    
    protected double[] xFeed;           // složení nástřiku 
//    protected double[] yFeed;           // počáteční složení parní fáze
    protected double[] xDest;           // koncové složení destilátu
    /**
     * Koncové složení zbytku (mol.zl.)
     */    
    protected double[] xBot;            // koncové složení zbytku    
    protected Smes sm,smDest, smBot;    // vstupní směs, směs destilátu  zbytku
    protected Proud pr, prDest, prBot;  // vstupní nástřikový proud a proudy destilátu a zbytku
    // pomocné interní proměnné
    protected double tS, tR, t, p;      // abs. teplota v rektifikační a stripovací sekci, teplota (K), tlak (Pa)
    /**
     * Teplota destilátu a zbytku (K)
     */
    protected double tD, tB;            // abs. teplota destilátu a zbytku (K)
    /**
     * Relativní těkavosti rektifikační a stripovací sekce
     */
    protected double[] alfaR,alfaS;     // relativní těkavosti složek (rekt. a strip. sekce)         
/**
 * Konstruktor třídy
 */    
    public SmithBrinkley(int nCelk, int nStrip, double reflux, double vfFeed,double destToFeed) {
        super();
        myInit();
        pocetInProudu = 1;
        pocetOutProudu = 2; 
        nazevAparatu = "Kolona SB";        
        this.nCelk = nCelk;
        this.nStrip = nStrip;
        this.reflux = reflux;
        this.vfFeed = vfFeed;
        this.destToFeed = destToFeed;

    }
    
/**
 * Konstruktor třídy s nástřikovým proudem
 */    
    public SmithBrinkley(Proud pr,int nCelk, int nStrip, double reflux, double vfFeed,double destToFeed) {
        super();
        pocetInProudu = 1;
        pocetOutProudu = 2; 
        nazevAparatu = "Kolona SB";        
        if(pr == null) {
            JOptionPane.showMessageDialog(null,
                "Zadání nástřiku kolony Smith-Brinkley č. " + cApar + "  " + nazevAparatu + "\nchybí. Končím!",
                 "Kolona Smith-Brinkley", JOptionPane.WARNING_MESSAGE);
//            return;                   
        } else {
        this.pr = pr;
        pr.doUzlu = this;
        p = pr.pVstup;
        sm = pr.sm;
        nSloz = sm.getnSloz();
        xFeed = sm.getxMol();   // molární složení nástřiku        
        }        
        this.nCelk = nCelk;
        this.nStrip = nStrip;
        this.reflux = reflux;
        this.vfFeed = vfFeed;
        this.destToFeed = destToFeed;
        myInit();        
    }
    
    
/**
 * Bezparametrový konstruktor
 */    
    public SmithBrinkley() {
        super();
        myInit();
        pocetInProudu = 1;
        pocetOutProudu = 2;        
        nazevAparatu = "Kolona SB";
    }
    
/**
 * Inicializace aparátu
 * popis labelů a default hodnot pro interaktivní zadávání
 */
    public void myInit() {
        lTexty = new String[11];
        tValue = new String[11];
        lTexty[0] = "Kolona SB";
        lTexty[1] = "Č.aparátu";
        lTexty[2] = "Název aparátu";
        lTexty[3] = "Počet In proudů";
        lTexty[4] = "Počet Out proudů";
        lTexty[5] = "Celkový počet pater";
        lTexty[6] = "Pater pod nástřikem";
        lTexty[7] = "Poměr parní fáze v nástřiku (0-1)";
        lTexty[8] = "Refluxní poměr";
        lTexty[9] = "% destilátu z nástříku (%)";
        lTexty[10] = ""; // musí být jako ukončovač
        if(nCelk <= 0) { 
            tValue[1] = "0";
            tValue[2] = "Kolona SB";
            tValue[3] = "1";
            tValue[4] = "1";
            tValue[5] = "15";
            tValue[6] = "6";
            tValue[7] = "0";
            tValue[8] = "2,5";
            tValue[9] = "15"; 
        } else{     // data z již zadaného
            tValue[1] = String.format("%3d",cApar);
            tValue[2] = nazevAparatu;
            tValue[3] = String.format("%2d",pocetInProudu);
            tValue[4] = String.format("%2d",pocetOutProudu);
            tValue[5] = String.format("%3d",nCelk);
            tValue[6] = String.format("%3d",nStrip);
            tValue[7] = String.format("%4.3f",vfFeed);
            tValue[8] = String.format("%4.3f",reflux);
            tValue[9] = String.format("%5.2f",destToFeed);             
        }
    }
/**
 * Dosadí do proměnných hodnoty z interaktivního zadání
 */    
    public void dosadInteraktivniHodnoty() {
        cApar = (int)doubleFromString(tValue[1]);
        nazevAparatu = tValue[2];
        pocetInProudu = (int)doubleFromString(tValue[3]);
        pocetOutProudu = (int)doubleFromString(tValue[4]);
        nCelk = (int)doubleFromString(tValue[5]);
        nStrip = (int)doubleFromString(tValue[6]);
        vfFeed = doubleFromString(tValue[7]);
        reflux = doubleFromString(tValue[8]);
        destToFeed = doubleFromString(tValue[9]);
    }    
    
    
/**
 * Funkce pro vyčíslení relativních těkavostí
 * v rektifikační a stripovací sekci
 */    
    protected void relTekavosti() {
        tR = (tD + t) / 2;
        tS = (tB + t) / 2;
        if(alfaR == null) {
           alfaR = new double[nSloz];
           alfaS = new double[nSloz];
        }
        for(int i = 0; i < nSloz;i++) {
           alfaR[i] = sm.getSlozka(i).tenze(tR)/ p;  // P0(i) / P
           alfaS[i] = sm.getSlozka(i).tenze(tS)/ p;
        }        
    }
    protected void prepocetKoncTeplot() {
//  Přepočet koncových teplot
      tD = smDest.sTrb(p);
      tB = smBot.sTbv(p);        
    }
/**
 * Testuje zadání aparátu
 * @return boolean ano-ne
 */    
    @Override
    public boolean jeAparatZadan() {
        boolean neni = false;
        if(proudyIn == null) return false; 
        if(nCelk == 0 || nStrip == 0 || reflux == 0) neni = true;
        if(destToFeed == 0 ) neni = true;
        return (!neni);
    }
    
/**
 * Výpočet kolony metodou Smith-Brinkley se zobrazením výsledků
 * @return počet iterací
 */    
    public int vypocetSB() {        
      int i, j, iter, itmax;
      double  mLR, mLS, mVR, mVS, tlum, dest, epsD, m, n, q, destSum, destMol, botSum;
      double f, f8, f9 = 0, h, q1, q2, q3, s1, s2, t7, t8, w1, w2;
      dest = destToFeed;   // % podíl destilátu z nástřiku (hmotnostně)
      double dest0 = dest;
      itmax = 200; // Max. počet iterací
      tlum = 0.5; // Tlumící konstanta      
      epsD = 0.01; // Rel.tolerance množství destilátu
        if(pr != null) {
            proudy.add(pr);
            if(proudyIn.isEmpty()) proudyIn.add(pr); else proudyIn.set(0, pr);  // dosadí zadaný proud
            if(proudyOut.isEmpty()) proudyOut.add(pr.kopirujProud(pr)); else proudyIn.set(0, pr.kopirujProud(pr));
            proudyOut.add(pr.kopirujProud(pr));
        } else {
            if(!proudyIn.isEmpty())pr = proudyIn.get(0);
        } 
        if(pr != null) {
            pr.doUzlu = this;
            p = pr.pVstup;
            sm = pr.sm;
            nSloz = sm.getnSloz();
            xFeed = sm.getxMol();   // molární složení nástřiku             
        }
      if(xDest == null) {
           xDest = new double[nSloz];
           xBot = new double[nSloz];
      }      
        try {
            sm = pr.sm;
            smDest = sm.kopirujSmes(sm);  // vytvoř směsi stejného složení jako nástřik
            smBot = sm.kopirujSmes(sm);
        } catch (JDOMException ex) {
            Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);                 
        } catch (IOException ex) {
            Logger.getLogger(SmithBrinkley.class.getName()).log(Level.SEVERE, null, ex);
        }
      q = vfFeed;    // původně 0?
      iter = 0;
      t8 = 0;
      // odhaty koncových teplot (hlava, pata)
      tD = sm.sTbv(p) - 5;
      tB = tD +10;
      t = (tB + tD) / 2;         
//  Vlastní výpočet
      do {
      do {
          relTekavosti();
          mVR = (reflux + 1) * dest0;   // 65: parní fáze v rekt. sekci
          mVS = mVR - 100 * q;          // parní fáze ve stripovací sekci
          mLR = reflux * dest0;         // kapalná fáze v rekt. sekci
          mLS = mLR + 100 * (1 - q);    // kapalná fáze ve strip. sekci
          q1 = mVR / mLR;
          q2 = mVS / mLS;
          q3 = mLR / mLS;
          if(q1 > 1E+28) q1 = 1E+28;    // pojistky
          if(q2 > 1E+28) q2 = 1E+28;
          if(q3 > 1E+28) q3 = 1E+28;
          destSum = 0;
          botSum = 0;
          for(i = 0;i < nSloz;i++){
            s1 = alfaR[i] * q1;
            s2 = alfaS[i] * q2;
            h = q3 * ((1 - s1) / (1 - s2)) * (q + (1 - q) * alfaS[i] / alfaR[i]);
            w1 = Math.pow(s1, nCelk - nStrip);    // alfaR*VR/LR na počet pater rekt. sekce
            w2 = 1 - w1 + reflux * (1 - s1);
            f = w2 / (w2 + h * w1 * (1 - Math.pow(s2, nStrip)));  // Smith-Brinkley vztah
            xBot[i] = 100 * xFeed[i] * f;    // Slož.zbytku (mol.%)
            xDest[i] = 100 * xFeed[i] - xBot[i]; // Slož.destilatu
            destSum = destSum + xDest[i];
            botSum = botSum + xBot[i];
          }
          if(destSum <= 0) destSum = 0.00000001;
          f8 = f9;
          iter = iter + 1;
          for(i = 0;i < nSloz;i++) {    // dopočet složení destilátu a zbytku a jejich normalizace
            xBot[i] = xBot[i] / (100 - destSum);
            xDest[i] = xDest[i] / destSum;
          }
          if(iter == 2) prepocetKoncTeplot();
          // přepočet poměru z hmotového na molární
          {
              try {
                  smDest.zadatSloz(false, xDest);
                  smBot.zadatSloz(false, xBot);
              } catch (IOException ex) {
                  Logger.getLogger(SmithBrinkley.class.getName()).log(Level.SEVERE, null, ex);
              }
          destMol = dest * sm.sMolHmot() / smDest.sMolHmot();  // molů destilátu
          }
          dest0 = destMol;
          f9 = (destMol - destSum) / destMol;   // požadovaná shoda v destilátu
          if(iter <=1) {
              t8 = t;
              t = t + tlum * 20 * f9;
          }
    //  Metoda sečen
          t7 = t - f9 * (t - t8) / (f9 - f8);   // 20:
          if(Math.abs((t - t7) / t) >= 0.1) {
           tlum = tlum * 0.5; // proti divergenci  11:
           if(tlum < 0.0001) {
             iter = -2;   // Příznak divergence
             return iter; // stop //Přetlumeno//
           }
          }
          t8 = t;
          t = t * (1 - tlum) + tlum * t7;      
      } while(Math.abs(f9) > epsD ); // 30: ----------------------
      prepocetKoncTeplot();
      } while(Math.abs(f9) >= 0.00035  && iter < itmax); // konec vnějšího cyklu -----

        report = "Výpočet kolony short-cut metodou Smith-Brinkley";
        report += "\nPočáteční tlak (MPa): " + String.format("%-6.4f", p / 1e6);
        report += "\nTeplota nástřiku (°C): " + String.format("%-5.1f%n", pr.tVstup);
        report += String.format("Bod varu nástřiku (°C):%-5.1f%n", sm.sTbv(p) -TABS);
        report += String.format("Teplota hlava (°C):%-5.1f  pata (°C):%-5.1f%n", tD - TABS, tB - TABS);
        report += String.format("Refluxní poměr: %5.3f%n",reflux);
        report += String.format("Hmotový průtok nástřiku(kg/s) =  %7.4f  Podíl par v nástřiku (0-1) = %5.3f%n",
                pr.getHmotPrutok(), vfFeed);                
        report += String.format("Hmotový průtok destilátu(kg/s) = %7.4f%n",destToFeed/100 * pr.getHmotPrutok());
        report += String.format("Hmotový průtok zbytku(kg/s) =    %7.4f%n",(100 - destToFeed)/100 * pr.getHmotPrutok());
        report += String.format("Destilát (mol/s) = %7.3f  Zbytek (mol/s) = %7.3f  Nástřik (mol/s) = %7.3f%n",
                destToFeed/100 * pr.getHmotPrutok() / smDest.sMolHmot(), (100 - destToFeed)/100 * pr.getHmotPrutok()/smBot.sMolHmot(),pr.hmotPrutok / sm.sMolHmot()); 
        report += " Sl.           Látka  Nástřik  Destilát Zbytek    Nástřik  Destilát Zbytek\n";
        report += "                           (mol. zlomky)              (hmot. zlomky)\n";
        for(i = 0; i <nSloz; i++) {
            report += String.format("%3d %16s  %6.4f   %6.4f   %6.4f    %6.4f   %6.4f   %6.4f%n",
                    sm.getCisloSlozky(i), sm.getNazevSlozky(i),xFeed[i], xDest[i], xBot[i],
                    sm.getxHmot(i), smDest.getxHmot(i), smBot.getxHmot(i));
        }
        report += String.format("Počet iterací = %3d%n",iter);
        report += " Sl.           Látka  Rel.těk.R   Rel.těk.S  Min.reflux složky\n";
        for(i = 0; i <nSloz; i++) {
            report += String.format("%3d %16s  %6.4f      %6.4f      %6.4f%n",
                    sm.getCisloSlozky(i), sm.getNazevSlozky(i),alfaR[i]/alfaR[nSloz-1], alfaS[i]/alfaS[nSloz-1],minRefluxUnd(i));
        }
        double pom = (reflux + 1)* destToFeed/100;
        report += String.format("Teplo vařák (kW)= %7.4g   kondenzátor (kW)= %7.4g%n",
                smBot.sdHv(tB) * pr.hmotPrutok/sm.sMolHmot() * (pom - vfFeed)/1000, smDest.sdHv(tD) * pr.hmotPrutok/sm.sMolHmot() * pom/1000);
        pr.pVystup = pr.pVstup;
        pr.tVystup = sm.sTbv(p) -TABS;
        pr.spocten = true;
        pr.setVysledek(report); 
        if(iter > itmax) iter = -1;
//      minRefluxUnd(sm, p, t, vfFeed, xFeed, xDest,  0);  // jen zkouška statické metody
        if(zobrazujVysledky) {
            new JDialogProud(null, "Kolona Smith-Brinkley (Verze " 
                        + VERZE + ")\n", pr); // modálně vyskakovací okno s výsledky              
        }      
      return iter;    
    }
/**
 * Výpočet kolony metodou Smith-Brinkley s volbou zobrazení výsledků
 * @param tisk boolean
 * @return počet iterací
 */
    public int vypocetSB(boolean tisk) {
        boolean bylo = zobrazujVysledky;
        zobrazujVysledky = tisk;
        int i = vypocetSB();      // výpočet flashe 
        zobrazujVysledky = bylo;
        return i;
    }
/**
 * Počítá kolonu a výsledky ukládá do pole tří proudů
 * Proud[0] je vstupní proud do kolony 
 * Proud[1] je proud destilátu 
 * Proud[2] je proud zbytku 
 * @return Proud[] výsledné proudy
 */    
    public Proud[] proudySmithBrinkley() {
        Proud[] prd = new Proud[3];
        prd[0]= pr;         // vstupní proud flashe
        vypocetSB(false);      // výpočet flashe
        prd[1] = pr.kopirujProud(pr);
        prd[2] = pr.kopirujProud(pr);
        {
        prd[1].pVystup = p;
        prd[1].tVystup = tD -TABS;
        prd[1].doUzlu = null;
        prd[1].zUzlu = this;
        prd[1].hmotPrutok = destToFeed/100*pr.hmotPrutok;
        prd[1].sm = smDest;
        prd[1].setxHmot(smDest.getxHmot());
        prd[1].setxMol(smDest.getxMol());
        prd[1].setVysledek(this.report);
        prd[1].nazProudu = "Destilát " + nazevAparatu;
        prd[1].spocten = true;
        proudy.add(prd[1]);     // přidá reakční proud do kolekce proudů jako výstupní
        prDest = prd[1];
        prd[2].pVystup = p;
        prd[2].tVystup = tB - TABS;
        prd[2].doUzlu = null;        
        prd[2].zUzlu = this;
        prd[2].hmotPrutok = (1 - destToFeed/100) * pr.hmotPrutok;
        prd[2].sm = smBot;
        prd[2].setxHmot(smBot.getxHmot());
        prd[2].setxMol(smBot.getxMol());
        prd[2].setVysledek(this.report);
        prd[2].nazProudu = "Zbytek " + nazevAparatu;
        prd[2].spocten = true;
        proudy.add(prd[2]);     // přidá reakční proud do kolekce proudů jako výstupní 
        if(proudyOut.isEmpty())proudyOut.add(prd[1]); else proudyOut.set(0, prd[1]);
        if(proudyOut.size()== 1) proudyOut.add(prd[2]); else  proudyOut.set(1, prd[2]);
        prBot = prd[2];
        }
        if(zobrazujVysledky) {
            new JDialogProud(null, "Kolona Smith-Brinkley  Nástřik (Verze " 
                        + VERZE + ")\n", prd[0]); // modálně vyskakovací okno s výsledky 
            new JDialogProud(null, "Kolona Smith-Brinkley  Destilát (Verze " 
                        + VERZE + ")\n", prd[1]); // modálně vyskakovací okno s výsledky              
            new JDialogProud(null, "Kolona Smith-Brinkley  Zbytek (Verze " 
                        + VERZE + ")\n", prd[2]); // modálně vyskakovací okno s výsledky              
            
        }        
        return prd;
    }
/**
 * Výpočet minimálního refluxu Underwoodovou metodou
 * pro zadanou lehkou klíčovou složku ((0 az nSloz-2)
 * @param key číslo klíčové složky ((0 az nSloz-2)
 * @return  (-1 divergence; -2 nesplněny startovní podmínky; 0 klíč. složka mimo;
 */        
    public double minRefluxUnd(int key)  {
        
        int i, iter, itmax;
        double alfaMin = +9999, d,  f, f1, s, theta, theta0, theta1;
        double[] alfa = new double[nSloz];
        if(key >= nSloz-1 || key < 0) return 0;
        iter = 0;
        itmax = 500;
        for(i = 0; i < nSloz; i++) {
           alfa[i] = sm.getSlozka(i).tenze(tR)/ p;  // P0(i) / P
           alfaMin = Math.min(alfa[i], alfaMin);
        }                 
        for(i = 0; i < nSloz; i++){
            alfa[i] = alfa[i]/alfaMin;  // relativně k nejtěžší složce
        }
        // vlastní výpočet
        d = alfa[key] - alfa[key + 1];
        theta0 = alfa[key + 1] + 0.00000001 * d;
        s = vfFeed - 1;
        for(i = 0; i < nSloz; i++) {
           s = s + alfa[i] * xFeed[i] / (alfa[i] - theta0);
        }
        double f0 = s;
        theta = alfa[key] - 0.00000001 * d;
        s = vfFeed - 1;
        for(i = 0; i < nSloz; i++) {
           s = s + alfa[i] * xFeed[i] / (alfa[i] - theta);
        }
        f = s;
        if(f * f0 > 0) {
            return  -2; // Nesplněny startovní podmínky metody Regula Falsi!
        }
        do {
            theta1 = (f * theta - f0 * theta0) / (f - f0);
            s = vfFeed - 1;
            for(i = 0; i < nSloz; i++) {
               s = s + alfa[i] * xFeed[i] / (alfa[i] - theta1);
            }
            f1 = s;
            iter ++;
//            If Math.abs(theta - theta0) / theta < 0.0000005 And Math.abs(f1) < 0.00005 Or iter > itmax thetaen GoTo K
            if(f0 * f1 < 0) {
                theta = theta1; f = f1;
            }else {
                theta0 = theta1; f0 = f1;
            }
          } while(Math.abs(theta - theta0) / theta > 0.0000005 || Math.abs(f1) > 0.00005 && iter < itmax);          
          s = -1;        // výpočet zkonvergoval
          for(i = 0; i < nSloz; i++) {
             s = s + alfa[i] * xDest[i] / (alfa[i] - theta1);
          }
        if(iter <= itmax) {
          return s;  
        } else {
          return -1;     // překročet max.počet iterací
        }       
    }

/**
 * Výpočet minimálního refluxu Underwoodovou metodou<br>
 * pro zadanou lehkou klíčovou složku (0 az nSloz-2)<br>
 * statická metoda<br>
 * @param sm  Smes
 * @param p  zadaný tlak (Pa)
 * @param t  teplota (K)
 * @param vfFeed    podíl par v nástřiku (0-1)
 * @param xFeed složení nástřiku (mol. zlomky)
 * @param xDest složení destilátu (mol.zlomky)
 * @param key   číslo klíčové složky (0 az nSloz-2)
 * @return minimální reflux (-1 divergence; -2 nesplněny startovní podmínky;<br>
 * -3 nesouhlasí zadané koncentrace s počtem složek směsi; 0 klíč. složka mimo
 */      
    static public double minRefluxUnd(Smes sm, double p, double t, 
            double vfFeed,double[] xFeed,double[] xDest, int key)  {
        
        int i, iter, itmax;
        double alfaMin = +9999, d,  f, f1, s, theta, theta0, theta1;
        int nSloz = sm.getnSloz();
        if(xFeed.length != nSloz || xDest.length != nSloz) {
            JOptionPane.showMessageDialog(null, "Min.reflux Und. - nesouhlasí zadané koncentrace s počtem složek směsi!", 
                    "minRefluxUnd static", JOptionPane.ERROR_MESSAGE);
            return -3;
        }
        double[] alfa = new double[nSloz];
        if(key >= nSloz-1  || key < 0) return 0;
        iter = 0;
        itmax = 500;
        for(i = 0; i < nSloz; i++) {
           alfa[i] = sm.getSlozka(i).tenze(t)/ p;  // P0(i) / P
           alfaMin = Math.min(alfa[i], alfaMin);
        }                 
        for(i = 0; i < nSloz; i++){
            alfa[i] = alfa[i]/alfaMin;  // relativně k nejtěžší složce
        }
        // vlastní výpočet
        d = alfa[key] - alfa[key + 1];
        theta0 = alfa[key + 1] + 0.00000001 * d;
        s = vfFeed - 1;
        for(i = 0; i < nSloz; i++) {
           s = s + alfa[i] * xFeed[i] / (alfa[i] - theta0);
        }
        double f0 = s;
        theta = alfa[key] - 0.00000001 * d;
        s = vfFeed - 1;
        for(i = 0; i < nSloz; i++) {
           s = s + alfa[i] * xFeed[i] / (alfa[i] - theta);
        }
        f = s;
        if(f * f0 > 0) {
            return  -2; // Nesplněny startovní podmínky metody Regula Falsi!
        }
        do {
            theta1 = (f * theta - f0 * theta0) / (f - f0);
            s = vfFeed - 1;
            for(i = 0; i < nSloz; i++) {
               s = s + alfa[i] * xFeed[i] / (alfa[i] - theta1);
            }
            f1 = s;
            iter ++;
//            If Math.abs(theta - theta0) / theta < 0.0000005 And Math.abs(f1) < 0.00005 Or iter > itmax thetaen GoTo K
            if(f0 * f1 < 0) {
                theta = theta1; f = f1;
            }else {
                theta0 = theta1; f0 = f1;
            }
          } while(Math.abs(theta - theta0) / theta > 0.0000005 || Math.abs(f1) > 0.00005 && iter < itmax);          
          s = -1;        // výpočet zkonvergoval
          for(i = 0; i < nSloz; i++) {
             s = s + alfa[i] * xDest[i] / (alfa[i] - theta1);
          }
        if(iter <= itmax) {
          return s;  
        } else {
          return -1;     // překročet max.počet iterací
        }       
    }
/**
 * implementace abstraktní metody
 */
    @Override
    public void spoctiAparat() {
//        vypocetSB();
        proudySmithBrinkley();  // nepoužije návratové hodnoty
        spocten = true;
    }

    @Override
    public void spoctiAparat(boolean tisk) {
        boolean bylo = zobrazujVysledky;
        if(tisk)zobrazujVysledky = true; else zobrazujVysledky = false;
        proudySmithBrinkley();
        zobrazujVysledky = bylo;
        spocten = true;
    }

    
//  gettery a settery  -------------------------------------------------------    
/** 
 * Vrací celkový počet pater kolony
 * @return int počet pater
 */
    public int getnCelk() {
        return nCelk;
    }
/**
 * Nastavuje  celkový počet pater kolony
 * @param nCelk (int)
 */
    public void setnCelk(int nCelk) {
        this.nCelk = nCelk;
    }
/**
 * Vrací počet pater pod nástřikem
 * @return počet pater (int)
 */
    public int getnStrip() {
        return nStrip;
    }
/**
 * Nastaveje  počet pater pod nástřikem
 * @param nStrip (int)
 */
    public void setnStrip(int nStrip) {
        this.nStrip = nStrip;
    }
/**
 * Vrací poměr parní fáze v nástřiku (0-1)
 * @return 
 */
    public double getVfFeed() {
        return vfFeed;
    }
/**
 * Nastavuje  poměr parní fáze v nástřiku (0-1)
 * @param vfFeed 
 */
    public void setVfFeed(double vfFeed) {
        this.vfFeed = vfFeed;
    }
/**
 * Vrací refluxní poměr kolony
 * @return 
 */
    public double getReflux() {
        return reflux;
    }
/**
 * Nastavuje  refluxní poměr kolony
 * @param reflux 
 */
    public void setReflux(double reflux) {
        this.reflux = reflux;
    }
/**
 * Vrací poměr destilátu k nástřiku
 * @return 
 */
    public double getDestToFeed() {
        return destToFeed;
    }
/**
 * Nastavuje  poměr destilátu k nástřiku
 * @param destToFeed 
 */
    public void setDestToFeed(double destToFeed) {
        this.destToFeed = destToFeed;
    }
/**
 * Vrací složení destilátu (mol.zl.)
 * @return 
 */
    public double[] getxDest() {
        return xDest;
    }
/**
 * Nastavuje  složení destilátu (mol.zl.)
 * @param xDest 
 */
    public void setxDest(double[] xDest) {
        this.xDest = xDest;
    }
/**
 * Vrací složení zbytku (mol.zl.)
 * @return 
 */
    public double[] getxBot() {
        return xBot;
    }
/**
 * Nastavuje  složení zbytku (mol.zl.)
 * @param xBot 
 */
    public void setxBot(double[] xBot) {
        this.xBot = xBot;
    }
/**
 * Vrací nástřikový proud
 * @return Proud
 */
    public Proud getPr() {
        return pr;
    }
/**
 * Nastavuje nástřikový proud
 * @param pr 
 */
    public void setPr(Proud pr) {
        this.pr = pr;
    }
/**
 * Vrací proud destilátu
 * @return 
 */
    public Proud getPrDest() {
        return prDest;
    }
/**
 * Nastavuje proud destilátu
 * @param prDest 
 */
    public void setPrDest(Proud prDest) {
        this.prDest = prDest;
    }
/**
 * Vrací proud zbytku
 * @return 
 */
    public Proud getPrBot() {
        return prBot;
    }
/**
 * Nastavuje  proud zbytku
 * @param prBot 
 */
    public void setPrBot(Proud prBot) {
        this.prBot = prBot;
    }
    
}
