/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package cheminzlib;
import fydat.I_RKM;
import static fydat.Numer.m_RKM;
import fydatlib.Smes;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import org.jdom2.JDOMException;

/**
 * Třída aparátu trubkového reaktoru s výměbou tepla.<br> 
 * Spojuje několik proudů reaktantů, může probíhat více reakcí (k).<br>
 * Reakce jsou definovány seznamem přidaných instancí objektů Reakce,<br>
 * který zahrnuje stechiometrické koeficienty,<br> 
 * kdy koef. produktu je kladný, reaktantu záporný a inertu nulový.<br>
 * Parametry kinetiky reakce:  ko, To, E. Mocninová kinetika v koncentracích (mol/m3)<br>
 * Rychlostní konstanta se počítá jako: k = ko.exp[E(T-To)/R/T/To]<br>
 * Pokud se zadá referenční teplota To <= 0, pak  k= ko.exp[-E/R/T]<br>
 * Třída zahrnuje i výměnu tepla reakční směsi s lázní (koef. prostupu, výměnná plocha),<br>
 * počítá výstupní koncentrace a parametry
 * @author V.Hlubuček
 */
public class TrubReaktor extends SmesovacIdeal implements I_RKM {

    protected String VERZE = "1.2";
    protected int nSlozReak, nSloz;   // počet složek reakční směsi a pův. směsi
    protected int nSlozReagujicich;    // počet reagujících složek
    protected int nReakci = 0;        // počet reakcí
    /**
     * Režim reaktoru: 0-izotermní; 1-adiab.; 2-výměna tepla 
     */
    protected int rezim = 0;          // režim reaktoru: 0-izotermní; 1-adiab.; 2-výměna tepla
    protected int pocetTrubek;        // počet trubek v reaktoru
    protected int[] iRef;             // referenční indx pro konverzi j-té reakce
    protected int[] cislaReak;        // pole s čísly složek rakčního proudu
    protected int[] iPor;             // indexy (pořadí složky ve směsi) reagujících složek
    protected double[] k;             // rychlostní konstanty reakcí; 
    protected double objemReaktoru;   // objem reaktoru (m)
    protected double objemPrutok;     // objemový průtok (m3/s)    
    protected double prumerTrubky;    // průměr trubky (m)
    protected double delkaTrubky;     // délka trubky (m)
    protected double teplosmenPlocha; // teplosměnná plocha (m2)
    protected double koefProstupu;    // koeficient prostupu tepla (W/m2/K)
    /**
     * Reakční teplota (K)
     */    
    protected double t;               // reakční teplota (K)
    protected double tMax;            // maximální teplota v reaktoru (K)
    /**
     * Chladicí teplota (K)
     */    
    protected double t0;              // chladicí teplota (K)
    /**
     * Reakční tlak (Pa)
     */    
    protected double p;               // reakční tlak
    protected double hust, hust0;     // hustota reakční směsi (aktuální, počáteční)
    /**
     * Spec.teplo Cp směsi na vstupu a výstupu (J/kg/K)
     */    
    protected double cIn, cOut;       // spec.teplo směsi na vstupu a výstupu (J/kg/K)
    protected double x[];             // pomocné pole
    protected double xx[];            // konverze
    protected double sumKonst;        // suma hmot.zl.neragujících složek
    protected double[] c0,c;          // počáteční a konečná koncentrace složek
    protected double[] xMolPoc;       // počáteční molární zlomky
    protected double[] xHmPoc;        // počáteční hmotové zlomky
    /**
     * V/F poměr objemu V a objemového nástřiku F (s)
     */    
    protected double vf;              // poměr objemu V a objemového nástřiku F (s)
    protected Proud reakPr;                   // reakční proud
    protected Smes smReak;                    // reakční směs
    protected List<Reakce> listReakci = new ArrayList<Reakce>();      // seznam reakcí
    
/**
 * Konstruktor reaktoru
 * @param cApar číslo aparátu (int)
 * @param nazevAparatu textový název reaktoru
 */    
    public TrubReaktor(int cApar,String nazevAparatu) {
        super();
        pocetInProudu = 1;
        pocetOutProudu = 1;        
        this.cApar = cApar;
        this.nazevAparatu = nazevAparatu;
        myInit();
    }
/**
 * Bezparametrový konstruktor
 */    
    public TrubReaktor() {
        super();
        pocetInProudu = 1;
        pocetOutProudu = 1;        
        nazevAparatu = "Trubkový reaktor";
        myInit();
    }
    
/**
 * Inicializace aparátu
 * popis labelů a default hodnot pro interaktivní zadávání
 */
    public void myInit() {
        lTexty = new String[12];
        tValue = new String[12];
        lTexty[0] = "Trubkový reaktor";
        lTexty[1] = "Č.aparátu";
        lTexty[2] = "Název aparátu";
        lTexty[3] = "Počet In proudů";
        lTexty[4] = "Počet Out proudů";
        lTexty[5] = "Délka trubek (m)";
        lTexty[6] = "Počet trubek";
        lTexty[7] = "Průměr trubek (m)";
        lTexty[8] = "Koeficient prostupu (W/m2/K)";
        lTexty[9] = "Chladicí teplota (°C)";
        lTexty[10] = "Režim reaktoru (0,1,2)";
        lTexty[11] = ""; // musí být jako ukončovač
        if(pocetTrubek > 0) {        
            tValue[1] = String.format("%3d",cApar);
            tValue[2] = nazevAparatu;
            tValue[3] = String.format("%2d",pocetInProudu);
            tValue[4] = String.format("%2d",pocetOutProudu);
            tValue[5] = String.format("%5.3f",delkaTrubky);
            tValue[6] = String.format("%4d",pocetTrubek);
            tValue[7] = String.format("%4.3f",prumerTrubky);
            tValue[8] = String.format("%5.0f",koefProstupu);
            tValue[9] = String.format("%4.1f",t0);
            tValue[10] = String.format("%4d",rezim);            
        } else {
            tValue[1] = "0";
            tValue[2] = "Trubkový reaktor";
            tValue[3] = "1";
            tValue[4] = "1";
            tValue[5] = "4";
            tValue[6] = "100";
            tValue[7] = "0,035";
            tValue[8] = "200";
            tValue[9] = "0";                        
            tValue[10] = "2";                        
        }
    }
/**
 * Dosadí do proměnných hodnoty z interaktivního zadání
 */    
    public void dosadInteraktivniHodnoty() {
        cApar = (int)doubleFromString(tValue[1]);
        nazevAparatu = tValue[2];
        pocetInProudu = (int)doubleFromString(tValue[3]);
        pocetOutProudu = (int)doubleFromString(tValue[4]);
        delkaTrubky = doubleFromString(tValue[5]);
        pocetTrubek = (int)doubleFromString(tValue[6]);
        prumerTrubky = doubleFromString(tValue[7]);
        koefProstupu = doubleFromString(tValue[8]);
        t0 = (int)doubleFromString(tValue[9]);
        rezim = (int)doubleFromString(tValue[10]);        
    }    
    
/**
 * Přidá reakci do seznamu reakcí reaktoru
 * @param r objekt reakce
 */    
    public void pridejReakci(Reakce r) {
        listReakci.add(r);
        nReakci++;
    }
    
/**
 * Odebere reakci ze seznamu reakcí reaktoru
 * @param r objekt reakce
 */    
    public void odeberReakci(Reakce r) {
        listReakci.remove(r);
        nReakci--;
    }
/**
 * implementace abstrakní metody
 */    
    @Override
    public void spoctiAparat() {
        reakce();
    }
/**
 * implementace abstraktní metody s volbou zobrazení
     * @param tisk boolean
 */     
    @Override
    public void spoctiAparat(boolean tisk) {
        boolean bylo = zobrazujVysledky;
        if(tisk)zobrazujVysledky = true; else zobrazujVysledky = false;
        reakce();
        zobrazujVysledky = bylo;
    }    
    
/**
 * Metoda vyčíslí průběh reakce v reaktoru a v reakčním (výstupním) proudu
 * poskytné zreaagovanou směs.
 */    
    public void reakce() {
        
        int i, j;
        // kontrola zadání
        if (proudy == null) {
            JOptionPane.showMessageDialog(null,
                "Zadání proudů reaktoru č. " + cApar + "  " + nazevAparatu + "\nnení úplné. Končím!",
                 "CSTR Reaktor - reakce", JOptionPane.WARNING_MESSAGE);            
        }
        reakPr = smisitProudy(false);    // poskytne výchozí proud s reaktanty, nezobrazí
        // vytvořit složky vzniklé reakcemi, pokud ještě nebyly ve směsi obsaženy
        nSlozReak = reakPr.getSmes().getnSloz();
        cislaReak = new int[nSlozReak+5];
//        cislaReak = reakPr.getSmes().getCislaSlozek();
        for (i = 0; i < nSlozReak; i++) cislaReak[i] = reakPr.getSmes().getCisloSlozky(i);
        for (Reakce rr : listReakci) {
            int[] rSl = rr.getiSlozky();    // čísla látek, účastnící se reakce
            for (i = 0; i < rSl.length;i++) {
                if (neniPrvkem(Math.abs(rSl[i]), cislaReak)) {  // není ještě v reakčních složkách
                    if (rr.getiStech()[i] > 0) {                     
                      cislaReak[nSlozReak] = Math.abs(rSl[i]);   // přidání složek produktů 
                      nSlozReak++;
//                        System.out.println("i,slozka, cisReak: "+i+"  "+rSl[i]+"  "+cislaReak[nSlozReak]);
                    } else {
                      JOptionPane.showMessageDialog(null,
                        "Reakční složka č. " + rSl[i] + ";  " + nazevAparatu +
                        "\nnení v zadaných proudech. Končím!",
                         "CSTR Reaktor - reakce", JOptionPane.WARNING_MESSAGE);                       
                    }
                }
            }
        }

        nSloz = reakPr.getSmes().getnSloz();  // počet složek dosavadní směsi
        x = new double[nSlozReak];  // deklarace pro pořebný počet složek
        for (i = 0; i < nSloz;i++) {
            x[i] = reakPr.getSmes().getxMol(i);     // složení původní směsi
        }
        for (i = nSloz; i < nSlozReak; i++) {
            x[i] = 0;           // nulová počáteční koncentrace produktů
        }       
        try {
            // vytvoření nové reakční směsi se zahrnutím produktů
            smReak = new Smes(nSlozReak,cislaReak);
            smReak.zadatSloz(false, x);
            reakPr.setSmes(smReak);     // v reakčním proudu nastavena směs před reakcí
            smReak.setNeMistniHlasky(true); // Potlačení zobrazování chybových hlášek v okně
        } catch (JDOMException ex) {
            Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);                 
        } catch (IOException ex) {
            Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);
        }
        xHmPoc = smReak.getxHmot();     // počáteční hmotové zlomky
        xMolPoc = smReak.getxMol();
        // vlastní průběh reakce
        t = reakPr.gettVstup() + TABS;   // absolutní teplota
        p = reakPr.getpVstup();
        hust = smReak.sHust(t, p);    // hustota reakční směsi
        hust0 = hust;
        cIn = smReak.sCpW(t, p);      // specifické teplo směsi  
        objemPrutok =  reakPr.getHmotPrutok()/hust;
        objemReaktoru = pocetTrubek * Math.PI * prumerTrubky * prumerTrubky / 4;
        vf = objemReaktoru/objemPrutok;
        c = new double[nSlozReak];
        c0 = new double[nSlozReak];
        for (i = 0; i < nSlozReak; i++) {
            c0[i] = cKonc(i);       // počáteční koncentrace složek
        }
        c = c0.clone();             // koncentrace aktuální
        // počet reagujících složek
        HashSet<Integer> reagSl = new HashSet();    // aby byl seznam jedinečných složek
        for (Reakce r : listReakci) {
            report += r.getNazevReakce() + String.format("%nk0= %6.2e  t0(K)= %4.1f  Eakt(kJ/mol)= %5.2f  dH(kJ/mol)= %6.2f%n", 
                    r.getK0(),r.getT0(),r.geteAkt()/1000,r.getdH()/1000);
            for (i = 0; i < r.getiKinSloz().length;i++) {
               report += String.format("Sl. = %3d  řád= %4.2f%n", r.getiKinSloz()[i],r.getRadSlozky()[i]);
            }
            for (j = 0; j < r.getiSlozky().length; j++) {
                int jj;
                jj = r.getiSlozky()[j];
                if(jj != 0) {
                    reagSl.add(jj);  // přidají se jen dosud neexistující a nenulové prvky (Hash set)
                    report += String.format("Sl.reak = %3d     %s%n",jj, reakPr.getSmes().getNazevSlozky(porCisloLatky(Math.abs(jj)))); 
                }
            }
            report += String.format("(Kladně produkty, záporně reaktanty)%n");
        }
        iPor = new int[reagSl.size()];    // pole s indexy neznámých (reagujících složek)
        nSlozReagujicich = reagSl.size();        // počet složek vstupujících do reakcí
        i = 0;
        for (Integer ii : reagSl) {
            iPor[i] = porCisloLatky(Math.abs(ii));
//            System.out.println("i, nezn. "+i+"   "+iPor[i]);
            i++;

        }
        reakPr.setVysledek(report);
        // řešení soustavy
//        RKMRightSide f = this;
//        try { 
            x = new double[nSlozReagujicich];
//            iRef = -999;
            double s = 0;
            for (i = 0; i < nSlozReagujicich;i++ ) {
                x[i] = smReak.getxHmot(iPor[i]);
                s += x[i];
//            System.out.println("i,c0,xMolPoc: "+iPor[i]+"  "+c0[iPor[i]]);                
            }
            sumKonst = 1 - s;   // suma nereagujících hm. zlomků
            int nProm = rezim > 0 ? nReakci+1:nReakci;                         
            xx = new double[nProm];
            int[] pom = new int[nProm];            
            for (i = 0; i < nProm; i++) {  // nastavení neznámé pro Newtona
                pom[i] = i;
                xx[i] = 0.0;   // počáteční konverze
            }
            if (nProm > nReakci) xx[nReakci] = reakPr.tVstup + TABS;   // počáteční teplota (K)
            int ii = 0, kk;     // výběr referenčních složek pro reakce
            s = 9999; 
            iRef = new int[nReakci];
            for (Reakce r:listReakci) {
//              Zjištění referenčních složek pro konverzi reakcí (ty se stech. nejnižší poč. koncentrací)
                for (int k = 0; k < r.getiKinSloz().length;k++) {
                    kk = porCisloLatky(r.getiKinSloz()[k]); 
                    if (smReak.getxHmot(kk)/Math.abs(stechKoef(kk,0)) < s) i = kk; s = smReak.getxHmot(kk)/Math.abs(stechKoef(kk,0));
                }
                iRef[ii] = i;
            }

//            rezim = 1;      // nuceně adiabatický ------------------
            rezim = 2;      // výměna tepla ------------------
        reakPr.setpVystup(p);   // výstupní tlak reakčního proudu (Pa)
        if (rezim >0) {
           reakPr.settVystup(xx[rezim >0 ? nReakci: nReakci-1]*reakPr.gettVstup());
        } else{
           reakPr.settVystup(reakPr.gettVstup());
        }
        report += "Výsledky výpočtu trubkového reaktoru  (Verze " + getVERZE() + ")\n";
        report += String.format("Hmotový průtok (kg/s) = %7.4f  Objem reaktoru (m3) = %7.3f%n",
                reakPr.getHmotPrutok(), objemReaktoru);
        report += String.format("Objemový průtok (m3/s) = %8.6f%n",objemPrutok);
        report += String.format("Hustota směsi (kg/m3) = %5.1f%n",hust);
        report += String.format("Délka trubky (m) = %5.2f   Průměr trubky (m) = %5.3f%n",delkaTrubky, prumerTrubky);
        report += String.format("Počet rubek  = %5d%n",pocetTrubek);
        report += "Režim " + ((rezim == 0)? "adiabatický" : ((rezim == 1) ? "adiabatický": "výměna tepla"))+"\n";
        if (rezim == 2) {
            report += String.format("Koef. prostupu (W/m2/K) =%5.0f  Výměnná plocha (m2) = %5.2f%n",
                    koefProstupu,teplosmenPlocha);
        }
        report += String.format("Teplota vstup (°C,K)= %4.1f  %4.1f%n"
                + "Teplota chlazení (°C) = %4.1f%nTlak(kPa)= %7.2f%n"
                + "Bod varu vstupní směsi (°C) = %4.1f%n", 
                reakPr.gettVstup(),reakPr.gettVstup()+TABS,t0,
                reakPr.getpVystup()/1000,smReak.sTbv(p)-TABS); 
        reakPr.xHmot = reakPr.sm.getxHmot();        
        reakPr.xMol = reakPr.sm.getxMol();
        reakPr.nazProudu = "Vstup do " + nazevAparatu;
        reakPr.setVysledek(report);
        proudyIn.clear();       // vynulovat vstupy ze směšovače
        proudyIn.add(reakPr.kopirujProud(reakPr));   // vložení smíšeného proudu reaktantů do vstupního proudu        
        // Integrace profilů  ------------------------------------------
            boolean st = true;
            double z = 0, dz;
            double[] y = new double[nReakci + 2];   // teplota a tok
            double[] zz = new double[5];
            dz = delkaTrubky/10;  dz = 0.2;
            zz[0] = pocetTrubek * Math.PI * prumerTrubky * prumerTrubky/4/reakPr.getHmotPrutok();   // konstanta integrace
            zz[1] = pocetTrubek * Math.PI * prumerTrubky * koefProstupu/reakPr.getHmotPrutok();   // konstanta
            zz[2] = zz[1]*reakPr.getHmotPrutok()/1000;    // konstanta pro rovnici toku(kW)
            if(rezim == 1) zz[1] = 0;   // nuceně adiabatický režim
            for (int jj = 0; jj < nReakci; jj++) y[jj] = c[iRef[jj]];   // ref. koncentrace
            t = reakPr.gettVstup() + TABS;
            tMax = t;
            y[nReakci] = reakPr.tVstup + TABS;     // vstupní teplota do reaktoru (K)
            y[nReakci + 1] = 0;     // počáteční nulový tok(kW) vyměněného tepla
            report += "Profil koncentrací (mol/m3) podél reaktoru\n";
            report += String.format("z(m)  t(°C)");
            for (i = 0; i < nSlozReak; i++) {
              report += String.format("%12s",reakPr.getSmes().getNazevSlozky(i));
            }
            report += String.format("  Reak r.%n");
            while (z  < delkaTrubky + dz) {   // ----------------  integrační cykl  --------------
              report += String.format("%4.2f   %4.1f",z, y[nReakci] - TABS);  
              for (i = 0; i < nSlozReak; i++)
              {
                  report += String.format("  %6.4e",c[i]); 
              }
//              report += String.format(" %6.1f%n", y[2]); 
              report += String.format(" %8.4g%n", Math.abs(zz[4])); 
              if(hust0 > 500 && y[nReakci] > smReak.sTbv(p)) {     // y[nReakci] je abs. teplota v reaktoru
                report += "*** Došlo k odpaření kapalné reakční směsi! Havarijní stav.\n" ;
                break;
              }
              if(z + 1e-3 >= delkaTrubky) break;
              m_RKM(this,nReakci + 2, z, z + dz, y, 0.5, st, zz); // volání integrace)
              z += dz;
              st = false;   // překlopení stratovního parametru
              if(y[nReakci]>tMax)tMax = y[nReakci]; // max. teplota v reaktoru
              objemPrutok =  reakPr.getHmotPrutok()/hust;              
            }               //  -------------------------------------------------------------
//        System.out.println("Koncové z= "+z);
                
        
        ii = 0;
        reakPr.tVystup = y[nReakci] - TABS;
        reakPr.pVystup = p;
        for (Reakce r: listReakci) {
            report += String.format("Konverze %1d vůči  %s    %7.5f%n",ii+1,reakPr.getSmes().getNazevSlozky(iRef[ii]),xx[0]);
            ii++;
        }
        report += String.format("Maximální teplota v reaktoru (°C) = %5.1f%nBod varu výsledné směsi(°C)= %5.1f%n"
                +"Hustota výsledné směsi (kg/m3)= %5.1f%n",tMax - TABS, smReak.sTbv(p) -TABS,smReak.sHust(t, p));
        report += "Složka           Název   c0(mol/m3)   c (mol/m3)   x0 hmot  x hmot   x0 mol   x mol\n";
        for (i = 0; i < nSlozReak; i++) {
            report += String.format(" %3d  %16s   %6.4e   %6.4e  %7.4f  %7.4f  %7.4f  %7.4f%n",
                    reakPr.getSmes().getCisloSlozky(i),
                    reakPr.getSmes().getNazevSlozky(i),c0[i],c[i],
                    xHmPoc[i],smReak.getxHmot(i),xMolPoc[i],smReak.getxMol(i));
        }
        double pom1 = objemReaktoru/objemPrutok;
        report += String.format("Doba zdržení(s, min)= %6.1f  %4.2f%n",pom1,pom1/60);
        report += String.format("Objemový průtok výstup (m3/s) = %8.6f%n",objemPrutok);
        s = 0;
        for (i = 0; i < nReakci;i++) {  // změna tepla jednotlivými reakcemi
            s += xx[i]*c0[iRef[i]]*objemPrutok * listReakci.get(i).getdH();     // (W)
        }
        report += String.format("Uvolněné(-)/spotřebované(+) teplo reakcí(kW)= %7.5g%n",s/1000);
        cOut = reakPr.getSmes().sCpW(t, p);
        double pomd = reakPr.hmotPrutok *(reakPr.tVystup * cOut - reakPr.tVstup * cIn)/1000;  // teplo akumulované na výstupu
        report += String.format("Teplo akumulované v reakční směsi(kW)= %7.5g%n",(zz[1]>0)? pomd:-s/1000);
        report += String.format("Teplo vyměněné s lázní(kW)= %7.5g%n",(zz[1]>0)? -s/1000 - pomd:0);
        reakPr.spocten = true;
        reakPr.zUzlu = this;
        reakPr.xHmot = reakPr.sm.getxHmot();        
        reakPr.xMol = reakPr.sm.getxMol();
        reakPr.nazProudu = "Výstup z " + nazevAparatu;        
        reakPr.setVysledek(report);
        proudyOut.clear();
        proudyOut.add(reakPr);
        if(zobrazujVysledky) {
            new JDialogProud(null, nazevAparatu + "  výstup (TrubReaktor Verze " 
                        + VERZE + ")\n", reakPr); // modálně vyskakovací  okno s výsledky             
        }        
    }
/**
 * Pravé strany soustavy dif. rovnic pro RKM integraci
 * @param n počet integrovaných proměnných
 * @param z délková axiální souřadnice
 * @param y vektor neznámých koncentrací a teploty
 * @param f vektor pravých stran
 * @param zz vektor pomocných proměnných
 */
    public void fkt(int n, double z, double[] y, double[] f, double[] zz) {
//        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
      double sum, sumQ;       // sumář reakcí
      double s = 0;
      double x[] = new double[nSlozReak];
      x = xHmPoc.clone();
      for (int j = 0; j < nReakci; j++) {
        c[iRef[j]] = y[j];  // aktualizace referenčních koncentrací reakcí
      }
      if (rezim >0) {
        t = y[nReakci];      // abs. teplota v reaktoru (K)
//        System.out.println("fun z, c, t: " + z + "  " + y[0] + "  " + y[1]);
      } else {
        t =  reakPr.gettVstup() + TABS;  // izotermní režim
      }
      for (int i = 0; i < nSlozReagujicich; i++) {    // dopočet koncentrací     
          c[iPor[i]] = c0[iPor[i]];
          for (int j = 0; j < nReakci; j++) {   // xx[] konverze ref. složek
            c[iPor[i]] += xx[j]*c0[iRef[j]]*(double)stechKoef(iPor[i], j);
            c[iPor[i]] *= hust/hust0;       // korekce koncentrace na hustotu
            if(iPor[i] == iRef[j]) c[iPor[i]] = y[j]; 
          }
          if (c[iPor[i]] < 0) c[iPor[i]] = 0;   // pojistka na podtečení do záporných
          x[iPor[i]] = c[iPor[i]] * smReak.getMolHmotSlozky(iPor[i])/hust;  // hmot.zl.
          s += x[iPor[i]];       
      }
      for (int i = 0;i < nSlozReagujicich; i++){
          x[iPor[i]] = x[iPor[i]]/s*(1-sumKonst);   // normalizace vzhledem k inertům
      }
      try {
          smReak.zadatSloz(true, x); // redefinice směsi s aktuálním složením
      } catch (IOException ex) {
          Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);
      }
      hust = smReak.sHust(t, p);    // hustota reakční směsi 
      cOut = smReak.sCpW(t, p);      // specifické teplo výstupní směsi      
      double[] reak = new double[listReakci.size()];
            for (int j = 0; j < listReakci.size();j++) {
                reak[j] = rychlostReakce(j, c);    // rychlost j-té reakce (mol/s/m3)
            }
      sum = 0;
      sumQ = 0;
//      for (int i = 0; i < nSlozReagujicich; i++) {         
          for (int j = 0; j < listReakci.size();j++) {
              sum = sum + (double)stechKoef(iRef[j], j)* reak[j];
              sumQ += (double)stechKoef(iRef[j], j)* reak[j]*listReakci.get(j).getdH();
//              System.out.println("  fun j, stech, sum: "+iRef
//                      +"  "+stechKoef(iRef, j)+"  "+sum+"  reak="+reak[j]);
          }
     zz[4] = sum;       // rychlost reakcí (mol/m3/s)     
     for (int j = 0; j < nReakci + 1; j++) {
             f[j] = zz[0] * hust * sum;     // koncentrace referenční složky j-té reakce
         } 
             zz[3] = zz[0]  * sumQ / smReak.sCpW(t, p); // * smReak.sMolHmot()
             f[nReakci] = zz[3] - zz[1] / smReak.sCpW(t, p) * (t - t0 - TABS);// změna teploty (t0 v °C)
             zz[3] = zz[3] /(zz[3] - f[nReakci]);
             f[nReakci + 1] = zz[2] * (t - t0 - TABS);  // změna výměnného toku tepla
         
//          System.out.println("ff i="+j+"   " +f[j]+"   x="+y[j]+"  sumQ="+sumQ);
     if(z>0.13){
         int i = 5;
     }    
     for (int j = 0; j < nReakci; j++) {
//         xx[j] = (c0[iRef[j]] - c[iRef[j]])/c0[iRef[j]];   // aktualizované ref. konverze j-té reakce
         xx[j] = (c0[iRef[j]] - c[iRef[j]]*((hust0 > 500) ? (hust0/hust):1))/c0[iRef[j]];   // aktualizované ref. konverze j-té reakce (kor. hust.)
     }
    }    
/**
 * Vrací rychlost j-té reakce (mol/s/m3)
 * @param jReak pořadové číslo reakce
 * @param c[] vektor koncentrací (mol/m3)
 * @return hodnota reakční rychlosti 
 */  
  private double rychlostReakce(int jReak, double[] c) {
      double rychlost;      // rychlost reakce (mol/s/m3)
      Reakce reak;
      reak = listReakci.get(jReak);     // j-tá reakce
      rychlost = reak.rychlKonst(t);    // rychlostní konstanta,  t(K)
      for (int m = 0; m < reak.getiKinSloz().length; m++) {  // přes kinet. složky reakce
          int por = porCisloLatky(reak.getiKinSloz()[m]);   // pořadové číslo složky ve směsi
          if (por < 0) {
              JOptionPane.showMessageDialog(null,
                        "Látka č. " + reak.getiKinSloz()[m] + 
                        "\nnení v reakční směsi " + reakPr.getSmes().getNazev() +". Končím!",
                         "CSTR Reaktor - reakce", JOptionPane.WARNING_MESSAGE);
              return 0;
          }
          double cc; 
//          cc = cKonc(por);    // dopočet koncetrace z hmotového složení
          cc = c[por];       // koncentrace kinetické složky
          cc = (cc > 0) ? cc: 0;
          if (cc < 0) {
              rychlost = 0;
          }
          rychlost *= Math.pow(cc, reak.getRadSlozky()[m]);
      }
      return rychlost;
  }
/**
 * Vrací pořadové číslo látky cLatky ve směsi
 * @param cLatky číslo látky z báze
 * @return pořadové číslo ve směsi
 */  
  private int porCisloLatky(int cLatky) {
      int por = -9999;
      for (int i = 0; i < reakPr.getSmes().getnSloz(); i++) {
          if (cLatky == reakPr.getSmes().getCisloSlozky(i)) por = i;
      }
      return por;
  }
/**
 * Vrací stechiometrický koeficient i-té složky reakční směsi (produkty kladně)
 * v j-té reakci
 * @param iSl pořadové číslo složky ve směsi
 * @param jReak pořadové číslo reakce
 * @return stechiometrický koeficient (int)
 */
  private int stechKoef(int iSl, int jReak) {
      int stech = 0;
      int cSloz = reakPr.getSmes().getCisloSlozky(iSl);
      Reakce r = listReakci.get(jReak);
      for (int i = 0; i < r.getiStech().length; i++) {
//          System.out.println("** cSloz, Stechi: "+cSloz+"  "+r.getiSlozky()[i]);
          if (cSloz == Math.abs(r.getiSlozky()[i])) stech = r.getiStech()[i];
      }
      return stech;
  }
  private void normalizujReagujici(double[] x) throws IOException {
      int i;
      double s = 0;
      for (i = 0; i < nSlozReagujicich; i++) {s += x[i];}
      for (i = 0; i < nSlozReagujicich; i++) {x[i] = x[i] / s * (1 -sumKonst); }
      // dosadit do objektu směsi
      double[] xHm = new double[nSlozReak];
      for (i = 0; i < nSlozReak; i++) xHm[i] = smReak.getxHmot(i);
      for (i = 0; i < nSlozReagujicich; i++)  xHm[iPor[i]] = x[i];
      smReak.zadatSloz(true, xHm);
//      for (i = 0; i < nSlozReak; i++) System.out.println("* i,x: "+i+"  "+xHm[i]);
  }
 /**
  * Vrací koncentraci i-té složky směsi (mol/m3)
  * @param i pořadové číslo složky směsi
  * @return koncentrace
  */   
    public double cKonc(int i) {
        return smReak.sHust(t, p) * smReak.getxHmot(i)/smReak.getMolHmotSlozky(i);
    }
///**
// * Vrací hmotový zlomek i-té složky (1) vypočtený z koncentrace
// * @param i pořadové číslo složky směsi
// * @return hmotový zlomek
// */    
//    public double xHmotC(int i) {
//    return c[i]/smReak.sHust(t, p) * smReak.getMolHmotSlozky(i);
//}
/**
 * Vrací pořadový index složky pro pořadové číslo neznámé
 * @param ii pořadové číslo neznámé
 * @return index složky ve směsi
 */
    private int iPorInv(int ii) {
        int k = -9999;
        for (int i = 0; i < iPor.length;i++) {
            if (iPor[i] == ii) k = i;
        }
        return k;
    }
/**
 * Vrací průměr trubky reaktoru (m)
 * @return průměr (double)
 */
    public double getPrumerTrubky() {
        return prumerTrubky;
    }
/**
 * Nastavuje průměr trubky reaktoru (m)
 * @param prumerTrubky 
 */
    public void setPrumerTrubky(double prumerTrubky) {
        this.prumerTrubky = prumerTrubky;
        myInit();
    }
/**
 * Vrací délku trubky reaktoru (m)
 * @return délka
 */
    public double getDelkaTrubky() {
        return delkaTrubky;
    }
/**
 * Nastavuje délku trubky reaktoru (m)
 * @param delkaTrubky 
 */
    public void setDelkaTrubky(double delkaTrubky) {
        this.delkaTrubky = delkaTrubky;
        myInit();
    }
/**
 * Vrací počet trubek reaktoru
 * @return počet
 */
    public int getPocetTrubek() {
        return pocetTrubek;
    }
/**
 * Nastavuje počet trubek reaktoru
 * @param pocetTrubek 
 */
    public void setPocetTrubek(int pocetTrubek) {
        this.pocetTrubek = pocetTrubek;
        myInit();        
    }
    
/**
 * Vrací objem reaktoru (m3)
 * @return objem (double)
 */
    public double getObjemReaktoru() {
        return objemReaktoru;
    }

/**
 * Vrací objemový průtok (m3/s)
 * @return průtok
 */
    public double getObjemPrutok() {
        return objemPrutok;
    }
///**
// * Nastavuje objemový průtok (m3/s)
// * @param objemPrutok 
// */
//    public void setObjemPrutok(double objemPrutok) {
//        this.objemPrutok = objemPrutok;
//    }
/**
 * Vrací reakční proud
 * @return proud
 */
    public Proud getReakPr() {
        return reakPr;
    }
/**
 * Nastavuje reakční proud
 * @param reakPr 
 */
    public void setReakPr(Proud reakPr) {
        this.reakPr = reakPr;
    }
/**
 * Vrací teplosměnnou plochu (m2)
 * @return plocha
 */
    public double getTeplosmenPlocha() {
        return teplosmenPlocha;
    }
/**
 * Nastavuje teplosměnnou plochu (m2)
 * @param teplosmenPlocha 
 */
    public void setTeplosmenPlocha(double teplosmenPlocha) {
        this.teplosmenPlocha = teplosmenPlocha;
    }
/**
 * Vrací koeficient prostupu tepla (W/m2/K)
 * @return koeficient
 */
    public double getKoefProstupu() {
        return koefProstupu;      
    }
/**
 * Nastavuje koeficient prostupu tepla (J/m2/K)
 * @param koefProstupu 
 */
    public void setKoefProstupu(double koefProstupu) {
        this.koefProstupu = koefProstupu;
        myInit();          
    }
/**
 * Vrací teplotu chlazení (°C)
 * @return teplota
 */
    public double getT0() {
        return t0;
    }
/**
 * Nastavuje teplotu chlazení (°C)
 * @param t0 (°C)
 */
    public void setT0(double t0) {
        this.t0 = t0;
        myInit();          
    }
/**
 * Vrací tepelný režim reaktoru
 * 0-izotermní; 1-adiabatický; 2-výměna tepla
 * @return režim
 */
    public int getRezim() {
        return rezim;
    }
/**
 * Nastavuje tepelný režim reaktoru
 * 0-izotermní; 1-adiabatický; 2-výměna tepla
 * @param rezim 
 */
    public void setRezim(int rezim) {
        this.rezim = rezim;
        myInit();
    }
    
 /**
  * Nastavuje parametry přestupu tepla
  * @param plocha (m2)
  * @param koef (W/m2/K)
  * @param tChlaz (°C)
  */
    public void paramPrestupuTepla(double plocha, double koef, double tChlaz) {
        teplosmenPlocha = plocha;
        koefProstupu = koef;
        t0 = tChlaz;
        rezim = 2;      // výměna tepla
        if (koefProstupu < 0.01) rezim = 1;     // adiabatický
        myInit();
    }

    
}
