/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;

import javax.swing.JOptionPane;

/**
 * Třída popisuje chování elementu trubka, kterou proudí daný proud.
 * Především umožní vyčíslit tlakovou ztrátu a předat výsledky.
 * @author V. Hlubuček
 */
public class Trubka extends Element {

/**
 * Typ parního režimu (adiabatický (true), izotermní (false)
 */    
    protected boolean adiab;  // typ parního režimu
    protected double prumerTrubky, delkaTrubky, drsnost, vyskovyRozdil;
/**
 * Konstruktor trubky, který zadává číslo a název trubky
 * @param cProudu číslo proudu (int)
 * @param nazElementu textový název elementu
 */
    public Trubka(Proud pr, String nazElementu) {
        super();
        this.pr = pr;
        this.nazElementu = nazElementu; 
        drsnost = 0.0001;
        myInit();
    }
/**
 * Konstruktor trubky, který zadává číslo, název trubky, průměr trubky
 * @param cProudu číslo proudu (int)
 * @param nazElementu textový název elementu
 * @param prumerTrubky průměr trubky (m)
 */
    public Trubka(int cProudu, String nazElementu, double prumerTrubky) {
        super();
        this.pr = pr;
        this.nazElementu = nazElementu; 
        this.prumerTrubky = prumerTrubky;
        drsnost = 0.0001;
        myInit();        
    }
    public Trubka() {
        super();
        nazElementu = "Trubka";
        drsnost = 0.0001;
        myInit();        
    }
/**
 * Hromadné zadání hlavních parametrů trubky
 * @param cProudu číslo proudu (int)
 * @param nazevTrubky textový název trubky
 * @param prumerTrubky průměr trubky (m)
 * @param delkaTrubky délka úseku trubky (m)
 * @param vyskovyRozdil výškový rozdíl (m)
 * @param drsnost absolutní drsnost (m)
 * @param adiab true - adiabatický režim; false - izotermní režim
 */
    public void zadejParametryTrubky(Proud pr, String nazevTrubky, double prumerTrubky,
        double delkaTrubky, double vyskovyRozdil, double drsnost, boolean adiab ) {
        this.pr = pr;
        nazElementu = nazevTrubky;
        this.prumerTrubky = prumerTrubky;
        this.delkaTrubky = delkaTrubky;
        this.vyskovyRozdil = vyskovyRozdil;
        this.drsnost = drsnost;
        this.adiab = adiab;
    }
/**
 * Inicializace elementu trubka
 * popis labelů a default hodnot pro interaktivní zadávání
 */
    public void myInit() {
        lTexty = new String[6];
        tValue = new String[6];
        lTexty[0] = "Trubka";
        lTexty[1] = "Průměr trubky(m)";
        lTexty[2] = "Délka trubky(m)";
        lTexty[3] = "Drsnost";
        lTexty[4] = "Výškový rozdíl(m)";
        lTexty[5] = "";
        tValue[1] = "0,05";
        tValue[2] = "100";
        tValue[3] = "0,0005";
        tValue[4] = "0";
        
    }
/**
 * Dosadí do proměnných hodnoty z interaktivního zadání
 */    
    public void dosadInteraktivniHodnoty() {
        prumerTrubky = doubleFromString(tValue[1]);
        delkaTrubky = doubleFromString(tValue[2]);
        drsnost = doubleFromString(tValue[3]);
        vyskovyRozdil = doubleFromString(tValue[4]);
    }
 /**
  * Výpočet tlakové ztráty na trubce
  * @param pr proud
  * @return tlaková ztráta (Pa)
  */   
    @Override
    public double tlakZtrata() {
        int it = 0;
        boolean kapalina;
        double prurez = Math.PI * prumerTrubky * prumerTrubky/4;
        double d, dH, ff, h = 0, nRe = 0, p, p1 = 0, p2, p3, pp, ro= 0, ro1,  t, tad,  v = 0, v1;
        double acc, acc1, f,fi, k = 1.2, mmax, tt, pom, tlum, tlum1;
        tlum = 0.5; acc = 0.00002;
        d = prumerTrubky;
        dH = vyskovyRozdil;        
        t = tVstup+TABS;    // abs. teplota (K)
        p = pVstup;
        if (pVstup*tVstup <= 0 && pr.isTisk()) {
           JOptionPane.showMessageDialog(null,
              "V elementu " + nazElementu + " pr. (" + pr.cProudu + ") nulový tlak nebo teplota !",
              "Trubka", JOptionPane.WARNING_MESSAGE);            
            return -9999;   // chyba
        }        
        kapalina = false;
        if (t < pr.getSmes().sTbv(p)) kapalina = true;
        rychlVstup = pr.getHmotPrutok()/ pr.getSmes().sHust(t, p)/prurez;
        rychlVystup = rychlVstup;
        if (rychlVstup*rychlVystup < 0) {
           JOptionPane.showMessageDialog(null,
              "V trubce " + nazElementu + " pr. (" + pr.cProudu + ") je dvoufázový tok !",
              "Trubka", JOptionPane.WARNING_MESSAGE);            
//            return -9999;   // zatím
        }
//        kapalina = true;
        if (kapalina) {     // výpočet pro kapalnou fázi
            ro = pr.getSmes().sHustL(t);
            v = 4 * pr.getHmotPrutok() / ro / Math.PI / d / d;
            if (v > 5 && pr.isTisk()) {
               JOptionPane.showMessageDialog(null,
                  "Rychlos (m/s) " + String.format("%6.3f%n",v) + " Nereálný hmot. průtok pro daný průměr trubky !\n"
                  + "Element vynechán!",
                  "Trubka parní fáze", JOptionPane.WARNING_MESSAGE);  
            report = "Trubka (" + nazElementu + ") (g)\n";
            report += "!!! Příliš velká rychlos. Nereálný hmot. průtok pro daný průměr trubky! Element vynechán!\n";
            report += " hmotový průtok (kg/s) " + String.format("%7.4g%n",pr.getHmotPrutok());            
            report += " průměr potrubí    (m) " + String.format("%5.2f%n",prumerTrubky);
            report += " rychlost        (m/s) " + String.format("%6.3f%n",v);
               return 0;
            }                 
            nRe = v * d * ro / pr.getSmes().sViskL(t);
            h = hltr(nRe, d, drsnost);      // koeficient tření v trubce
            pVystup = p - 9.81 * ro * dH -  h * delkaTrubky / d * v * v * ro / 2;
            tVystup = tVstup;
            rychlVystup = v;
            rychlVstup = v;
            // výsledky
            report = "Trubka (" + nazElementu + ") (l)\n";
            report = report + " hmotový průtok (kg/s) " + String.format("%7.4g%n",pr.getHmotPrutok());
            report = report + " průměr potrubí    (m) " + String.format("%6.4f%n",prumerTrubky);
            report = report + " délka potrubí     (m) " + String.format("%5.2f%n",delkaTrubky);
            report = report + " výškový spád      (m) " + String.format("%5.2f%n",dH);
            report = report + " drsnost potrubí   (m) " + String.format("%7.5f%n",drsnost);
            report = report + " hustota       (kg/m3) " + String.format("%5.2f%n",ro);
            report = report + " teplota bodu varu(°C) " + String.format("%5.1f%n",pr.getSmes().sTbv(p)-TABS);
            report = report + " viskozita      (Pa.s) " + String.format("%6.4g%n",pr.getSmes().sViskL(t));
            report = report + " rychlost        (m/s) " + String.format("%6.3f%n",v);
            report = report + " Reynoldsovo č.    (1) " + String.format("%6.4g%n",nRe);
            report = report + " součinitel tření  (1) " + String.format("%6.4f%n",h);
            report = report + " teplota          (°C) " + String.format("%5.1f%n",tVstup);            
            report = report + " tlak na vstupu   (Pa) " + String.format("%7.0f%n",pVstup);
            report = report + " tlak na výstupu  (Pa) " + String.format("%7.0f%n",pVystup);
            report = report + " tlaková ztráta   (Pa) " + String.format("%6.1f%n",p - pVystup);
//            System.out.println(report);
        }
        if (!kapalina) {    // výpočet pro parní fázi
            tad = t;
            p1 = pVstup;
            ro1 = pr.getSmes().srHustGW(t, p1);
            v1 = 4 * pr.getHmotPrutok() / ro1 / Math.PI / d / d;
            if (v1 > 200 && pr.isTisk()) {
               JOptionPane.showMessageDialog(null,
                  "Rychlost (m/s) " + String.format("%6.3f%n",v1) + " Nereálný hmot. průtok pro daný průměr trubky !\n"
                  + "Element vynechán!", 
                  "Trubka parní fáze", JOptionPane.WARNING_MESSAGE);  
            report = "Trubka (" + nazElementu + ") (g)\n";
            report += "!!! Příliš velká rychlos. Nereálný hmot. průtok pro daný průměr trubky ! Element vynechán!\n";
            report += " hmotový průtok (kg/s) " + String.format("%7.4g%n",pr.getHmotPrutok());            
            report += " průměr potrubí    (m) " + String.format("%6.4f%n",prumerTrubky);
            report += " rychlost        (m/s) " + String.format("%6.3f%n",v1);
               return 0;
            }            
            p2 = p1 - 0.0015 * delkaTrubky * v1 * v1 * ro1 / d;
            ff = p1 / 2 / ro1 / v1 / v1;       // bezrozm.par 
            p3 = p2 +100;
            while ((p3 - p2)/p2 > 0.00002) {
                p2 = p3;
                if (p2 < 0) p2 = 0.03 * p1;   // pojistka
                pp = (p1 + p2) / 2;
                it++;
                ro = pr.getSmes().srHustGW(t, pp);
                v = 4 * pr.getHmotPrutok() / ro / Math.PI / d / d;                
                nRe = v * d * ro / pr.getSmes().srViskG(t, pp);
                h = hltr(nRe, d, drsnost);
                tt = h * delkaTrubky / d / 2;        // bezrozm.par
                if (!adiab) {    // izotermní režim
                  if (tt < 0.5) {
                    pom = 0.5 + 0.721 * Math.pow(tt, 1.0585) + 1.0307 * Math.pow(tt, 0.50383);
                  } else {
                    pom = 1.0304 + 1.1174 * Math.pow(tt, 0.98134);
                  }
                  mmax = Math.PI / 4 * d * d * p1 / 2 / v1 / pom ;
                  f = -Math.log(p2 / p1) + p1 / ro1 / v1 / v1 * ((p2*p2 / p1/p1) - 1) / 2;
                  f = f + 9.81 * vyskovyRozdil / v / v + h * delkaTrubky / 2 / d;
                  fi = -1 / p2 + p2 / p1 / ro1 / v1 / v1;
                } else {            // adiabatický režim
                  tad = t * Math.pow(p3 / p1, (k - 1) / k);  
                  k = pr.getSmes().sKapa(tad, pp);  
                  f = -Math.log(p2 / p1) / k + k / (k + 1) * p1 / ro1 / v1 / v1 * (Math.pow(p2 / p1,(k + 1) / k)  - 1);
                  f = f + 9.81 * vyskovyRozdil / v / v + h * delkaTrubky / 2 / d;
                  fi = -1 / p2 / k + Math.pow(p2 / p1, 1 / k)  / ro1 / v1 / v1;
                }
                if (v < 30) { tlum1 = tlum; } else { tlum1 = tlum / 5;}
                if (v < 50) { acc1 = acc; } else { acc1 = 0.00005; }
                p3 = p2 - tlum1 * f / fi;
                if (it > 100) {
                   JOptionPane.showMessageDialog(null,
                      "Diverguje rychlost (m/s) " + v + "  !",
                      "Trubka parní fáze", JOptionPane.WARNING_MESSAGE);  
                   break;
                }
            }
            pVystup = p3;
            tVystup = (adiab) ? tad-TABS : tVstup;
            rychlVstup = v;
            rychlVystup = v;
            // výsledky
            report = "Trubka (" + nazElementu + ") (g)\n";
            report = report + " hmotový průtok (kg/s) " + String.format("%7.4g%n",pr.getHmotPrutok());
            report = report + " průměr potrubí    (m) " + String.format("%6.4f%n",prumerTrubky);
            report = report + " délka potrubí     (m) " + String.format("%5.2f%n",delkaTrubky);
            report = report + " výškový spád      (m) " + String.format("%5.2f%n",dH);
            report = report + " drsnost potrubí   (m) " + String.format("%7.5f%n",drsnost);
            report = report + " hustota       (kg/m3) " + String.format("%6.3f%n",ro);
            report = report + " teplota bodu varu(°C) " + String.format("%5.1f%n",pr.getSmes().sTbv(p)-TABS);
            if (adiab) {
            report = report + " adiab.koef.       (1) " + String.format("%5.3f%n",k);
            }
            report = report + " viskozita      (Pa.s) " + String.format("%6.4g%n",pr.getSmes().srViskG(t,p));
            report = report + " rychlost        (m/s) " + String.format("%6.3f%n",v);
            report = report + " Reynoldsovo č.    (1) " + String.format("%6.4g%n",nRe);
            report = report + " součinitel tření  (1) " + String.format("%6.4f%n",h);
            report = report + " teplota          (°C) " + String.format("%5.1f%n",tVystup);            
            report = report + " tlak na vstupu   (Pa) " + String.format("%7.0f%n",pVstup);
            report = report + " tlak na výstupu  (Pa) " + String.format("%7.0f%n",pVystup);
            report = report + " tlaková ztráta   (Pa) " + String.format("%6.1f%n",p - pVystup);            
        }
        return pVstup - pVystup;   // tlaková ztráta
    }

    @Override
    public String vysledkyElement() {
        return report;
    }
/**
 * Vrací průměr trubky (m)
 * @return průměr trubky (m)
 */
    public double getPrumerTrubky() {
        return prumerTrubky;
    }
/**
 * Nastavuje průměr trubky (m)
 * @param prumerTrubky 
 */
    public void setPrumerTrubky(double prumerTrubky) {
        this.prumerTrubky = prumerTrubky;
    }
/**
 * Vrací délku trubky (m)
 * @return délka trubky
 */
    public double getDelkaTrubky() {
        return delkaTrubky;
    }
/**
 * Nastavuje délku trubky (m)
 * @param delkaTrubky 
 */
    public void setDelkaTrubky(double delkaTrubky) {
        this.delkaTrubky = delkaTrubky;
    }

    
/**
 * Vrací absolutní drsnost trubky (m)
 * @return 
 */
    public double getDrsnost() {
        return drsnost;
    }
/**
 * Nastavuje absolutní drsnost trubky (m)
 * @param drsnost 
 */
    public void setDrsnost(double drsnost) {
        this.drsnost = drsnost;
    }
/**
 * Vrací výškový rozdíl(m)
 * @return 
 */
    public double getVyskovyRozdil() {
        return vyskovyRozdil;
    }
/**
 * Nastavuje výškový rozdíl(m)
 * @param vyskovyRozdil 
 */
    public void setVyskovyRozdil(double vyskovyRozdil) {
        this.vyskovyRozdil = vyskovyRozdil;
    } 
/**
 * Vrací jestli je parní režim proudění v trubce adiabatický nebo izotermní
 * @return ano/ne
 */
    public boolean isAdiab() {
        return adiab;
    }
/**
 * Nastavuje jestli je parní režim proudění v trubce adiabatický nebo izotermní
 * @param adiab 
 */
    public void setAdiab(boolean adiab) {
        this.adiab = adiab;
    }
    
    
}
