/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package fydat;

import static fydat.FydatGUI.TABS;
import fydatlib.Smes;
import java.io.IOException;
import org.jdom2.JDOMException;

/**
 *  Třída pro vyhledání látky podle hodnoty vlastnosti
 * @author Hlubuček
 */
public class PodleVlastnosti implements Comparable {
    int ltk;            // číslo látky
    int nVl;            // číslo vlastnosti
    private double[] x=new double[1];
    double hodnVl, hodnota, odchylka, t, p;       // hodnota vlastnosti, požadovaná a odchylka
    Smes sm;
/**
 * Konstruktor třídy
 * @param ltk   - číslo látky
 * @param nVl   - číslo vlastnosti
 * @param hodnota   - zadaná hodnota vlastnosti
 * @param t - teplota (K)
 * @param p - tlak (Pa)
 */
    public PodleVlastnosti(int ltk, int nVl, double hodnota, double t, double p) {
        this.ltk = ltk;
        this.nVl = nVl;
        this.hodnota = hodnota;
        this.t = t;
        this.p = p;

        try {            
            sm = new Smes("Hodn",ltk);
            sm.setNeMistniHlasky(true);
            x[0]=1.0;
            sm.slozToMol(x);
            sm.odmaskujChybu(2); 
            hodnVl = tabelFce(sm, nVl);
//            System.out.println("hodn "+hodnVl);
            if (hodnVl == 0.0 || Double.isNaN(hodnVl)) {
                odchylka = -9999.0;
            }
            else {
                odchylka = Math.abs(hodnota - hodnVl);
            }
        }
        catch (JDOMException e) {
                e.printStackTrace();
        }
        catch (IOException e)   {
            e.printStackTrace();
        }
        switch (nVl) {
            case 0:
        }
    }
/**
 * Metoda nastavuje způsob srovnávání
 * @param Object o
 * @return 
 */
    public int compareTo(Object o) {
        try {
        double odchylkaO = ((PodleVlastnosti) o).odchylka;
        if (Double.isNaN(odchylkaO)) {
          return -1;
        }
//        System.out.println("comp "+sm.getSlozka(0).getCislo()+ " "+
//                sm.getSlozka(0).getNazev()+":"+this.odchylka+"  "+odchylkaO);
        if (this.odchylka > odchylkaO) return +1;
        if (this.odchylka == odchylkaO) return 0;
        if (this.odchylka < odchylkaO) return -1;        }
        catch (NullPointerException e) {
        //throw new UnsupportedOperationException("Not supported yet.");
            System.out.println("NULL "+sm.getSlozka(0).getCislo()+ " "+
                sm.getSlozka(0).getNazev()+":"+this.odchylka);
            return 0;
        }
        return 0;
    }

    @Override
    public String toString() {
        return String.format("%-10.4g", hodnVl) + " : "
                + String.format("%3d", sm.getSlozka(0).getCislo()) + " "
                + String.format("%-30s", sm.getSlozka(0).getNazev())
                + "  " + sm.getSlozka(0).getVzorec();
//                + " "+String.format("%-10.4g", odchylka) ;
    }
/**
 * Pomocná funkce pro tabelaci
 * @param sm
 * @param i
 * @return 
 */
    Double tabelFce(Smes sm, int i) {
        double f = 0;
        switch (i) {
            case 0:
                f = sm.sViskG(t);
                break;
            case 1:
                f = sm.srViskG(t, p);
                break;
            case 2:
                f = sm.sCpG(t);
                break;
            case 3:
                f = sm.srCpG(t, p);
                break;
            case 4:
                f = sm.sCpGW(t);
                break;
            case 5:
                f = sm.srCpGW(t, p);
                break;
            case 6:
                f = sm.srCvG(t, p);
                break;
            case 7:
                f = sm.srCvGW(t, p);
                break;
            case 8:
                f = sm.sKapa(t, p);
                break;
            case 9:
                f = sm.sVodG(t);
                break;
            case 10:
                f = sm.srVodG(t, p);
                break;
            case 11:
                f = sm.sHustGW(t, p);
                break;
            case 12:
                f = sm.srHustGW(t, p);
                break;
            case 13:
                f = sm.sHustL(t);
                break;
            case 14:
                f = sm.sHust(t, p);
                break;
            case 15:
                f = sm.sCpL(t);
                break;
            case 16:
                f = sm.sCpLW(t);
                break;
            case 17:
                f = sm.sCpW(t, p);
                break;
            case 18:
                f = sm.sVodL(t);
                break;
            case 19:
                f = sm.sVod(t, p);
                break;
            case 20:
                f = sm.sViskL(t);
                break;
            case 21:
                f = sm.sVisk(t, p);
                break;
            case 22:
                f = sm.sdHv(t);
                break;
            case 23:
                f = sm.sdHvW(t);
                break;
            case 24:
                f = sm.sSigma(t);
                break;
            case 25:
                f = sm.sTenze(t);
                break;
            case 26:
                f = sm.sTbv(p);
                break;
            case 27:
                f = sm.sTrb(p);
                break;
            case 28 :
                f = sm.sTbv(p) - TABS;
                break;
            case 29:
                f = sm.sTrb(p) - TABS;
                break;
            case 30:
                f = sm.srUG(t, p);
                break;
            case 31:
                f = sm.srUGW(t, p);
                break;
            case 32:
                f = sm.srHG(t, p);
                break;
            case 33:
                f = sm.srHGW(t, p);
                break;
            case 34:
                f = sm.srSG(t, p);
                break;
            case 35:
                f = sm.srSGW(t, p);
                break;
            case 36:
                f = sm.sLnFug(t, p);
                break;
            case 37:
                f = sm.sJTcoef(t, p);
                break;
            case 38:
                f = sm.sPrL(t);
                break;
            case 39:
                f = sm.sPrG(t, p);
                break;
            case 40:
                f = sm.sObjRozL(t);
                break;
            case 41:
                f = sm.sViskKinemL(t);
                break;
            case 42:
                f = sm.sViskKinemG(t, p);
                break;
            case 43:
                f = sm.sHL(t);
                break;
            case 44:
                f = sm.sHLW(t);
                break;
            case 45:
                f = sm.sSL(t);
                break;
            case 46:
                f = sm.sSLW(t);
                break;
            case 47:
                f = sm.sUL(t);
                break;
            case 48:
                f = sm.sULW(t);
                break;
            default:
                break;
        }
        return f;
    }

}
