/*
 * Package fydatlib
 * Knihovna metod pro fyzikální vlastnosti org. látek a technických plynů
 * a jejich směsí
 */
package fydatlib;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import org.jdom2.JDOMException;

/**
 * Třída vytváří směs chemických látek z báze FYDAT.<br>
 * Definuje složky směsi a jejich složení
 * Počítá fyziklání vlastnosti směsi pro zadanou teplotu(K) a tlak(Pa).<br>
 * Tuto třídu používají všechny uživatelské chemicko-inženýrské programy.
 * @author V.Hlubuček
 */
public class Smes {
/**
 * Modifikace 23.12.2013 (oprava interpolace l i g pro Vod a Visk)<br>
 * Modifikace 1.4.2014 (přidání funkce kopirujSmes)<br>
 * Modifikace 12.4.2014 (přidání funkce seradDleTekavosti)<br>
 * Modifikace 20.5.2014 (přidání chybového stavu)<br>
 * Uživatelský default název směsi
 */
    String nazev = "Smes";      // default název, uživatel může změnit
    /**
     * Počet složek smesi
     */    
    protected int nSloz = 0;          // počet složek
    /**
     * indikace chybového stavu: <0 indikuje chybový stav
     */
    protected int chyba = -3;          // <0 indikuje chybový stav (počátek)
    protected int[] iCisla;           // pole čísel složek
    /**
     * Pole s molárním složením směsi (mol. zlomky)
     */    
    protected double[] xMol;  // složení molární zlomky
    /**
     * Pole s hmotovým složením směsi (hmot. zlomky)
     */    
    protected double[] xHmot; // složení hmotové zlomky
    protected String slozeni; // textový report složení směsi
    /**
     * Seznam látek dané směsi
     */    
    protected List<Latka> slozky = new ArrayList();     // seznam látek dané směsi
    // proměnné pro termodynamické vlastnosti
    protected double tksm, pksm, vksm, zksm, den, urea, hrea, srea;
    protected double cprea, cvrea, cojot, cfln;
    protected boolean neMistniHlasky = false;     // potlačení místních chybových hlášek

    public Smes(int nSloz, int[] iCisla) throws IOException,JDOMException {
        this.nSloz = nSloz;
        this.iCisla = new int[nSloz];  // dynamické dimenzování
        init();
//        if (nSloz != iCisla.length && !neMistniHlasky) {  // vadí při interaktivním zadávání !!!!
//            int input = JOptionPane.showOptionDialog(null, "Nesouhlasí počet složek v konstruktoru a dimenze pole se složkami (" 
//                    + iCisla.length + ", " + nSloz + ")\nsměs: " + nazev, "FYDAT - Smes", JOptionPane.OK_CANCEL_OPTION, 
//                    JOptionPane.INFORMATION_MESSAGE, null, null, null);
//            // chyba = -2 Nebylo zadáno složení směsi
//            if(input == JOptionPane.OK_OPTION) System.exit(0);
//            nSloz = iCisla.length;
//            this.nSloz = nSloz;
//        }            
        for (int i = 0; i < nSloz; i++) {
            this.iCisla[i] = iCisla[i];     // pole čísel složek 
            slozky.add(Latka.getLatka(this.iCisla[i]));
        }
        if(nSloz > 0)odmaskujChybu(1);  // složky zadány
    }

    public Smes(int[] iCisla) throws IOException,JDOMException {
//        this(iCisla.length, iCisla);
        this.iCisla = new int[iCisla.length];   // dynamické dimenzování
        init();
        for (int i = 0; i < iCisla.length; i++) {
            if (iCisla[i] == 0 && i > 0) {
                break;
            }
            this.iCisla[i] = iCisla[i];     // pole čísel složek 
            this.nSloz = i;
            slozky.add(Latka.getLatka(this.iCisla[i]));
        }
        if(nSloz > 0)odmaskujChybu(1);  // složky zadány
    }

    public Smes(String nazev, int... iCisla) throws IOException,JDOMException {
        this.nazev = nazev;
        this.iCisla = new int[iCisla.length]; // dynamické dimenzování
        init();
        for (int i = 0; i < iCisla.length; i++) {
            if (iCisla[i] == 0  && i > 0) {
                break;
            }
            this.iCisla[i] = iCisla[i]; // pole čísel složek 
            this.nSloz = i+1;
//            System.out.println("" + Latka.getLatka(this.iCisla[i]).getNazev());
            slozky.add(Latka.getLatka(this.iCisla[i]));
        }
        if(nSloz > 0)odmaskujChybu(1);  // složky zadány
    }

/**
* Inicializuje načtení údajů o všech látkách přes třídu Latka
* @throws IOException
*/
    final void init() throws IOException,JDOMException {
        Latka.getLatka(1);
//        latkaList = Latka.latkaList;
    }
//    Gettery settery ------------------------------------------------
/**
 * Vrací složku daného čísla
 * @param index - číslo složky
 * @return Latka
 */
    public Latka getSlozka(int index) {
        return slozky.get(index);
    }
    /**
 * Vrací hmotový zlomek i-té složky
 * @return  xHmot[i]
 */
    public double getxHmot(int i) {
        return xHmot[i];
    }
/**
 * Vrací molový zlomek i-té složky
 * @return  xMol[i]
 */
    public double getxMol(int i) {
        return xMol[i];
    }
/**
 * Vrací počet složek směsi
 * @return
 */
    public int getnSloz() {
        return nSloz;
    }
/*
 * Vrací seznam složek směsi
 */
    public List<Latka> getSlozky() {
        return slozky;
    }
/*
 * Nastavuje seznam složek směsi
 */
    public void setSlozky(List<Latka> slozky) {
        this.slozky = slozky;
    }
/*
 *  Vrací pole s čísly složek
 * @return  int[] skutečná čísla složek
 */
    public int[] getCislaSlozek() {
        int[] pom = new int[nSloz];
        for (int i = 0; i < nSloz; i++) { pom[i] = iCisla[i]; }
        return pom;
    }
/**
 * Vrací číslo látky, která je i-tou složkou směsi
 * @param i pořadové číslo složky ve směsi
 * @return číslo látky
 */    
    public int getCisloSlozky(int i) {
        return iCisla[i];
    }
/**
 * Vrací molekulovou hmotnost i-té složky směsi (kg/mol)
 * @param i pořadové číslo složky ve směsi
 * @return molekulová hmotnost (kg/mol)
 */
    public double getMolHmotSlozky(int i) {
        return slozky.get(i).getMolHmot();
    }
/**
 * Vrací pole molárních zlomků směsi
 * @return molární zlomky
 */
    public double[] getxMol() {
        return xMol;
    }
/**
 * Vrací pole hmotových zlomků směsi
 * @return hmotové zlomky
 */
    public double[] getxHmot() {
        return xHmot;
    }
//   gettery  settery ------- konec ---------------------------------------    
/**
 * Zadání složení směsi
 * @param JeHmot boolean (true hmotové zlomky; false molové zlomky)
 * @param x double[] pole hodnot zadaného složení
 * @throws IOException (pokud se nenačtou data o látkách)
 */
    public void zadatSloz(boolean JeHmot, double[] x) throws IOException {
        int i;
        double s = 0, sh = 0, sm = 0;
        for(i = 0; i < x.length; i++) s += x[i];
        if(x == null || s == 0) {
            int input = JOptionPane.showOptionDialog(null, "Nebylo zadáno složení směsi:\n" 
                    + nazev + "\nChcete ukončit program?", "FYDAT - Smes", JOptionPane.OK_CANCEL_OPTION, 
                    JOptionPane.INFORMATION_MESSAGE, null, null, null);
            // chyba = -2 Nebylo zadáno složení směsi
            if(input == JOptionPane.OK_OPTION) System.exit(0);
            return;
        }
        if (nSloz != x.length && !neMistniHlasky) {
            int input = JOptionPane.showOptionDialog(null, "Nesouhlasí počet navolených složek a molových zlomků složení (" 
                    + x.length + ", " + nSloz + ")\nsměs: " + nazev, "FYDAT - Smes", JOptionPane.OK_CANCEL_OPTION, 
                    JOptionPane.INFORMATION_MESSAGE, null, null, null);
            // chyba = -2 Nebylo zadáno složení směsi
            if(input == JOptionPane.OK_OPTION) System.exit(0);
            return;            
        }
        xMol = new double[x.length];
        xHmot = new double[x.length];
        s = 0;
        for (i = 0; i < nSloz; i++) {
            sm += x[i] / slozky.get(i).getMolHmot();
            sh += x[i] * slozky.get(i).getMolHmot();
            s += x[i];
        }
        for (i = 0; i < nSloz; i++) {
            if (JeHmot) {   // zadány hmotové zlomky
                xMol[i] = x[i] / slozky.get(i).getMolHmot() / sm;
                xHmot[i] = x[i] / s;
            } else {        // zadány molové zlomky
                xHmot[i] = x[i] * slozky.get(i).getMolHmot() / sh;
                xMol[i] = x[i] / s;
            }
        }
        odmaskujChybu(2);       // složení zadáno
        naplnReportSlozeni();   // naplní textový report o složení
    }

    /**
     * Převádí zadané složení na molární zlomky<br>
     * Možno zadávat jako díly, normalizuje
     * @param x  pole zadávaného složení
     */
    public void slozToMol(double[] x) throws IOException {
        int i;
        double s, s1;
        if (nSloz != x.length && !neMistniHlasky) {
            JOptionPane.showMessageDialog(null, "Nesouhlasí počet navolených složek a molových zlomků složení ("
                    + slozky.size() + ", " + nSloz + ")", "FYDAT", 1);
        }
        xMol = new double[x.length];
        xHmot = new double[x.length];
        s = 0;
        s1 = 0;
        for (i = 0; i < nSloz; i++) {
            s += x[i] / slozky.get(i).getMolHmot();
            s1 += x[i];
        }
        for (i = 0; i < nSloz; i++) {
            xMol[i] = x[i] / slozky.get(i).getMolHmot() / s;
            xHmot[i] = x[i] / s1;
        }
        if (slozeni != null) {
            if (slozeni.equals("")) naplnReportSlozeni();
        }
    }

    /**
     * Převádí zadané složení na hmotové zlomky<br>
     * Možno zadávat jako díly, normalizuje
     * @param x  pole zadávaného složení
     */
    public void slozToHmot(double[] x) {
        int i;
        double s, s1;
        s = 0;
        s1 = 0;
        for (i = 0; i < nSloz; i++) {
            s += x[i] * slozky.get(i).getMolHmot();
            s1 += x[i];
        }
        for (i = 0; i < nSloz; i++) {
            xHmot[i] = x[i] * slozky.get(i).getMolHmot() / s;
            xMol[i] = x[i] / s1;
        }
        if (slozeni.equals("")) naplnReportSlozeni();        
    }
/**
 * Naplní textový report o složení směsi
 */    
    public void naplnReportSlozeni() {
         int i;
        String tx;
        if (xMol[0] <=0) {
            slozeni ="";
            return ;
        }
        tx = getNazev();
        tx += "\nDefinice a složení směsi               Stř.mol.hmotnost(kg/mol)= " + String.format("%-6.4f%n", sMolHmot());
        tx += "Pseudokritické veličiny:               Tkrs(K)= "
                + String.format("%-6.1f", sTksm()) + " Pkrs(MPa)= "
                + String.format("%-6.2f", sPksm() / 1e6) + " Vkrs(ml/mol)= "
                + String.format("%-6.2f%n", sVksm() * 1e6);
        tx += "Čís. Látka                     Vzorec          Molhm.  Tkrit   Pkrit  Vkrit   Složení  Složení\n";
        tx += "                                               kg/mol    K      MPa   ml/mol   Xhmot    Xmol\n";
        tx += "__________________________________________________________________________________________________\n";
        for (i = 0; i < nSloz; i++) {
            tx += String.format("%3d  %-25s %-15s %-6.4f  %-5.1f   %-5.2f  %-5.1f   %-6.4f   %-6.4f%n",
                    iCisla[i], getSlozka(i).getNazev(),
                    getSlozka(i).getVzorec(), getSlozka(i).getMolHmot(),
                    getSlozka(i).gettKr(), getSlozka(i).getpKr() / 1e6,
                    getSlozka(i).getvKr() * 1e6, getxHmot(i),getxMol(i));
        }
        tx += "__________________________________________________________________________________________________\n";
        
        slozeni = tx;
    }
    
//  Gettery a settery  -------------------------------------------------------    
/**
 * Vrací textový report složení směsi
 * @return report slozeni
 */
    public String getSlozeni() {
        return slozeni;
    }  
 
/**
 * Vrací název směsi
 * @return
 */
    public String getNazev() {
        return nazev;
    }
/**
 * Nastavuje název směsi
 * @param nazev
 */
    public void setNazev(String nazev) {
        this.nazev = nazev;
    }
/**
 * Vrací název i-té složky směsi
 * @return String
 */
    public String getNazevSlozky(int i) {
        return slozky.get(i).getNazev();
    } 
/**
 * Nastavuje název i-té složky směsi
 * @param nazev
 */
    public void setNazevSlozky(int i, String nazev) {
        slozky.get(i).setNazev(nazev);
    }    
/**
 * Vrací nastavení zobrazování místních hlášek
 * @return
 */
    public boolean isNeMistniHlasky() {
        return neMistniHlasky;
    }
/**
 * Nastavuje vypnutí místních hlášek
 * @param neMistniHlasky
 */
    public void setNeMistniHlasky(boolean neMistniHlasky) {
        this.neMistniHlasky = neMistniHlasky;
    }
/**
 * Vrací chybové stavy: 0 -OK; <0 chyby
 * @return int  chybový stav
 */
    public int getChyba() {
        return chyba;
    }
/**
 * Nastavuje chybový stav
 * @param chyba (int)
 */
    public void setChyba(int chyba) {
        this.chyba = chyba;
    }
/**
 * Nastaví příslušný bit chyby<br>
 * třeba zadávat k mocniny dvou (1,2,4)
 * @param k 
 */    
    public void nastavChybu(int k) {
        int n = Math.abs(chyba);
        chyba = -(n | k);
    }
/**
 * Odstraní příslušný chybový bit<br>
 * třeba zadávat k mocniny dvou (1,2,4)
 * @param k 
 */    
    public void odmaskujChybu(int k) {
        int n = Math.abs(chyba);
        boolean zap = false;
        if(chyba < 0) zap = true;
        chyba = n ^ k;  // operátor XOR
        if(zap) chyba = -chyba;
    }
/**  -----------  Funkce fyz. vlastností  -------------------------------
/**
 * Střední molekulová hmotnost směsi látek
 * @return mol.hm (kg/mol)
 */
    public double sMolHmot() {
        // střední molekulová hmotnostsměsi (kg/mol)
        int j;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        double s = 0;
        for (int i = 0; i < nSloz; i++) {
            s += xMol[i] * slozky.get(i).getMolHmot();
        }
        return s;
    }
/**
 * Měrná izobarická tepelná kapacita směsi kapalin
 * @param t abs. teplota (K)
 * @return Cp (J/mol/K)
 */
    public double sCpL(double t) {
        // Cp (l) směsi (J/mol/K)
        int j;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        double s = 0;
        for (int i = 0; i < nSloz; i++) {
            s += slozky.get(i).cpL(t) * xMol[i];
        }
        return s;
    }
/**
 * Měrná izobarická tepelná kapacita směsi kapalin
 * @param t abs. teplota (K)
 * @return Cp (J/kg/K)
 */
    public double sCpLW(double t) {
        return sCpL(t) / sMolHmot();  // (J/kg/K)
    }

/**
 * Měrná izobarická tepelná kapacita směsi plynů
 * @param t abs. teplota (K)
 * @return Cp (J/mol/K)
 */
    public double sCpG(double t) {
        // Cp (g) směsi (J/mol/K)
        int j;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        double s = 0;
        for (int i = 0; i < nSloz; i++) {
            s += slozky.get(i).cpG(t) * xMol[i];
        }
        return s;
    }
/**
 * Měrná izobarická tepelná kapacita směsi plynů
 * @param t abs. teplota (K)
 * @return Cp (J/kg/K)
 */
    public double sCpGW(double t) {
        return sCpG(t) / sMolHmot();  // (J/kg/K)
    }

/**
 * Počítá tepelnou kapacitu se stanovením typu fáze<br>
 * Pro t < Tbodu varu vrací tepelnou kapacitu kapaliny<br>
 * Pro t > Trosbodu varu vrací tepelnou kapacitu páry<br>
 * Při koexistenci obou fází vrací interpolovanou hodnotu 
 * @param t
 * @param p
 * @return hustota 
 */    
    public double sCpW(double t, double p) {
        double cp = -9999,tbv, trb; 
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tbv = sTbv(p); trb = sTrb(p);
        if (t < tbv) cp = sCpLW(t);
        if (t > trb) cp = srCpGW(t, p);
        if (t > tbv && t < trb) {      // dvě fáze, interpolace     
            double cpbv = sCpLW(tbv);
            cp = cpbv + (srCpGW(trb,p) - cpbv)/(trb - tbv)*(t - tbv);
        }        
        return cp;
    }    
    
/**
 * Tepelná vodivost směsi kapalin
 * @param t abs. teplota (K)
 * @return vodivost (W/m/K)
 */
    public double sVodL(double t) {
        // vodivost (l) směsi (W/m/K)
        int j;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        double s = 0, pom;
        for (int i = 0; i < nSloz; i++) {
            pom = slozky.get(i).vodL(t);
            s += xMol[i] * slozky.get(i).getMolHmot() / pom / pom;
        }
        return Math.sqrt(sMolHmot() / s);
    }
/**
 * Tepelná vodivost směsi plynů
 * @param t abs. teplota (K)
 * @return vodivost (W/m/K)
 */
    public double sVodG(double t) {
        // vodivost (g) směsi (W/m/K)
        int j, jk;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        double s = 0, aik, pom, ss = 0;
        for (int i = 0; i < nSloz; i++) {
            s = xMol[i];
            for (int k = 0; k < nSloz; k++) {
                if (i != k) {
                    pom = slozky.get(i).getMolHmot() / slozky.get(k).getMolHmot();
//                    aik = Math.pow((1 + Math.sqrt(slozky.get(i).vodG(t) / slozky.get(k).vodG(t)) / Math.pow(pom, 0.25)), 2);
                    aik = (1 + Math.sqrt(slozky.get(i).vodG(t) / slozky.get(k).vodG(t)) / Math.pow(pom, 0.25));
                    aik = aik * aik;
                    aik = 0.35355 * aik / Math.sqrt(1 + pom);
                    s += aik * xMol[k];
                }
            }
            ss += xMol[i] * slozky.get(i).vodG(t) / s;
        }
        return ss;
    }
    
/**
 * Počítá tepelnou vodivost se stanovením typu fáze<br>
 * Pro t < Tbodu varu vrací tepelnou vodivost kapaliny<br> 
 * Pro t > Trosbodu varu vrací tepelnou vodivost páry<br>
 * Při koexistenci obou fází vrací interpolovanou hodnotu
 * @param t
 * @param p
 * @return hustota 
 */    
    public double sVod(double t, double p) {
        double vod = -9999,tbv, trb; 
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tbv = sTbv(p); trb = sTrb(p);
        if (t < tbv) vod = sVodL(t);
        if (t > trb) vod = srVodG(t, p);
        if (t > tbv && t < trb) {       // dvě fáze, interpolace
            double vodbv = sVodL(tbv);
            vod = vodbv + (srVodG(trb, p) - vodbv)/(trb - tbv)*(t - tbv);
        }        
        return vod;
    }
    
/**
 * Hustota směsi kapalin
 * @param t abs. teplota (K)
 * @return hust (kg/m3)
 */
    public double sHustL(double t) {
        // hustota (l) směsi (kg/m3)
        int j, jk;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        double s = 0, tkps = 0, psi, tr, tt, pom = 1;
        for (int i = 0; i < nSloz; i++) {
            psi = 0;
            for (int k = 0; k < nSloz; k++) {
                psi += xMol[k] * slozky.get(k).getvKr();
            }
            psi = xMol[i] * slozky.get(i).getvKr() / psi;
            tkps += psi * slozky.get(i).gettKr();
        }
        tr = t / tkps;
        if (tr > 1) {
            tr = 0.8;    // empirická pojistka na nesourodé směsi
        }
        s = 0;
        for (int i = 0; i < nSloz; i++) {
            if (xMol[i] > 1e-6) {
                tt = slozky.get(i).gettKr() * tr;
                pom = slozky.get(i).hustL(tt);
                s += xMol[i] * slozky.get(i).getMolHmot() / pom;
            }
        }
        return sMolHmot() / s;
    }
/**
 * Počítá hustotu se stanovením typu fáze<br>
 * Pro t < Tbodu varu vrací hustotu kapaliny<br> 
 * Pro t > Trosbodu varu vrací hustotu páry<br>
 * Při koexistenci obou fází vrací interpolovanou hustotu 
 * @param t
 * @param p
 * @return hustota 
 */    
    public double sHust(double t, double p) {
        double hust = -9999, tbv, trb;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tbv = sTbv(p); trb = sTrb(p);        
        if (t < tbv) hust = sHustL(t);
        if (t > sTrb(p)) hust = srHustGW(t, p);
        if (t > tbv && t < trb) {       // dvě fáze, interpolace
            double robv = sHustL(tbv);
            hust = robv + (srHustGW(trb, p) - robv)/(trb - tbv)*(t - tbv);
        }
        return hust;
    }
/**
 * Tenze par směsi látek
 * @param t abs. teplota (K)
 * @return tenze (Pa)
 */
    public double sTenze(double t) {
        // Tenze par směsi (Pa)
        int j;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        double s = 0;
        for (int i = 0; i < nSloz; i++) {
            s += slozky.get(i).tenze(t) * xMol[i];
        }
        return s;
    }
/**
 * Výparné teplo směsi kapalin
 * @param t abs. tepolta (K)
 * @return dHv  (J/mol)
 */
    public double sdHv(double t) {
        // Výparné teplo směsi (J/mol)
        int j;
        double s = 0;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        for (int i = 0; i < nSloz; i++) {
            if (t <= slozky.get(i).gettKr()) {
                s += slozky.get(i).dHv(t) * xMol[i];
            }
        }
        return s;
    }

/**
 * Výparné teplo směsi kapalin
 * @param t abs. tepolta (K)
 * @return dHv  (J/kg)
 */
    public double sdHvW(double t) {
        return sdHv(t) / sMolHmot();  // (J/kg)
    }

/**
 * Povrchové napětí směsi kapalin
 * @param t abs. teplota (K)
 * @return sigma (N/m)
 */
    public double sSigma(double t) {
        // Povrchové napětí směsi (N/m)
        int j;
        double s = 0, pom1 = 0, pom2 = 1;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        for (int i = 0; i < nSloz; i++) {
            if (xMol[i] > 0) {
                pom2 = slozky.get(i).hustL(t);
                pom1 = Math.pow(slozky.get(i).sigma(t), 0.25);
            }
            s += xMol[i] * pom1 / pom2;
        }
        return Math.pow(sHustL(t) * s, 4);
    }
/**
 * Viskozita směsi plynů
 * @param t abs. teplota (K)
 * @return visk (Pa.s)
 */
    public double sViskG(double t) {
        // Viskozita (g) směsi (Pa.s)
        int j;
        double s = 0, s1 = 0, pom;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        for (int i = 0; i < nSloz; i++) {
            pom = Math.sqrt(slozky.get(i).getMolHmot());
            s += slozky.get(i).viskG(t) * xMol[i] * pom;
            s1 += xMol[i] * pom;
        }
        return s / s1;
    }
/**
 * Viskozita směsi kapalin (dynamická)
 * @param t abs. teplota (K)
 * @return visk (Pa.s)
 */
    public double sViskL(double t) {
        // Viskozita (l) směsi (Pa.s)
        int j;
        double s = 0, pom = 1;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        for (int i = 0; i < nSloz; i++) {
            if (xMol[i] > 1e-6) {
                pom = slozky.get(i).viskL(t) * slozky.get(i).getMolHmot() / slozky.get(i).hustL(t);
            }
            s += xMol[i] * Math.log(pom);

        }
        return Math.exp(s) * sHustL(t) / sMolHmot();
    }

/**
 * Počítá dynamickou viskozitu se stanovením typu fáze<br>
 * Pro t < Tbodu varu vrací viskozitu kapaliny<br> 
 * Pro t > Trosbodu varu vrací viskozitu páry<br>
 * Při koexistenci obou fází vrací interpolovanou hodnotu 
 * @param t
 * @param p
 * @return hustota 
 */    
    public double sVisk(double t, double p) {
        double visk = -9999,tbv, trb;  
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tbv = sTbv(p); trb = sTrb(p);
        if (t < tbv) visk = sViskL(t);
        if (t > trb) visk = srViskG(t, p);
        if (t > tbv && t < trb) {       // dvě fáze, interpolace
            double viskbv = sViskL(tbv);
            visk = viskbv + (srViskG(trb, p) - viskbv)/(trb - tbv)*(t - tbv);
        }        
        return visk;
    }
    
/**
 * Kinematická viskozita kapaliné směsi
 * @param t teplota (K)
 * @return kin. viskozita
 */
    public double sViskKinemL(double t) {
        return sViskL(t)/sHustL(t);
    }
/**
 * Kinematická viskozita plynné směsi
 * @param t teplota (K)
 * @return kin. viskozita
 */
    public double sViskKinemG(double t, double p) {
        return srViskG(t, p)/srHustGW(t,p);
    }
/**
 * Vrací pořadové číslo látky cLatky ve směsi (kolikátá složka)
 * @param cLatky číslo látky z báze
 * @return pořadové číslo ve směsi
 */  
  public int porCisloLatky(int cLatky) {
      int por = -9999;
      for (int i = 0; i < getnSloz(); i++) {
          if (cLatky == getCisloSlozky(i)) por = i;
      }
      return por;
  }    
/**
 * Podprogram výpočtu terodynamických vlastností směsi reálných plynů
 * @param t teplota (K)
 * @param p tlak (Pa)
 */
    protected void tevrp(double t, double p) {
        double[] zki = new double[20], vki = new double[20], omega = new double[20];
        double t0, t1, t2, f, f1, f2, e1, e2, h, h1, h2, ss1, ss2, omsm;
        double aos, bos, cos, dos, eos, fos, sos1, sos2, ru, p0, cpid, cvid, htnbv;
        double stpid, hsmst, htpid, riku, rikf, rikc, p1, p2, p9, x9, o1, o2, o3, o4, o5;
        double z, z1, z2, x1, q1, q2, q3, q4, qt, s, c, c1, c2, c3, c4, c5, c6, c7, c8, c9, d1, d2, d3;
        int i, i1, j, j2, npol, n;
        if(chyba < 0) return;     // chyby v zadání směsi
        if (Math.abs(p) < 0.000001) {
            p = 0.000001;  // kvůli Tabelaci
        }//        If Redl Then GoTo RED    ' --> požadován výpočet podle Redlich-Kwonga
        p = 0.000001 * p;   // převod ma MPa
        p0 = 0.101325;      // std. tlak
        aos = 0;
        bos = 0;
        cos = 0;
        dos = 0;
        eos = 0;
        fos = 0;
        ru = 8.314;
        omsm = 0;
        zksm = 0;
        tksm = 0;
        pksm = 0;
        vksm = 0;
//        Výpočet konstant směsi
//        n = nSloz;
        for (j = 0; j < nSloz; j++) {
            npol = slozky.get(j).vlast[3].vztah;  // stupeň polynomu bylo typ vztahu + 1
            //X(j) = rr(j)
            omega[j] = slozky.get(j).getAcent();       // acentrický faktor
            if (omega[j] <= 0) {
                // acentrický faktor výpočtem  om = -1. - log10(Pr) při Tr = 0.7
                omega[j] = -1.0 - Math.log10(slozky.get(j).tenze(0.7 * slozky.get(j).gettKr()) / slozky.get(j).getpKr());
            }
            zki[j] = 0.2905 - 0.085 * omega[j];
            vki[j] = zki[j] * ru * slozky.get(j).gettKr() / slozky.get(j).getpKr() / 0.000001;     //TK(j) / PK(j)
            if (npol >= 1) {
                aos += xMol[j] * slozky.get(j).vlast[1].a[0];
            }
            if (npol >= 2) {
                bos += xMol[j] * slozky.get(j).vlast[1].a[1];
            }
            if (npol >= 3) {
                cos += xMol[j] * slozky.get(j).vlast[1].a[2];
            }
            if (npol >= 4) {
                dos += xMol[j] * slozky.get(j).vlast[1].a[3];
            }
            if (npol >= 5) {
                eos += xMol[j] * slozky.get(j).vlast[1].a[4];
            }
            if (npol >= 6) {
                fos += xMol[j] * slozky.get(j).vlast[1].a[5];
            }
            omsm = xMol[j] * omega[j] + omsm;
        }
        for (i = 0; i < nSloz; i++) {
            for (j = 0; j < nSloz; j++) {
                tksm = xMol[i] * xMol[j] * Math.pow((Math.pow(vki[i], 0.333) + Math.pow(vki[j], 0.333)), 3)
                        * Math.sqrt(slozky.get(i).gettKr() * slozky.get(j).gettKr()) + tksm;
                vksm = xMol[i] * xMol[j] * Math.pow((Math.pow(vki[i], 0.333) + Math.pow(vki[j], 0.333)), 3) + vksm;
            }
        }
        vksm = vksm / 8;
        tksm = tksm / 8 / vksm;
        zksm = 0.2905 - 0.085 * omsm;
        pksm = zksm * ru * tksm / vksm;
        // Výpočet termodynamických vlastností ideálního plynu
        cpid = aos + bos * t + cos * t * t + dos * t * t * t + eos * t * t * t * t + fos * t * t * t * t * t;
        cvid = cpid - ru;
        sos1 = 0;
        sos2 = 0;
        hsmst = 0;
        stpid = 0;
        htpid = 0;
        for (i = 0; i < nSloz; i++) {            // cyklus přes složky
            t0 = slozky.get(i).gettNbv();    // standardní stav při teplotě norm.bodu varu složky!
//          If T0 < 0.01 Then MsgBox "Chybí údaj Tnbv " & Str(i) & ". složky"
            htnbv = slozky.get(i).dHv(t0) * xMol[i];     // dH při Tnbv
            hsmst = hsmst + htnbv;
            sos1 = sos1 + htnbv / t0;
            if (xMol[i] > 0.0) {
                sos2 += xMol[i] * Math.log(xMol[i]);
            }
            htpid += xMol[i] * (slozky.get(i).vlast[1].a[0] * (t - t0)
                    + slozky.get(i).vlast[1].a[1] * (t * t - t0 * t0) / 2
                    + slozky.get(i).vlast[1].a[2] * (t * t * t - t0 * t0 * t0) / 3
                    + slozky.get(i).vlast[1].a[3] * (t * t * t * t - t0 * t0 * t0 * t0) / 4
                    + slozky.get(i).vlast[1].a[4] * (t * t * t * t * t - t0 * t0 * t0 * t0 * t0) / 5
                    + slozky.get(i).vlast[1].a[5] * (t * t * t * t * t * t - t0 * t0 * t0 * t0 * t0 * t0) / 6);
            stpid += xMol[i] * slozky.get(i).vlast[1].a[0] * Math.log(t / t0);
//         ' STPID = STPID + B0S * (T - T0) + C0S * (T ^ 2 - T0 ^ 2) / 2# + D0S * (T ^ 3 - T0 ^ 3) / 3# + _
//         '               E0S * (T ^ 4 - T0 ^ 4) / 4# + F0S * (T ^ 5 - T0 ^ 5) / 5#
            for (j = 0; j < slozky.get(i).vlast[1].vztah ; j++) {    // bylo vz +1
//                j2 = j - 1;
                if (j > 0) {
                    stpid += xMol[i] * slozky.get(i).vlast[1].a[j] / j * (Math.pow(t, j) - Math.pow(t0, j));
                }
            }
        }
        htpid += hsmst;
        stpid += -ru * Math.log(p / p0) + sos1 - ru * sos2;
//     Výpočet stavového chování podle LEE-KESLERA
        i1 = 0;
        t1 = t / tksm;
        c2 = 0.118119 - 0.265728 / t1 - 0.15479 / t1 * t1 - 0.030323 / t1 * t1 * t1;
        c3 = 0.0236744 - 0.0186984 / t1;
        c4 = (0.155488 + 0.623689 / t1) * 0.0001;
        c5 = 0.042724 / t1 * t1 * t1;
        p1 = p / pksm;
        d1 = p1 / t1;
//175:
        do {
            c1 = Math.exp(-0.060167 * d1 * d1);
            z1 = 1.0 + c2 * d1 + c3 * d1 * d1 + c4 * d1 * d1 * d1 * d1 * d1;
            z1 = z1 + c5 * d1 * d1 * (0.65392 + 0.060167 * d1 * d1) * c1;
            p2 = z1 * t1 * d1;
            q1 = 1.0;
            q2 = 1.0;
            i1++;
            q1 = (1.96176 + 0.222146 * d1 * d1 - 0.0072401 * d1 * d1 * d1 * d1) * c1 * d1 * d1 * c5;
            q1 = 1.0 + q1 + 2.0 * c2 * d1 + 3.0 * c3 * d1 * d1 + 6.0 * c4 * d1 * d1 * d1 * d1 * d1;
            d3 = (p1 - p2) / q1;
            if (Math.abs(d3)< 0.000005 || i1 > 35) break;
            if (Math.abs(d3) >= 0.1 * d1) {
                d3 = Math.abs(d3) / d3 * d1 / 10;
                d1 = d1 + d3;
            }
        } while (true);
//        GoTo 175
//200:
        o2 = 0.202658 - 0.331511 / t1 - 0.027655 / t1 * t1 - 0.203488 / t1 * t1 * t1;
        i1 = 0;
        o3 = 0.0313385 - 0.0503618 / t1 + 0.016901 / t1 * t1 * t1;
        o4 = (0.48736 + 0.0740336 / t1) * 0.0001;
        o5 = 0.041577 / t1 * t1 * t1;
        d2 = 1.4 * d1;
//210:
        do {
            o1 = Math.exp(-0.037542 * d2 * d2);
            z2 = 1.0 + d2 * o2 + o3 * d2 * d2 + o4 * d2 * d2 * d2 * d2 * d2;
            z2 = z2 + o5 * d2 * d2 * (1.226 + 0.03754 * d2 * d2) * o1;
            p2 = z2 * t1 * d2;
            i1++;
            q2 = (3.678 + 0.09565 * d2 * d2 - 0.0028185 * d2 * d2 * d2 * d2) * o5 * d2 * d2 * o1;
            q2 = 1.0 + q2 + 2.0 * o2 * d2 + 3.0 * o3 * d2 * d2 + 6.0 * o4 * d2 * d2 * d2 * d2 * d2;
            d3 = (p1 - p2) / q2;
            if (Math.abs(d3) < 0.000005 || i1>30) break;
            if (Math.abs(d3)>= 0.1 * d2) d3 = Math.abs(d3) / d3 * d2 / 10;
            d2 = d2 + d3;
        } while (true);
//        GoTo 210
//300:
        z = z1 + omsm / 0.3978 * (z2 - z1);
        den = p / z / ru / t;
//        if (ivol == 2) return;      //    Výpočet pomocných proměnných (dříve)
                e1 = c5 / 0.120334 * (1.65392 - (1.65392 + 0.060167 * d1 * d1) * c1);
        e2 = o5 / 0.07508 * (2.226 - (2.226 + 0.03754 * d2 * d2) * o1);
        f1 = z1 - 1.0 - Math.log(z1) + c2 * d1 + c3 * d1 * d1 / 2;
        f1 = f1 + c4 * d1 * d1 * d1 * d1 * d1 / 5 + e1;
        f2 = z2 - 1.0 - Math.log(z2) + o2 * d2 + o3 * d2 * d2 / 2;
        f2 = f2 + o4 * d2 * d2 * d2 * d2 * d2 / 5 + e2;
        h1 = z1 - 1.0 - d1 / t1 * (0.265728 + 0.30958 / t1 + 0.090969 / t1 * t1);
        h1 = h1 - d1 * d1 * 0.0093492 / t1 + 0.124738 * 0.0001 * d1 * d1 * d1 * d1 * d1 / t1;
        h1 = (h1 + 3 * e1) * t1;
        h2 = z2 - 1 - d2 / t1 * (0.331511 + 0.05531 / t1 + 0.610464 / t1 * t1);
        h2 = h2 - 0.5 * d2 * d2 / t1 * (0.0503618 - 0.050703 / t1 * t1);
        h2 = (h2 + 0.0148067 * 0.0001 * d2 * d2 * d2 * d2 * d2 / t1 + 3 * e2) * t1;
        ss1 = h1 / t1 - f1;
        ss2 = h2 / t1 - f2;
        c6 = d1 / t1 * t1 * 2 * (0.15479 + 0.090969 / t1) - 6 * e1;
        c7 = d2 / t1 * t1 * 2 * (0.027655 + 0.610464 / t1) - 6 * e2;
        c7 = c7 - 0.050703 * d2 * d2 / t1 * t1 * t1;
        q3 = d1 * (0.118119 + 0.15479 / t1 * t1 + 0.060646 / t1 * t1 * t1);
        q3 = q3 + 2 * d1 * d1 * 0.0118372 + 5 * 0.031097 * 0.0001 * d1 * d1 * d1 * d1 * d1;
        q3 = 1 + q3 - 2 * c5 * d1 * d1 * (0.65392 + 0.060167 * d1 * d1) * c1;
        q4 = d2 * (0.202658 + 0.027655 / t1 * t1 + 0.406976 / t1 * t1 * t1);
        q4 = q4 + d2 * d2 * (0.0313385 - 2 * 0.016901 / t1 * t1 * t1);
        q4 = q4 - 2 * o5 * d2 * d2 * (1.226 + 0.03754 * d2 * d2) * o1;
        q4 = q4 + 1 + 0.48736 * 0.0001 * d2 * d2 * d2 * d2 * d2;
        c8 = c6 - 1 + q3 * q3 / q1;
        x1 = omsm / 0.3978;
        c9 = c7 - 1 + q4 * q4 / q2;
        x9 = q3 * q3 / q1 + (c6 + c7) / 0.3978 - Math.pow(q3 + q4 / 0.3978, 2) / (q1 + q2 / 0.3978);
        f = Math.exp(f1 + x1 * (f2 - f1));
        h = h1 + x1 * (h2 - h1);
        s = ss1 + x1 * (ss2 - ss1);
        c1 = c6 + x1 * (c7 - c6);
        q1 = q1 + x1 * (q2 - q1);
        q3 = q3 + x1 * (q4 - q3);
        c = c1 - 1 + q3 * q3 / q1;
        riku = -1 + z - h;
        rikf = Math.log(z * f) - z + 1;
        rikc = -2 * riku - c1;
//     Výpočet reálných termodynamických vlastností
        urea = htpid - ru * t * (1 + riku);
        hrea = htpid - ru * t * (riku + 1 - z);
        srea = stpid + ru * (Math.log(z) - rikf - riku);
        cfln = -Math.log(f);
        cvrea = cvid - ru * (2 * riku + rikc);
        cprea = cvrea + ru * q3 * q3 / q1;
        cojot = (q3 - q1) / cprea / q1 / den;
        return;
    } // konec tevrp
/**
 * Výpočet hustoty směsi reálných plynů s korekcí na neidealitu
 * @param t teplota (K)
 * @param p tlak (Pa)
 * @return  hustota (kg/m3)
 */
    public double srHustGW(double t, double p) {
        double sm;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        sm = 0;
        for (int i=0; i<nSloz; i++) {
            sm += xMol[i] * slozky.get(i).getMolHmot(); // Mvi
        }
    return  (1e+6 * den * sm);
    }
/**
 * Výpočet hustoty směsi plynů ze stavové rov. ideálního plynu
 * @param t teplota (K)
 * @param p tlak (Pa)
 * @return  hustota (kg/m3)
 */
    public double sHustGW(double t, double p) {
        return sMolHmot() * p / 8.314 / t;
    }
/**
 * Entalpie směsi reálných plynů
 * @param t abs. tepota (K)
 * @param p tlak (Pa)
 * @return entalpie (J/mol)
 */
    public double srHG(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return hrea;
    }
/**
 * Entalpie směsi reálných plynů
 * @param t abs. tepota (K)
 * @param p tlak (Pa)
 * @return entalpie (J/kg)
 */
    public double srHGW(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return hrea/sMolHmot();
    }
/**
 * Vnitřní energie směsi reálných plynů
 * @param t abs. tepota (K)
 * @param p tlak (Pa)
 * @return U (J/mol)
 */
    public double srUG(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return urea;
    }
/**
 * Vnitřní energie směsi reálných plynů
 * @param t abs. tepota (K)
 * @param p tlak (Pa)
 * @return U (J/kg)
 */
    public double srUGW(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return urea/sMolHmot();
    }
/**
 * Entropie směsi reálných plynů
 * @param t abs. tepota (K)
 * @param p tlak (Pa)
 * @return S (J/mol/K)
 */
    public double srSG(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return srea;
    }
/**
 * Entropie směsi reálných plynů
 * @param t abs. tepota (K)
 * @param p tlak (Pa)
 * @return S (J/kg/K)
 */
    public double srSGW(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return srea/sMolHmot();
    }
/**
 * Tepelná izobarická kapacita směsi reálných plynů Cp
 * @param t abs. tepota (K)
 * @param p tlak (Pa)
 * @return Cp (J/mol/K)
 */
    public double srCpG(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return cprea;
    }
/**
 * Tepelná izobarická kapacita směsi reálných plynů Cp
 * @param t abs. tepota (K)
 * @param p tlak (Pa)
 * @return Cp (J/kg/K)
 */
    public double srCpGW(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return cprea/sMolHmot();
    }
/**
 * Tepelná izochorcká kapacita směsi reálných plynů Cv
 * @param t abs. tepota (K)
 * @param p tlak (Pa)
 * @return Cv (J/mol/K)
 */
    public double srCvG(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return cvrea;
    }
/**
 * Tepelná izochorcká kapacita směsi reálných plynů Cv
 * @param t abs. tepota (K)
 * @param p tlak (Pa)
 * @return Cv (J/kg/K)
 */
    public double srCvGW(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return cvrea/sMolHmot();
    }
/**
 * Adiabatický koeficent Cp/Cv směsi reálných plynů
 * @param t abs. teplota (K)
 * @param p tlak (Pa)
 * @return kapa
 */
    public double sKapa(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return cprea/cvrea;
    }
/**
 * Joule-Thopmsnův koeficinet
 * @param t abs. teplota (K)
 * @param p tlak (Pa)
 * @return JTkoef
 */
    public double sJTcoef(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return cojot;
    }
/**
 * Logaritmus fugacity
 * @param double t
 * @param double p
 * @return
 */
    public double sLnFug(double t, double p) {
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tevrp(t, p);
        return cfln;
    }
/**
 * Tepelná vodivost reálných plynů
 * @param t  abs. teplota (K)
 * @return vodivost (W/m/K)
 */
    public double srVodG(double t, double p) {
        double rored, theta,s;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        rored=srHustGW(t,p)*vksm/sMolHmot()*1e-6;
        theta=Math.pow(tksm,(double)(1/6))*Math.sqrt(1000*sMolHmot())/ Math.pow(pksm / 0.101324,(double)(2/3));
//      pro výpočt THETA:  Pkr [atm]; M[g/mol]; Tkr[K]
        s = 0.00005733 * (Math.exp(1.155 * 2.8) + 2.016);   // max. korekce
        if (rored <= 0.5) {    //korekce jsou v mPa.s
         s = 0.0002697 * (Math.exp(0.535 * rored) - 1.0);
        }
        if (rored > 0.5 && rored <= 2) {
         s = 0.0002524 * (Math.exp(0.67 * rored) - 1.0);
        }
        if (rored > 2 && rored < 2.8) {
         s = 0.00005733 * (Math.exp(1.155 * rored) + 2.016);
        }
        s = sVodG(t) + s / theta / zksm * zksm * zksm * zksm * zksm;
        return s;
    }
/**
 * Entalpie kapalin (molární)
 * Vztažena k Tnbv
 * @param t  abs. teplota (K)
 * @return entalpie (J/mol)
 */
    public double sHL(double t) {
        int i,typ;
        double s = 0, t0;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        for (i = 0; i < nSloz; i++) {
            if (xMol[i] > 0.000001) {
                t0 = slozky.get(i).gettNbv();     // Tnbv
                typ = slozky.get(i).vlast[6].vztah;
                if (typ > 0 && typ <= 5 ) {       // typ vztahu polynom
                    for (int j = 1; j<= (typ + 1); j++) {
                      s +=  xMol[i] * slozky.get(i).vlast[6].a[j-1] / j * (Math.pow(t,j) - Math.pow(t0, j));
                    }
                }
            }
        }
        return s;
    }
/**
 * Entalpie kapalin
 * Vztažena k Tnbv
 * @param t  abs. teplota (K)
 * @return entalpie (J/kg)
 */
    public double sHLW(double t) {
        return sHL(t)/sMolHmot();
    }
/**
 * Entropie kapalin (molární)
 * Vztažena k Tnbv
 * @param t  abs. teplota (K)
 * @return entropie (J/mol/K)
 */
    public double sSL(double t) {
        int i,typ;
        double s = 0, t0;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        for (i = 0; i < nSloz; i++) {
            if (xMol[i] > 0.000001) {
                t0 = slozky.get(i).gettNbv();     // Tnbv
                typ = slozky.get(i).vlast[6].vztah;
                if (typ > 0 && typ <= 5 ) {       // typ vztahu polynom
                    s += xMol[i]*slozky.get(i).vlast[6].a[0]*Math.log(t/t0);
                    for (int j = 1; j<(typ + 1); j++) {
                      s +=  xMol[i] * slozky.get(i).vlast[6].a[j] / j * (Math.pow(t,j) - Math.pow(t0, j));
                    }
                }
            }
        }
        return s;
    }
/**
 * Entropie kapalin
 * Vztažena k Tnbv
 * @param t  abs. teplota (K)
 * @return entropie (J/kg/K)
 */
    public double sSLW(double t) {
        return sSL(t)/sMolHmot();
    }

/**
 * Mol. vnitřní energie kapalin
 * U = H - R*(T - Ts)
 * Vztažena k Tnbv
 * @param t  abs. teplota (K)
 * @return UL (J/mol)
 */
    public double sUL(double t) {
        int i,typ;
        double s = 0, t0 = 0;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        for (i = 0; i < nSloz; i++) {
            if (xMol[i] > 0.000001) {
                t0 = slozky.get(i).gettNbv();     // Tnbv
                typ = slozky.get(i).vlast[6].vztah;
                if (typ > 0 && typ <= 5 ) {       // typ vztahu polynom
                    for (int j = 1; j<= (typ + 1); j++) {
                      s +=  xMol[i] * slozky.get(i).vlast[6].a[j-1] / j * (Math.pow(t,j) - Math.pow(t0, j));
                    }
                }
            }
        }
        s = s - 8.314 *(t - t0);
        return s;
    }
/**
 * Vnitřní energie kapalin
 * U = H - R/Mv*(T - Ts)
 * Vztažena k Tnbv
 * @param t  abs. teplota (K)
 * @return ULW (J/kg)
 */
    public double sULW(double t) {
        return sUL(t)/sMolHmot();
    }

/**
 * Objemová roztažnost kapalin 1/V(dV/dT) neboli: - 1/ro(dro/dT); (1/K)
 * @param t abs. teplota (K)
 * @return
 */
    public double sObjRozL(double t) {
        return (sHustL(t+3)-sHustL(t))/sHustL(t)/3.0;
    }
/**
 * Prandtlovo číslo směsi kapalin
 * @param t abs. teplota (K)
 * @return Pr (-)
 */
    public double sPrL(double t) {
        return sCpLW(t)*sViskL(t)/sVodL(t);
    }

/**
 * Prandtlovo číslo směsi plynů
 * @param t abs. teplota (K)
 * @return Pr (-)
 */
    public double sPrG(double t, double p) {
        return srCpGW(t,p)*srViskG(t,p)/srVodG(t,p);
    }
/**
 * Počítá Prandtlovo číslo se stanovením typu fáze<br>
 * Pro t < Tbodu varu vrací Prandtlovo číslo kapaliny<br> 
 * Pro t > Trosbodu varu vrací Prandtlovo číslo páry<br>
 * Při koexistenci obou fází vrací interpolovanou hodnotu 
 * @param t
 * @param p
 * @return hustota 
 */    
    public double sPr(double t, double p) {
        double pr = -9999, tbv, trb;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        tbv = sTbv(p); trb = sTrb(p);
        if (t < tbv) pr = sPrL(t);
        if (t > trb) pr = sPrG(t, p);
        if (t > tbv && t < trb) {       // dvě fáze, interpolace
            double prbv = sPrL(tbv);
            pr = prbv + (sPrG(trb, p) - prbv)/(trb - tbv)*(t - tbv);
        }        
        return pr;
    }

  /**
 * Viskozita reálných plynů
 * @param t  abs. teplota (K)
 * @return visk (Pa.s)
 */
    public double srViskG(double t, double p) {
        double rored, xKsi,s;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        rored = srHustGW(t,p)*vksm/sMolHmot()*1e-6;
        xKsi = Math.pow(tksm,(double)1/6)*Math.sqrt(1000*sMolHmot())/ Math.pow(pksm / 0.101324,(double)2/3);
//      pro výpočt Xksi:  Pkr [atm]; M[g/mol]; Tkr[K]    korekce je v mPa.s
        s = sViskG(t) + 0.000108 *( Math.exp(1.439*rored) -
                Math.exp(-1.111*Math.pow(rored, 1.858)))/xKsi/1000;
        return s;
    }
/**
 * Pseudokritická teplota směsi (K)
 * @return sTkr (K)
 */
    public double sTksm() {
        if (tksm<1) tevrp(320,101320);  // pomocné volání
        return tksm;
    }
/**
 * Pseudokritický tlak směsi (Pa)
 * @return sPkr (Pa)
 */
    public double sPksm() {
        if (pksm<1) tevrp(320,101320);  // pomocné volání
        return pksm*1e6;
    }

/**
 * Pseudokritický objem směsi (ml/mol)
 * @return sPkr (ml/mol)
 */
    public double sVksm() {
        if (vksm<1) tevrp(320,101320);  // pomocné volání
        return vksm/1e6;
    }

 /**
 * Výpočet rosného bodu směsi (K)
 * @param p tlak (Pa)
 * @return TRB (K)
 */
    public double sTrb(double p)  {
        int i, iter, itmax=90;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        double trb=0, tt, ttp,fun, fun0=0, funp, fder=0,ypom, dx=0.5;
        double[] p0 = new double[nSloz];
        tt=300;     // poč. odhad teploty rosného bodu (K)
        tt = preBV(p) + 3;
//        System.out.println("TRB odhad"+tt );
        iter=0;
        do {
            fun=-1;
            fder=0;
            for (i=0; i<nSloz; i++) {
//                if (tt > slozky.get(i).gettKr()) tt = 0.95*slozky.get(i).gettKr();
            }
            for (i=0; i<nSloz; i++) {
                p0[i] = slozky.get(i).tenze(tt);
                if (iter > 36) dx = 0.05;
                if (tt >= slozky.get(i).gettKr()) {
                    ypom=0.0;   // zlobilo
                    ypom = xMol[i]*p/p0[i];
                }
                else {
                    ypom = xMol[i]*p/p0[i];
                }
                fder += -ypom/p0[i]*(slozky.get(i).tenze(tt+dx)
                        -slozky.get(i).tenze(tt-dx))/2/dx;
                fun += ypom;
            }
            if (Math.abs(fun+1) < 1e-6) {
//                tt = 0.7*tt;
                continue;
            }
            trb = tt - fun/(fder + 1e-9 )*0.3;     // Newton
            iter++;
//            System.out.println("trb: "+iter+" " +trb+" tt: "+tt+" fder= "+fder+" fun= "+fun);
            // sem test divergence
            if (iter> itmax ) {
                if ( !neMistniHlasky) {
                    JOptionPane.showMessageDialog(null, "Překročen max. počet iterací ros. bodu ("
                       + itmax  + ")\nTeplota(K)= "+ String.format("%5.1f",tt) + "  tlak(Pa)= " + String.format("%6.0f",p), "FYDAT - TRB", 1);
                }
                    break;
            }
            if (trb/tt < 0.5) {
                tt = 0.5*tt;
                continue;
            }
            if (trb/tt > 1.5) {
                tt = 1.5*tt;
                continue;
            }
            if (Math.abs(trb-tt) < 0.02) {
                return trb;
            }
            if (fun*fun0 <0 && iter > 2) {
                tt = trb;
                fun0 = fun;
            }
            if (iter < 40) {
                tt = trb;
                fun0 = fun;
            }
            else {
                tt = (tt + trb)/2.0;  // opatření proti zacyklování
            }
        } while (true);
        // půlení intervalu
        iter = 0;
        while (Math.abs(tt-trb)>0.005 && iter < 15){
           ttp = (tt + trb)/2.0;
           funp = -1;
            for (i=0; i<nSloz; i++) {
                p0[i] = slozky.get(i).tenze(tt);
                if (iter > 36) dx = 0.05;
                if (tt >= slozky.get(i).gettKr()) {
                    ypom=0.0;
                }
                else {
                    ypom = xMol[i]*p/p0[i];
                }
                funp += ypom;
            }
//            System.out.println("P tt, trp: "+ tt + " " + trb + " fun0, fun: "+fun0 +" " +fun);
//            System.out.println("PUL ttp "+ttp+" funp "+funp);
           if (funp*fun0<0) {
               trb = ttp;
               fun = funp;
           }
           else {
               tt = ttp;
               fun0 = funp;
           }
        }
        return trb;
    }
/**
 * Výpočet bodu varu směsi (K)
 * @param p tlak (Pa)
 * @return TBV (K)
 */
    public double sTbv(double p)  {
        int i, iter, itmax = 80;
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        double tbv=0, tt, ttp,fun, fun0=0, funp, fder=0,ypom, dx=0.5;
        double[] p0 = new double[nSloz];
        tt=300;     // poč. odhad teploty bodu varu (K)
//        ypom=0;     // -------- alt. odhad ------------------
//        for (i=0; i<nSloz; i++) {
//            ypom += xMol[i]*slozky.get(i).gettNbv();
//        }
//        tt=ypom;    // odhad jako vážený Tnbv ---------------
        tt = preBV(p);
//        System.out.println("TBV odhad"+tt );
        iter=0;
        do {
            fun=-1;
            fder=0;
            for (i=0; i<nSloz; i++) {
//                if (tt > slozky.get(i).gettKr()) tt = 0.95*slozky.get(i).gettKr();
            }
            for (i=0; i<nSloz; i++) {
                p0[i] = slozky.get(i).tenze(tt);
                if (tt >= slozky.get(i).gettKr()) {
                    ypom=xMol[i]*p0[i]/p;       // dříve =0.0;
                }
                else {
                    ypom = xMol[i]*p0[i]/p;
                }
                fder += ypom/p0[i]*(slozky.get(i).tenze(tt+dx)
                        -slozky.get(i).tenze(tt-dx))/2/dx;
                fun += ypom;
            }
            tbv = tt - fun/(fder + 1e-9)*0.5;     // Newton
            iter++;
//            System.out.println("tbv: "+iter+" " +tbv+" tt: "+tt+" fder= "+fder+" fun= "+fun);
            // sem test divergence
            if (iter> itmax) {
                if ( !neMistniHlasky) {
                  JOptionPane.showMessageDialog(null, "Překročen max. počet iterací bodu varu ("
                    + itmax  + ")\nTeplota(K)= "+ String.format("%5.1f",tt) + "  tlak(Pa)= " + String.format("%6.0f",p), "FYDAT - TBV", 1);
                }
                    break;
            }
            if (tbv/tt < 0.5) {
                tt = 0.5*tt;
                continue;
            }
            if (tbv/tt > 1.5) {
                tt = 1.5*tt;
                continue;
            }
            if (Math.abs(tbv-tt) < 0.02) {
                return tbv;
            }
            if (fun*fun0 <0 && iter > 2) {
                tt = tbv;
                fun0 = fun;
            }
            tt = tbv;
        } while (true);
        // půlení intervalu
        iter = 0;
        while (Math.abs(tt-tbv)>0.005 && iter < 15){
           ttp = (tt + tbv)/2.0;
           funp = -1;
            for (i=0; i<nSloz; i++) {
                p0[i] = slozky.get(i).tenze(tt);
                if (iter > 36) dx = 0.05;
                if (tt >= slozky.get(i).gettKr()) {
                    ypom=0.0;
                }
                else {
                    ypom = xMol[i]*p/p0[i];
                }
                funp += ypom;
            }
//            System.out.println("P tt, trp: "+ tt + " " + tbv + " fun0, fun: "+fun0 +" " +fun);
//            System.out.println("PUL ttp "+ttp+" funp "+funp);
           if (funp*fun0<0) {
               tbv = ttp;
               fun = funp;
           }
           else {
               tt = ttp;
               fun0 = funp;
           }
        }
        return tbv;
    }
    /**
     * Stanovení odhadu pro bod varu a rosný bod lin. interpolací
     * @param p tlak (Pa)
     * @return odhad t (K)
     */
    private double preBV(double p) {
        int i;
        double ypom, t = 0, t0, t1, y0, y1;
        if(xMol == null) chyba = -1;    // nezadáno složení
        if(chyba < 0) return -9999;     // chyby v zadání směsi
        ypom=0;     // -------- alt. odhad ------------------
        for (i=0; i<nSloz; i++) {
            ypom += xMol[i]*slozky.get(i).gettNbv();
        }
        t0 = ypom;    // odhad jako vážený Tnbv ---------------
        t1 = t0 + 10;
        for (i=0; i<3; i++) {
            y0 = Math.log(sTenze(t0)) - Math.log(p);
            y1 = Math.log(sTenze(t1)) - Math.log(p);
            t = (y1*t0 - y0*t1)/(y1 -y0);
            if (y1 < y0) {
                t0 = t;
            }
            else {
                t1 = t;
            }
        }
        return t;
    }
/**
 * Vytvoří kopii směsi
 * se stejnými složkami a složením
 * @param sm kopírovaná směs
 * @return nově vytvořená směs
 * @throws IOException (pokud by nenačetla data o látkách)
 */    
    public Smes kopirujSmes(Smes sm) throws IOException,JDOMException {       
        int nSl = sm.getnSloz();
        Smes sms = new Smes(nSl,sm.getCislaSlozek());
        sms.zadatSloz(false, sm.getxMol());
        sms.nazev = sm.getNazev();
        return sms;
    }
/**
 * Seřadí látky ve směsi podle klesající těkavosti
 */    
    public void seradDleTekavosti() {
       int i;
       boolean jesteNe;
       if(chyba < 0) return;     // chyby v zadání směsi
       double[] tek = new double[nSloz];
       for(i = 0; i <nSloz;i++) {
           tek[i] = slozky.get(i).gettNbv();    // norm. bod varu (K)          
       }
       // setřídění čísel látek a složení dle těkavosti
       do {
        jesteNe= false;   
        for(i = 0; i< nSloz - 1; i++) {
         if(tek[i] > tek[i+1]) {
            int ipom = iCisla[i];   // čísla složek
            double pom = tek[i];
            double xpom = xMol[i];
            tek[i] = tek[i+1];
            tek[i+1] = pom;
            xMol[i] = xMol[i+1];
            xMol[i+1] = xpom;
            iCisla[i] = iCisla[i+1];
            iCisla[i+1] = ipom;
            jesteNe = true;
         }           
        }
       } while(jesteNe);
        try {
            // redefinice směsi
            slozky.removeAll(slozky);
            for (i = 0; i < nSloz; i++) { 
                slozky.add(Latka.getLatka(iCisla[i]));  // znovu načte látky
            }            
            zadatSloz(false, xMol); // promítne složení
        } catch (IOException ex) {
            Logger.getLogger(Smes.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
